;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	File PBIOS.INC	Created 4-DEC-2000	Richard B. Johnson
;	This was broken away from PBIOS.ASM on this date to make things
;	a bit easier to manipulate.
;
;	Various constants and macros are included within this file.
;
.486p
;TEST_FLOPPY = 1
;TEST_PCI = 1
NEW_SUPERIO	EQU	1		; New Super I/O Chip
;OLD_RAMDISK	EQU	1		; Old Ramdisk board
;INSTALLED_MEM	EQU	64		; Megabytes of installed memory
INSTALLED_MEM	EQU	32		; Megabytes of installed memory
FAST_A20 = 1				; Use fast A<20> code
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This slows down port I/O by writing to port 0x80 which is on
;	the motherboard so it usually has the correct wait-states.
;
SLOW_IO	MACRO
	OUT	MFG_PORT,AL
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This converts a BCD value in AL to something that can be
;	read by a human.
;
CVT_ASC	MACRO
	MOV	AH,AL				; Duplicate low byte
	SHR	AH,4				; Move into location
	AND	AL,00001111B			; Mask high nibble
	ADD	AX,3030H			; ASCII bias
	XCHG	AH,AL				; Swap bytes
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This makes a linear address out of a DWORD pointer.
;
MAKE_REAL MACRO BASE, POINTER
	MOV	AX,BASE
	XOR	DH,DH			; Clear high byte
	MOV	DL,AH			; DL = BASE * 2^8
	SHL	AX,4			; AX = BASE * 2^4
	SHR	DX,4			; DX = BASE * 2^4
IFNB	<POINTER>
	ADD	AX,POINTER		; Sum the offset
	ADC	DX,0			; Take care of CY
ENDIF
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	These two macros handle local timer manipulation. The interval
;	counter is a down-counter so timer-wrap is never a problem
;	Note: It is presumed that the data segment has been set!
;
SET_TIMER MACRO TIME
	MOV	WORD PTR DS:[INTV_TIM],TIME
ENDM
;
WAIT_TIMER MACRO WHERE
	CMP	WORD PTR DS:[INTV_TIM],0
	JNZ	SHORT WHERE
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This shows the current boot-up progress by writing an 8-bit
;	value to a port. Register AX and DX are distroyed.
;
SHOW_PROGRESS MACRO N
	MOV	AL,N
	MOV	DX,DISPL_PORT
	OUT	DX,AL
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This reads from PCI configuration space.
;
READ_PCI MACRO DEV, REG
	XOR	EAX,EAX			; Clear whole DWORD
	MOV	AX,DEV			; Device into low WORD
IF	(REG NE 0)
	SHL	EAX,9			; Start shift towards 11 bits
	OR	AX,REG			; Set the DWORD register number
	SHL	EAX,2			; Finish the shift. Bits <0:1> clear
ELSE
	SHL	EAX,11			; Shift all 11 bits
ENDIF
	OR	EAX,80000001H		; Set access bit / mode 1
	MOV	DX,PCI_INDEX		; Index register
	OUT	DX,EAX			; Set index
	SLOW_IO				; Let it set up
	MOV	DX,PCI_DATA		; Set data register
	IN	EAX,DX			; Get PCI data
	SLOW_IO
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes to PCI configuration space.
;
WRITE_PCI MACRO DEV, REG, VAL
	MOV	EBX,VAL			; Value to be written
	XOR	EAX,EAX			; Clear whole DWORD
	MOV	AX,DEV			; Device into low WORD
IF	(REG NE 0)
	SHL	EAX,9			; Start shift towards 11 bits
	OR	AX,REG			; Set the DWORD register number
	SHL	EAX,2			; Finish the shift. Bits <0:1> clear
ELSE
	SHL	EAX,11			; Shift all 11 bits
ENDIF
	OR	EAX,80000001H		; Set access bit / mode 1
	MOV	DX,PCI_INDEX		; Index register
	OUT	DX,EAX			; Set index
	SLOW_IO				; Let it set up
	MOV	DX,PCI_DATA		; Set data register
	MOV	EAX,EBX			; Value to be written
	OUT	DX,EAX			; Write the data
	SLOW_IO
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This MACRO leaves the configuration registers parked on the
;	PCI bridge.
;
PARK_PCI MACRO
	MOV	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Index register
	OUT	DX,EAX			; Set index
	SLOW_IO				; Let it set up
	MOV	DX,PCI_DATA		; Set data register
	IN	EAX,DX			; Get data, throw away
	SLOW_IO
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	These two MACROS enable the A<20> bit. Some machines have the
;	so-called 'fast' enable directly off a system-control port and
;	others use the keyboard controller.
;
IFDEF	FAST_A20
SET_A20_ON	MACRO
	MOV	AL,00000010B		; Enable A<20> bit
	OUT	SYSCTLA,AL		; System control port A
ENDM
SET_A20_OFF	MACRO
	XOR	AL,AL			; Get a zero
	OUT	SYSCTLA,AL		; System control port A
ENDM
ELSE
SET_A20_ON	MACRO
	LOCAL	A204, A205, A206
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This turns on the A<20> address bit, necessary to access RAM
;	above 1 megabyte. Register AX is destroyed.
;
	PUSH	CX
	XOR	CX,CX
A204:	IN	AL,STATUS_PORT
	SLOW_IO
	TEST	AL,02H			; Is last command complete?
	LOOPNZ	A204			; No, wait
	MOV	AL,0D1H			; Yes, tell it we have a command
	OUT	STATUS_PORT,AL
	SLOW_IO
	XOR	CX,CX
A205:	IN	AL,STATUS_PORT
	SLOW_IO
	TEST	AL,02H			; Command complete?
	LOOPNZ	A205			; Wait until it's accepted
	MOV	AL,0DFH			; Enable bit 20
	OUT	PORT_A,AL
	SLOW_IO
	XOR	CX,CX
A206:	IN	AL,STATUS_PORT
	SLOW_IO
	TEST	AL,02H			; Command complete?
	LOOPNZ	A206			; Wait until it's complete
	POP	CX
ENDM
SET_A20_OFF	MACRO
	LOCAL	A201, A202, A203
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This turns off the A<20> address bit, necessary to preserve
;	address-wrap through 1 megabyte. AX register is destroyed.
;
	PUSH	CX
	XOR	CX,CX			; Loop count
A201:	IN	AL,STATUS_PORT		; Get status
	SLOW_IO
	TEST	AL,02H			; Command command complete?
	LOOPNZ	A201			; Not yet
	MOV	AL,0D1H
	OUT	STATUS_PORT,AL		; Set command
	SLOW_IO
	XOR	CX,CX			; Loop count
A202:	IN	AL,STATUS_PORT		; Get status
	SLOW_IO
	TEST	AL,02H			; Command complete?
	LOOPNZ	A202			; Wait until it's accepted
	MOV	AL,0DDH			; Disable bit 20
	OUT	PORT_A,AL		; Out base port
	SLOW_IO
	XOR	CX,CX			; Loop count
A203:	IN	AL,STATUS_PORT		; Get status
	SLOW_IO
	TEST	AL,02H			; Command complete?
	LOOPNZ	A203			; Not yet
	POP	CX
ENDM
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
; IRQ (number)	Interrupt vector
;
IRQ0		EQU	08H	; Interrupt vectors
IRQ1		EQU	09H
IRQ2		EQU	0AH
IRQ3		EQU	0BH
IRQ4		EQU	0CH
IRQ5		EQU	0DH
IRQ6		EQU	0EH
IRQ7		EQU	0FH
IRQ8		EQU	70H
IRQ9		EQU	71H
IRQ10		EQU	72H
IRQ11		EQU	73H
IRQ12		EQU	74H
IRQ13		EQU	75H
IRQ14		EQU	76H
IRQ15		EQU	77H
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Generic 8250 UART equates.
;
BAS	EQU	0			; Base (data) port
IER	EQU	1			; Interrupt enable register
IIR	EQU	2			; Interrupt ident register
LCR	EQU	3			; Line control register
MCR	EQU	4			; Modem control register
LSR	EQU	5			; Line status register
MSR	EQU	6			; Modem status register
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Keyboard status in 40:18 (KEY_XSHIFT)
;
LEFT_CTL	EQU	00000001B	; Left Ctrl active
LEFT_ALT	EQU	00000010B	; Left Alt active
RIGHT_CTL	EQU	00000100B	; Right Ctrl active
RIGHT_ALT	EQU	00001000B	; Right Alt active
SCRL_LOCK	EQU	00010000B	; Scroll lock active
NUM_LOCK	EQU	00100000B	; Num lock active
CAP_LOCK	EQU	01000000B	; Caps lock active
SYS_REQ		EQU	10000000B	; Sys request pressed
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Keyboard status in 40:17 (KEY_SHIFT)
;
RIGHT_SHIFT	EQU	00000001B	; Right shift active
LEFT_SHIFT	EQU	00000010B	; Left shift active
ANY_CTL		EQU	00000100B	; Left or right control pressed
ANY_ALT		EQU	00001000B	; Left or right alt pressed
COPY_SCRL	EQU	SCRL_LOCK	; Scroll lock active
COPY_NUM	EQU	NUM_LOCK	; Num lock active
COPY_CAP	EQU	CAP_LOCK	; Caps lock active
INS_MODE	EQU	10000000B	; Insert mode
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Control function scan-codes
;
SC_LF_SHF	EQU	2AH		; Left shift
SC_RT_SHF	EQU	36H		; Right Shift
SC_LF_CTL	EQU	1DH		; Left Ctrl
SC_RT_CTL	EQU	1DH		; Right Ctrl
SC_LF_ALT	EQU	38H		; Left Alt
SC_RT_ALT	EQU	38H		; Right Alt
SC_INS		EQU	52H		; Insert
SC_DEL		EQU	53H		; Delete code
SC_NUM_LCK	EQU	45H		; Num lock.
SC_CAP_LCK	EQU	3AH		; Caps-lock
SC_SCR_LCK	EQU	46H		; Scroll-lock
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Keyboard parameters
;
PORT_A		EQU	60H		; Keyboard
PORT_B		EQU	61H		; Keyboard
STATUS_PORT	EQU	64H		; Keyboard status
SYSCTLA		EQU	92H		; System control port A
LED_CMD		EQU	0EDH		; Turn on LEDS
KBD_ENABLE	EQU	0F4H		; Keyboard enable
KBD_SPEED	EQU	0F3H		; Set keyboard repeat rate
KBD_REPEAT	EQU	0		; Keyboard repeat rate
KBD_RSET	EQU	0FFH		; Keyboard reset
INP_FULL	EQU	00000010B	; Controller input full
OUT_FULL	EQU	00000001B	; Controller output full
KBD_CAPS_LED	EQU	00000100B	; Caps-lock LED
KBD_NUML_LED	EQU	00000010B	; Num-lock LED
KBD_SCRL_LED	EQU	00000001B	; Scroll-lock LED
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for the Disk I/O vector (interrupt 13H).
;	These are the offsets relative to BP. They will be negative
;	offsets when accessed. I have deliberately allocated one
;	extra word on the stack. This is not a bug.
;
AX_SAV	EQU	2			; Function code
BX_SAV	EQU	4			; Save / modify BX
CX_SAV	EQU	6			; Save / modify CX
DX_SAV	EQU	8			; Save / modify DX
SI_SAV	EQU	10			; Save / modify SI
DI_SAV	EQU	12			; Save / modify DI
DS_SAV	EQU	14			; Save / modify DS
ES_SAV	EQU	16			; Save / modify ES
DMA_OFF	EQU	18			; Offset for data transfer
DMA_PAG	EQU	20			; Page number for data transfer
TMP_SAV	EQU	22			; Temporary storage
XBYTES	EQU	24			; Transfer bytes
D_ALLOC	EQU	28			; Total local storage
;
;	Define some long variable names.
;
RETURN_CODE		EQU	WORD PTR [BP-AX_SAV]
SECTORS_TO_TRANSFER	EQU	BYTE PTR [BP-AX_SAV]
DTA_OFFSET		EQU	WORD PTR [BP-BX_SAV]
STARTING_SECTOR		EQU	BYTE PTR [BP-CX_SAV]
TRACK_NUMBER		EQU	BYTE PTR [BP-CX_SAV+1]
SECTOR_NUMBER		EQU	BYTE PTR [BP-CX_SAV]
TRACK_SECTOR		EQU	WORD PTR [BP-CX_SAV]
DRIVE_NUMBER		EQU	BYTE PTR [BP-DX_SAV]
HEAD_NUMBER		EQU	BYTE PTR [BP-DX_SAV+1]
HEAD_DRIVE		EQU	WORD PTR [BP-DX_SAV]
DTA_SEGMENT		EQU	WORD PTR [BP-ES_SAV]
SECTORS_TRACK		EQU	BYTE PTR [BP-TMP_SAV]
NEXT_COMMAND		EQU	BYTE PTR [BP-TMP_SAV]
BYTES_TO_TRANSFER	EQU	WORD PTR [BP-XBYTES]
DMA_PAGE		EQU	WORD PTR [BP-DMA_PAG]
DMA_OFFSET		EQU	WORD PTR [BP-DMA_OFF]
RET_OFFSET		EQU	WORD PTR [BP-DI_SAV]
RET_SEGMENT		EQU	WORD PTR [BP-ES_SAV]
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Floppy disk control equates.
;
FDC_MOTOR	EQU	3F2H		; Motor control port
FDC_STATUS	EQU	3F4H		; Floppy disk controller status
FDC_DATA	EQU	3F5H		; Floppy disk controller data port
FDC_RATE	EQU	3F7H		; Floppy data rate port
FDC_RATE_PAR	EQU	00H		; For 500 kb/s drives
FDC_ENAB	EQU	00001100B	; Default FDC enables
FDC_MOTON	EQU	00010000B	; Motor ON bit
FDC_INVAL	EQU	0100H		; Invalid function code
FDC_ADDRM	EQU	0200H		; Address mark not found
FDC_WPROT	EQU	0300H		; Write protected
FDC_SECNF	EQU	0400H		; Sector not found
FDC_CHANG	EQU	0600H		; Diskette changed
FDC_DMAOV	EQU	0800H		; DMA overrun
FDC_DMABY	EQU	0900H		; DMA boundary
FDC_MEDIA	EQU	0C00H		; Bad media type
FDC_NOCRC	EQU	1000H		; Bad CRC
FDC_CONTF	EQU	2000H		; Controller failed
FDC_SEEKF	EQU	4000H		; Seek failed
FDC_TIMEO	EQU	8000H		; Timeout
FDC_CMD_CONFIG	EQU	00010011B	; NEC configure command
FDC_CMD_READ	EQU	11100110B	; NEC Read command
FDC_CMD_WRITE	EQU	11100101B	; NEC Write command
FDC_CMD_RECAL	EQU	00000111B	; NEC recalibrate command
FDC_CMD_SENSE	EQU	00001000B	; NEC Sense interrupt status
FDC_CMD_SPECIFY	EQU	00000011B	; NEC Specify
FDC_CMD_SEEK	EQU	00001111B	; Seek command
FDC_CMD_FORMAT	EQU	01001101B	; Format write
FDC_STA_EOC	EQU	10000000B	; End of cylinder
FDC_STA_CRC	EQU	00100000B	; CRC error, either ID or data
FDC_STA_ORE	EQU	00010000B	; Data overrun error
FDC_STA_RNF	EQU	00000100B	; No data (record not found)
FDC_STA_WPT	EQU	00000010B	; Write protect error
FDC_STA_ADR	EQU	00000001B	; Address mark not found
FDC_TIMEOUT	EQU	10H		; Tick intervals for timeout
NEC_TIMEOUT	EQU	10H		; Tick intervals for NEC chip timeout
FDC_DMA_READ	EQU	46H		; Get data from FCD to memory
FDC_DMA_WRITE	EQU	4AH		; Put data from memory to FDC
FDC_DMA_VERIFY	EQU	42H		; Verify for DMA
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	PCI BIOS-16 functions.
;
SUCCESSFUL		EQU	00h
PCI_FUNCTION_ID 	EQU	0B1H
FUNC_NOT_SUPPORTED	EQU	81H
BAD_DEVICE_ID		EQU	82H
BAD_VENDOR_ID		EQU	83H
BAD_BUS_ID		EQU	84H
BAD_SLOT_ID		EQU	85H
DEVICE_NOT_FOUND	EQU	86H
BAD_REGISTER_NUMBER	EQU	87H
NO_INTERRUPT_ASSIGNED	EQU	88H
SET_FAILED		EQU	88H
BUFFER_TO_SMALL		EQU	89H
PCI_INDEX		EQU	0CF8H	; Bridge index port
PCI_DATA		EQU	0CFCH	; Bridge data port
PCI_OBJ_ALLOC		EQU	2000H	; Temporary storage for PCI allocation
PCI_MEM			EQU	0	; Offset for PCI memory
PCI_PRT			EQU	8	; Offset for PCI ports
PCI_OFF			EQU	05000H	; Lowest PCI port
PCI_CMD_STA		EQU	01H	; Command / status register
PCI_CLASS		EQU	02H	; PCI class code
PCI_LAT_CACHE		EQU	03H	; Latency / cache register
PCI_BASE0		EQU	04H	; First base address register
PCI_CARDBUS		EQU	0AH	; Cardbus CIS pointer
PCI_ROMADDR		EQU	0CH	; Expansion ROM address
PCI_INTER		EQU	0FH	; Interrupt pin/IRQ
PCI_AMD			EQU	10H	; AMD PCI lock timer
PCI_MAX			EQU	20H	; Maximum PCI slots
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Other equates.
;
BIOS_SEG	EQU	40H		; BIOS data segment
TIMER_MAX	EQU	18H		; Max high value for day wrap
TIMER_DAT	EQU	0B0H		; Low timer tick at new day
TIMER		EQU	40H		; Base port for timer
HW_TIMER	EQU	40H		; Hardware timer count
MFG_PORT	EQU	80H		; Used to slow down I/O
DMA		EQU	00H		; DMA control channel port
DMA2_ADDR	EQU	04H		; Memory address register
DMA2_COUNT	EQU	05H		; Transfer byte count
DMA2_MASK	EQU	0AH		; DMA mask register
DMA2_MODE	EQU	0BH		; DMA mode register
DMA2_CLEAR	EQU	0CH		; DMA clear byte pointer
DMA2_PAGE	EQU	81H		; DMA channel 2 page register
DMA_MASTER	EQU	0C0H		; DMA master control port
STACK_SEG	EQU	BIOS_SEG	; Stack segment during POST
BIOS_STACK	EQU	4000H		; Where to put the stack during POST
RND_SEG		EQU	STACK_SEG + (BIOS_STACK / 16) + 1
MAGIC		EQU	586CH		; Randomizer
NOS_EOI		EQU	20H		; Non-Specific end-of-interrupt
ACK_CTL0	EQU	20H		; ACK for first controller
ACK_CTL1	EQU	0A0H		; ACK for second controller
MSK_CTL0	EQU	21H		; Mask for first controller
MSK_CTL1	EQU	0A1H		; Mask for second controller
CMOS_INDEX	EQU	70H		; Base port for CMOS clock
CMOS_DATA	EQU	71H		; CMOS data port
CMOS_VALID	EQU	0EH		; Offset to 'valid' byte.
CMOS_BOOTF	EQU	1FH		; Offset for boot flag
COPROCESSOR	EQU	0F0H		; Coprocessor control port
VGA_BIOS	EQU	0C000H		; VGA BIOS location
DSK_BIOS	EQU	0D000H		; SCSI Disk BIOS
SCR_SEGMENT	EQU	0B800H		; Screen regen buffer
SCR_REGEN_LEN	EQU	1000H		; Length of screen to test
SCR_BASE	EQU	0A000H		; First segment owned by screen
BIOS_SCAN_START	EQU	0D000H		; Where to start BIOS module scan
BIOS_SCAN_END	EQU	0F000H		; Where to end BIOS module scan
BIOS_SCAN_INCR	EQU	500H		; BIOS module scan increment
MOTOR_TIME	EQU	20H		; Motor timeout
SETUP_BUF	EQU	5000H		; Buffer segment for setup
SETUP_BUF_LEN	EQU	10H		; Length of above buffer
DUMB_TERM	EQU	14		; Video dumb terminal mode
CR		EQU	0DH		; ASCII
LF		EQU	0AH		; ASCII
BS		EQU	08H		; ASCII
ESC_KEY		EQU	1BH		; ASCII
TAB_KEY		EQU	09H		; ASCII
COLS		EQU	80		; Columns on the screen
PUT_TEXT	EQU	(24 * 160) + 52	; Offset for 32-bit screen print
MEM32_LOC	EQU	((BIOS_SEG SHL 4) + PRINT_3)
STACK32_LEN	EQU	1024		; When in 32-bit mode
CR0_PG		EQU	80000000H	; Paging bit
CR0_CD		EQU	40000000H	; Cache disable
CR0_NW		EQU	20000000H	; Not write-through
DISPL_PORT	EQU	680H		; Elan Display port
CKSUM		EQU	0FFEFH		; Offset of the checksum
TEST_SEG	EQU	9000H		; MS-DOS testing
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Software interrupts and vectors.
;
SCREEN		EQU	10H		; Screen BIOS
EQUIP		EQU	11H		; Equipment list
MEMSIZE		EQU	12H		; Memory size
DISKS		EQU	13H		; Hard and floppy disks
SERIAL		EQU	14H		; RS-232C serial communications
TOOLS		EQU	15H		; System services (was cassette)
KBD_INP		EQU	16H		; Keyboard input
PRINTER		EQU	17H		; Printer output
BOOT		EQU	19H		; BIOS boot vector
TOD		EQU	1AH		; Time of day services
USER_TICK	EQU	1CH		; User timer-tick
FLOPPY_TAB	EQU	1EH		; Floppy table
KBD_BZY		EQU	28H		; Keyboard busy loop
FLOPPY		EQU	40H		; Floppy diskettes
PRIVATE		EQU	80H		; Our private interface to 32 bits
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function codes for our private 32-bit code functions.
;	The function code is to be put into the AH register and
;	our private interrupt called.
;
EXT_RAM_SIZE	EQU	0		; Return extended RAM size
CPY_MMCR_REG	EQU	1		; Copy PAR registers
CPY_ROM_BIOS	EQU	2		; Copy ROM bios contents
CHK_ROM_BIOS	EQU	3		; Check for a ROM bios
;
;	Offsets for CPY_ROM_BIOS
;
CPY_ROM_LEN	EQU	0000H		; Offset from 1000:0000
CPY_ROM_DST	EQU	0004H		; Offset for destination address
CPY_ROM_SRC	EQU	0008H		; Offset for source address
CPY_ROM_SEG	EQU	1000H		; Nice even place to put it
CPY_ROM_ABS	EQU	80000000H	; Where to enable the decoder
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Equates for protected-mode copy operation.
;
LIMIT		EQU	0		; Segment limit value
LOWBASE		EQU	2		; Low base address offset
HIBASE		EQU	4		; High base address offset
ACCESS		EQU	5		; Access byte offset
HILIMIT		EQU	6		; High limit offset
PROTECTED	EQU	1		; Protected-mode bit
PROT_RET	EQU	09H		; Return from protected mode
SHUTDOWN	EQU	0FH		; Offset of shutdown byte
MEM_CHK		EQU	080H		; Memory parity check
IOC_CHK		EQU	040H		; I/O channel check
PAR_CHK		EQU	(MEM_CHK OR IOC_CHK)
PAR_CHK_ON	EQU	0F3H
PAR_CHK_OFF	EQU	(NOT PAR_CHK_ON)
NMI_OFF		EQU	80H		; CMOS index 0, NMI disabled
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Global descriptor table for block memory moves to or from anywhere
;	in memory including extended memory up to 16 megabytes.
;
GDT	STRUC
DUMMY1	DQ	(0)			; Dummy entry
GDT_LOC	DQ	(0)			; Init by BIOS
SRC_LEN	DW	(0)			; Length in BYTES
SRC_LOW	DW	?			; Where to get the data
SRC_HIG	DB	?			; Where to get the data
SRC_ACC	DB	93H			; Access byte for source
SRC_NUL	DW	?			; * M A N D A T O R Y *
DES_LEN	DW	?			; Destination length in BYTES
DES_LOW	DW	?			; Where to put the data
DES_HIG	DB	?			; Where to put the data
DES_ACC	DB	93H			; Access byte for destination
DES_NUL	DW	?			; * M A N D A T O R Y *
BIO_GDT	DQ	(0)			; Completed by BIOS
SSP_DGT	DQ	(0)			; Completed by BIOS
GDT	ENDS
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	These are offsets in the BIOS segment (40H). This should be
;	"BIOS SEGMENT AT 40H" with data allocations, but some assemblers
;	won't take that syntax.
;
RS232C_0	EQU	0	; First RS-232C base-port
RS232C_1	EQU	2
RS232C_2	EQU	4
RS232C_3	EQU	6	; Last RS-232C base port
PRINT_0		EQU	8H	; First printer base port
PRINT_1		EQU	0AH
PRINT_2		EQU	0CH
PRINT_3		EQU	0EH	; Last printer (stolen for extended RAM)
DEVICE_LIST	EQU	10H	; Installed equipment found
MFJ_TEST_PORT	EQU	12H	; Some junk
MEM_KBYTES	EQU	13H	; Memory found in kilobytes
MFG_TEST_PORT	EQU	15H	; Some test area
KBD_STAT	EQU	16H	; User test area
KEY_SHIFT	EQU	17H	; Shift key status
KEY_XSHIFT	EQU	18H	; Extended shift
ALT_KEY		EQU	19H	; Alt key was pressed
KBD_TAIL	EQU	1AH	; Tail pointer for keyboard
KBD_HEAD	EQU	1CH	; Head pointer for keyboard
KBD_BUF		EQU	1EH	; 16 WORDS for keyboard buffer
DISK_CAL_STA	EQU	3EH	; Diskette calibrate status
DISK_MOT_STA	EQU	3FH	; Diskette motor status
DISK_MOT_TIM	EQU	40H	; Diskette motor timer
DISK_RET_STA	EQU	41H	; Diskette return status
NEC_STATUS	EQU	42H	; 42 to 48 hex
VID_MOD_SET	EQU	49H	; Video mode setting
VID_NR_COLS	EQU	4AH	; Number of columns
VID_PAGE_SIZE	EQU	4CH	; Video page size
VID_PAG_ADDR	EQU	4EH	; Current page address
VID_CUR_POS0	EQU	50H	; Cursor position, page 0
VID_CUR_POS1	EQU	52H	; Cursor position, page 1
VID_CUR_POS2	EQU	54H	; Cursor position, page 2
VID_CUR_POS3	EQU	56H	; Cursor position, page 3 (more)
VID_CUR_TYPE	EQU	60H	; 6845-compatible high byte = start
VID_CUR_PAGE	EQU	62H	; Current video page
VID_CNTL_BASE	EQU	63H	; Video control base port
VID_MOD_SEL	EQU	65H	; Current mode select register
VID_PAL_VAL	EQU	66H	; Current pallet value
POST_RES_LOW	EQU	67H	; Offset for reset code
POST_RES_HIGH	EQU	69H	; Segment for reset code
LAST_INTER	EQU	6BH	; Last unexpected interrupt
TIMER_LOW	EQU	6CH	; Low word of timer tick
TIMER_HIG	EQU	6EH	; High word of timer tick
TIMER_DAY	EQU	70H	; 24 hour roll over flag
BOOT_FLAG	EQU	72H	; Set to 0x1234 on boot
DISK_INFO	EQU	8FH	; Diskette controller info
MEDIA_TYPE0	EQU	90H	; Media type for first floppy
MEDIA_TYPE1	EQU	91H	; Media type for second floppy
TRACK_DRV0	EQU	94H	; Current track number
TRACK_DRV1	EQU	95H	; Current track number
KBD_LED		EQU	97H	; Current keyboard LED
DAYS_SINCE	EQU	0CEH	; Days since 1-1-80
INTV_TIM	EQU	0E0H	; Private interval timer (WORD)
COPR_CTLW	EQU	0F8H	; Coprocessor control word
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=- END of PBIOS.INC =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
