;
;
;	Created	12-APR-2000		Richard B. Johnson
;	Analogic Corporation
;
;	This is the machine-specific preface to the generic BIOS
;	code. Control is passed here using a NEAR jump so that the
;	high address bits are still active.  Some information for
;	this code code was taken from START16.ASM source provided
;	by AMD. Its copyright notice follows.
;
;=============================================================================
;
;   Copyright 1999 Advanced Micro Devices, Inc.
;
;  This software is the property of Advanced Micro Devices, Inc  (AMD)  which
;  specifically grants the user the right to modify, use and distribute this
;  software provided this COPYRIGHT NOTICE is not removed or altered.  All
;  other rights are reserved by AMD.
;
;  THE MATERIALS ARE PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTY
;  OF ANY KIND INCLUDING WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT OF
;  THIRD-PARTY INTELLECTUAL PROPERTY, OR FITNESS FOR ANY PARTICULAR PURPOSE.
;  IN NO EVENT SHALL AMD OR ITS SUPPLIERS BE LIABLE FOR ANY DAMAGES WHATSOEVER
;  (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, BUSINESS
;  INTERRUPTION, LOSS OF INFORMAITON) ARISING OUT OF THE USE OF OR INABILITY
;  TO USE THE MATERIALS, EVEN IF AMD HAS BEEN ADVISED OF THE POSSIBILITY OF
;  SUCH DAMAGES.  BECAUSE SOME JURSIDICTIONS PROHIBIT THE EXCLUSION OR
;  LIMITATION OF LIABILITY FOR CONSEQUENTIAL OR INCIDENTAL DAMAGES, THE ABOVE
;  LIMITATION MAY NOT APPLY TO YOU.
;
;  AMD does not assume any responsibility for any errors that may appear in
;  the Materials nor any responsibility to support or update the Materials.
;  AMD retains the right to make changes to its test specifications at any
;  time, without notice.
;
;  So that all may benefit from your experience, please report  any  problems
;  or suggestions about this software back to AMD.  Please include your name,
;  company,  telephone number,  AMD product requiring support and question or
;  problem encountered.
;
;  Advanced Micro Devices, Inc.         Worldwide support and contact
;  Embedded Processor Division            information available at:
;  Systems Engineering                       epd.support@amd.com
;  5204 E. Ben White Blvd.                          -or-
;  Austin, TX 78741                http: www.amd.com/html/support/techsup.html
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
INCLUDE SC520.INC
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The first order of business is waiting the required reset
;	interval, then setting the CBAR register to allow us access
;	to it at a real mode address. Then we clear the programmable
;	address regions. Note from Page 10-31 of the Elan SC520
;	Microcontroller User's Manual; "Once power is applied and the
;	clock is stable, most SDRAM devices require a 100us delay prior
;	to applying an executable command..." 
;	Therefore we wait a bit before doing anything.
;
	XOR	CX,CX			; Max count
	LOOP	$			; Wait here for a bit.
	MOV	DX,CBAR			; Configuration base address
	MOV	EAX,80000000H OR (MCR_SEG SHL 4) OR 0CBH
					; Set the MMCR base to MCR_SEG:0000H
	OUT	DX,EAX			; 32 bit register
	MOV	AX,MCR_SEG		; Set the MMCR segment
	MOV	DS,AX			; Set segment
;
;	Need to clear these to handle a reboot. In other words, these
;	may have been set up from a previous boot. This could be a
;	'warm' boot or worse, a crash. We need to recover from a crash
;	without having to hit the power switch.
;
	CLEAR_REGION_PAR PAR0
	CLEAR_REGION_PAR PAR1
	CLEAR_REGION_PAR PAR2
	CLEAR_REGION_PAR PAR3
	CLEAR_REGION_PAR PAR4
	CLEAR_REGION_PAR PAR5
	CLEAR_REGION_PAR PAR6
	CLEAR_REGION_PAR PAR7
	CLEAR_REGION_PAR PAR8
	CLEAR_REGION_PAR PAR9
	CLEAR_REGION_PAR PAR10
	CLEAR_REGION_PAR PAR11
	CLEAR_REGION_PAR PAR12
	CLEAR_REGION_PAR PAR13
	CLEAR_REGION_PAR PAR14
	CLEAR_REGION_PAR PAR15
	BOOT_PROM_PAR 0E0000H PAR14	; Set PROM real mode access
;
;	The PROM is 128k. The top 64k has the code. Therefore, its base
;	address is at 0xe0000 instead of 0xf0000 where we see the BIOS code.
;	The boot PROM should now be visible at the real-mode address.
;	Jump to it.
;
	DB	FAR_JMP			; Far jump
	DW	OFFSET REST		; Our code
	DW	ROM_SEG			; Load segment
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	We are starting from a reset.  So we are setting everything from
;	scratch. We are in real mode. The cache, write buffer, and read
;	ahead features should all be off.  They default to off after a
;	reset but we do it again so we can test by executing this even
;	after initial configuration.
;
REST:	MOV	BYTE PTR DS:[DBCTL],0	; Turn off the write buffer
	MOV	BYTE PTR DS:[CPUCTL],2	; 133 MHz
	MOV	BYTE PTR DS:[RESCFG],2	; Reset GP Bus
	MOV	CX,1000H		; Small wait
	LOOP	$			; Wait a bit
	MOV	BYTE PTR DS:[RESCFG],0	; Clear reset event
;
;	Reset the PCI bus. When re-booting, failure to do this
;	may (will) cause a hang when entering protected mode.
;
	MOV	WORD PTR DS:[HBCTL],8000H	; Assert PCI RST
	MOV	CX,1000H			; Small wait
	LOOP	$				; Wait a bit
	MOV	WORD PTR DS:[HBCTL],0		; Clear PCI RST
;
;	Turn off the cache.
;
	MOV	EAX,CR0_CD		; Disable caching
	MOV	CR0,EAX			; Set control register
	WBINVD				; Flush the cache (correct)...
					; Need to generate a write-cycle
					; out of the cache.
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The PCI bus may be on from a previous boot or from a failure to
;	reset. This causes a lot of problems that took me weeks to find.
;	Make sure the damn thing is dead, dead, dead before we continue.
;	It is not sufficient to just clear SYSARBMEMB as the documentation
;	shows. PCI devices will "jabber", corrupting data if the host
;	bridge is not turned off.
;
	MOV	DX,PCICFGADR			; PCI configuration address
	MOV	EAX,80000004H			; Enable, index 1 (command)
	OUT	DX,EAX				; Set index register
	MOV	DX,PCICFGDATA			; Data address
	IN	EAX,DX				; Get status/command
	AND	EAX,0FFFF0000H			; Really turn it OFF
	OUT	DX,EAX				; This should do it.
;
	MOV	WORD PTR DS:[SYSARBMEMB],00H	; Now disable PCI access
	MOV	DX,PCICFGADR			; PCI configuration address
	MOV	EAX,80000000H			; Enable, index 0
	OUT	DX,EAX				; Set index register
	MOV	DX,PCICFGDATA			; Data address
	IN	EAX,DX				; Get, throw away
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now finish building the machine where the hardware engineers
;	left off.
;
	MOV	 BYTE PTR DS:[ADDDECTL],   003H	; Disable internal UARTS
	MOV	 BYTE PTR DS:[GPCSRT],     001H	; 1 Set CS recovery time
	MOV	 BYTE PTR DS:[GPCSPW],     009H	; 9 Set CS write pulse width
	MOV	 BYTE PTR DS:[GPCSOFF],    001H	; 1 Set CS offset time
	MOV	 BYTE PTR DS:[GPRDW],      007H	; 7 Set CP read pulse width
	MOV	 BYTE PTR DS:[GPRDOFF],    002H	; 2 Set GP read offset
	MOV	 BYTE PTR DS:[GPWRW],      007H	; 7 Set GP write pulse width
	MOV	 BYTE PTR DS:[GPWROFF],    002H	; 2 Set GP write offset
	MOV	 BYTE PTR DS:[GPALEW],     004H	; 4 Width of BALE pulse
	MOV	 BYTE PTR DS:[GPALEOFF],   001H	; 1 Offset of BALE pulse
        MOV      BYTE PTR DS:[SL1PICMODE], 002H
	MOV	 WORD PTR DS:[PIOPFS15_0],	0FFFFH
	MOV	 WORD PTR DS:[PIOPFS31_16],	0FFFFH
	MOV	 WORD PTR DS:[PIODIR15_0],	00000H
	MOV	 WORD PTR DS:[PIODIR31_0],	00000H
	MOV      WORD PTR DS:[PIOCLR15_0],	00000H
	MOV	 WORD PTR DS:[PIOCLR31_16],	00000H
	MOV	 WORD PTR DS:[PIODATA15_0],	0DE04H
	MOV	 WORD PTR DS:[PIODATA31_16],	0FF9DH
	MOV	 WORD PTR DS:[CSPFS],    000F8H	; Chip select pin functions
	MOV	 WORD PTR DS:[CLKSEL],   00013H	; Set the bus clock
;
	MOV	 BYTE PTR DS:[PIT0MAP],	   001H	; Interrupt 0 pin mapping
	MOV	 BYTE PTR DS:[PIT1MAP],	   000H	; Interrupt 1 pin mapping
	MOV	 BYTE PTR DS:[PIT2MAP],	   000H	; Interrupt 2 pin mapping
	MOV	 BYTE PTR DS:[PICICR],     000H	; Interrupt control
	MOV	 BYTE PTR DS:[MPICMODE],   000H	; Master PIC intr mode
	MOV	 BYTE PTR DS:[UART1MAP],   00CH	; UART interrupt mapping
	MOV	 BYTE PTR DS:[UART2MAP],   00BH	; UART interrupt mapping
	MOV	 BYTE PTR DS:[PCIINTAMAP], PCI_IRQ10	; PCI interrupt A
	MOV	 BYTE PTR DS:[PCIINTBMAP], PCI_IRQ11	; PCI interrupt B
	MOV	 BYTE PTR DS:[PCIINTCMAP], PCI_IRQ12	; PCI interrupt C
	MOV	 BYTE PTR DS:[PCIINTDMAP], PCI_IRQ9	; PCI interrupt D
	MOV	 BYTE PTR DS:[RTCMAP],     003H	; Realtime clock
	MOV	 BYTE PTR DS:[GP0IMAP],	   000H	; No input
	MOV	 BYTE PTR DS:[GP1IMAP],	   002H	; Master IRQ1
	MOV	 BYTE PTR DS:[GP2IMAP],	   007H	; Slave  IRQ4 (IRQ12)
	MOV	 BYTE PTR DS:[GP3IMAP],	   00BH	; Master IRQ3
	MOV	 BYTE PTR DS:[GP4IMAP],	   00CH	; Master IRQ4
	MOV	 BYTE PTR DS:[GP5IMAP],	   00AH	; Slave  IRQ7  (IRQ15)
	MOV	 BYTE PTR DS:[GP6IMAP],	   015H	; Master IRQ6  (floppy)
	MOV	 BYTE PTR DS:[GP7IMAP],	   016H	; Master IRQ7
	MOV	 BYTE PTR DS:[GP10IMAP],   009H	; Slave  IRQ6  (IRQ14)
	MOV	 WORD PTR DS:[GPDMAEXTCHMAPA],	00200H
	MOV	 WORD PTR DS:[GPDMAEXTCHMAPB],	0FFFFH
	MOV	 WORD PTR DS:[BOOTCS1CTL],	01C37H
	MOV	 WORD PTR DS:[BOOTCS2CTL],	01C37H
	MOV	 WORD PTR DS:[SYSARBMEMB], 01FH	; Arbiter control
	MOV	 BYTE PTR DS:[SYSARBCTL], 06H   ; Park on last, concurrent
	MOV	 WORD PTR DS:[HBCTL], 08H	; Posted write enable
	MOV	DWORD PTR DS:[ARBPRICTL], 040000F0FH ; Arbiter priority
        MOV     DWORD PTR DS:[MSTINTADD], 0000H	; Clear the default
	MOV	 WORD PTR DS:[DSCTL],    00200H	; Drive strength control
	MOV	 BYTE PTR DS:[UART1CTL], 004H	; UART 1 general control
	MOV	 BYTE PTR DS:[UART2CTL], 004H	; UART 2 general control
	IO_REGION_PAR 680H, 680H, PAR8		; 0x680 LED
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The LEDs should now be working.
;
	MOV	AL,01H				; Output a 1 to LED 0x80
	OUT	80H,AL
	MOV	DX,680H
	MOV	AX,02H				; Output a 2 to LED 0x680
	OUT	DX,AX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	In a 'warm' boot, we want to preserve memory. We check for a
;	warm boot by checking if the initial short jump has been changed
;	(by us, previously), to a far jump. Commented out. The time-
;	to-boot is quick anyways.
;
;	CMP	BYTE PTR CS:[0FFF0H],FAR_JMP	; See if a soft reset
;	JZ	SOFT				; Yes
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is a cold boot so set up the SDRAM Controller. I do not
;	do "auto-sizing" because the size of the installed RAM is known.
;	Note: Page 10-20, Elan SC520 Microcontroller user's manual shows
;	10.5.5.4 "when specifying NOP, All Banks Precharge, Load Node
;	Register, or Auto Refresh, the command is not actually applied
;	to the SDRAM devices until an Am5x86 CPU access to SDRAM occurs".
;	Clarification obtained during my trip to Austin was that anything
;	put into the DRCCTL register needs a SDRAM write to make it
;	effective.
;
	XOR	EAX,EAX				; Zero
	MOV	SS,AX				; Clear segment registers
	MOV	ES,AX				; except DS = MCR_SEG
	MOV	FS,AX
	MOV	GS,AX
	MOV	EBP,EAX				; SS=EBP=EAX=0
	MOV	SP,0FFFEH			; Just park it somewere
	MOV	BYTE PTR DS:[DRCCTL],01H	; Refresh disable (NOP)
	MOV	[EBP],EAX			; This forces the command out
	MOV	BYTE PTR DS:[DRCCTL],02H	; All banks precharge
	MOV	[EBP],EAX			; This forces the command out
;
IF	(RAM_SIZE EQ 128)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	All banks enabled, 64 meg size each (128 megabytes).
;
	MOV	 BYTE PTR DS:[DRCTMCTL], 01AH
	MOV	 WORD PTR DS:[DRCCFG], 09999H
	MOV	DWORD PTR DS:[DRCBENDADR], 0A0989088H
ELSEIF	(RAM_SIZE EQ 64)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	One bank enabled, 64 meg size.
;
	MOV	 BYTE PTR DS:[DRCTMCTL], 01AH
	MOV	 WORD PTR DS:[DRCCFG], 00909H
	MOV	DWORD PTR DS:[DRCBENDADR], 000900088H

ELSEIF	(RAM_SIZE EQ 32)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Analogic memory mezzanine, 32 meg size.
;	Eight, 4 megabyte blocks.
;
	MOV	 BYTE PTR DS:[DRCTMCTL], 01AH
	MOV	 WORD PTR DS:[DRCCFG], 08888H
	MOV	DWORD PTR DS:[DRCBENDADR], 08884H
ELSE
	%OUT	RAM Size not correctly defined.
	.ERR1
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Two auto-refreshes.
;
	MOV	BYTE PTR DS:[DRCCTL],04H
	MOV	[EBP],EAX
	MOV	[EBP],EAX
;
;	Load mode register command.
;
	MOV	BYTE PTR DS:[DRCCTL],03H
	MOV	[EBP],EAX			; This forces the command out
;
;	Back to normal mode, refresh enable 15.6 us.
;
	MOV	BYTE PTR DS:[DRCCTL],18H
	MOV	[EBP],EAX			; This forces the command out
;
;	Okay, RAM should be working. Clear low 1 megabyte and try it.
;
	MOV	AL,03H				; Show we try it.
	OUT	80H,AL				; First LED
	XOR	AX,AX				; Zero
	MOV	DX,AX
	MOV	DI,AX				; ES:DI = 0
CLRRAM:	MOV	ES,DX				; 64k blocks
	MOV	CX,8000H			; 64k (in words)
	REP	STOSW				; Clear
	ADD	DX,1000H			; Next 64k
	CMP	DX,MCR_SEG			; Don't ruin MMCRs
	JC	SHORT CLRRAM
;
	MOV	AX,BIOS_SEG			; BIOS data segment
	MOV	DS,AX				; DS = BIOS data
	MOV	SS,AX				; SS = BIOS data
	MOV	SP,0FFFEH			; Put on some even address
	MOV	WORD PTR DS:[0],AX		; Write something
	MOV	AX,055AAH			; Pattern
	MOV	WORD PTR DS:[0],AX		; Set a pattern
	PUSH	CX				; Exercise the bus
	POP	CX
	XOR	WORD PTR DS:[0],AX		; Should set zero
	JZ	SHORT RAMOK			; Yes, continue
	MOV	AL,0FFH				; Show it failed.
	OUT	80H,AL				; First LED
	JMP	$				; Wait forever
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	After RAM is enabled, we can set the rest of the PARs. Hidden
;	in the documentation is the caveat that RAM must be enabled first
;	or memory-stuff always points to the PCI bus.
;
;	But first we must get rid of the BIOS ROM. We shadow this code.
;	No big problem.	Copy the contents of this PROM to the low memory
;	area we just cleared.
;
RAMOK:	COPY64K	ROM_SEG CPY_SEG		; Copy to CPY_SEG
	DB	FAR_JMP			; Far jump
	DW	OFFSET RLOC0		; Our next instruction
	DW	CPY_SEG			; Load segment
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	We are now executing in low memory (I hope). Disable the
;	PAR that enabled the PROM at F000:0000. Then copy everything
;	back. RAM should exist where the BIOS ROM used to be. If
;	it doesn't, we are screwed anyway.
;
RLOC0:	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR 
	CLEAR_REGION_PAR PAR14		; Clear PROM real mode access
	CLEAR64K ROM_SEG		; Clear memory now visible
	COPY64K  CPY_SEG ROM_SEG	; Copy program code back
	DB	FAR_JMP			; Far jump
	DW	OFFSET RLOC1		; Our next instruction
	DW	ROM_SEG			; Load segment
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	We should now be executing out of the BIOS area, but from RAM.
;	We should write-protect this. We need to checksum the area we
;	modify,  set a new warm-boot jump-vector, then modify the checksum.
;
RLOC1:	XOR	BL,BL			; Clear, use for accumulator
	MOV	AX,CS			; Current code
	MOV	ES,AX			; ES = CS
	MOV	DS,AX			; DS = CS
	MOV	DI,0FFF0H		; Reset vector offset
	SUB	BL,BYTE PTR [DI]	; Get whatever is there
	MOV	AL,FAR_JMP		; Far jump
	ADD	BL,AL			; Start running checksum
	STOSB				; Into jump vector
	SUB	BL,BYTE PTR [DI]	; Sum checksum
	SUB	BL,BYTE PTR [DI+1]	; Next byte of word
	MOV	AX,OFFSET BEGIN		; Offset to start
	ADD	BL,AL			; Adjust running checksum
	ADD	BL,AH			; Adjust running checksum
	STOSW				; Write offset
	SUB	BL,BYTE PTR [DI]	; Sum checksum
	SUB	BL,BYTE PTR [DI+1]	; Next byte of word
	MOV	AX,CS			; Get code segment
	ADD	BL,AL			; Adjust running checksum
	ADD	BL,AH			; Adjust running checksum
	STOSW				; Write code segment
	SUB	BYTE PTR DS:[CKSUM],BL	; Sum new checksum
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now set up access for the possible ISA devices and write-protect
;	the BIOS area.
;
	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR
	MEM_WPROT_PAR  0FH, 1,    PAR2	; Write-protect BIOS
	MEM_REGION_PAR 0AH, 2,    PAR3	; 0xa0000 -> 0xbffff screen buffers
	MEM_REGION_PAR 0CH, 1,    PAR4	; 0xc0000 -> 0xcffff screen BIOS
	IO_REGION_PAR 46E8H, 46E8H,PAR5 ; VGA video control register
IFDEF OLD_RAMDISK
	MEM_REGION_PAR 0DH, 1,	PAR6	; Ramdisk board (private)
ELSE
	NVRAM_DEV_PAR  0DH, 1,	PAR6	; NVRAM on board
ENDIF
	BOOT_ROM_SETUP 1000H, 2, PAR9	; Put Boot ROM at 0x10000000 abs
	MEM_REGION_CS3 2000H, 16, PAR10	; Put VXI RAM at 0x20000000 abs
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The documentation says something about "Execution from  GPMEM is
;	discouraged". So I'll shadow the VGA BIOS, and anything else in
;	that area, to RAM. I will shadow 64k from C000:0000 to C000:FFFF
;
	MOV	AX,SCR_SEG		; Screen-card BIOS
	MOV	DS,AX			; DS = screen-card
CKSCRB:	MOV	WORD PTR DS:[0],00000H	; Clear any shadowed RAM
	CMP	WORD PTR DS:[0],0AA55H	; Anything there?
	JZ	SHORT SCRFND		; Yes
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	No screen BIOS, probably it's PCI. Disable PARs for the video-
;	control register and the screen-BIOS area. To enable the low-
;	value port addresses to be accessed from the PCI bus, we have to
;	set the 'I/O hole' bit in the address control register. Once we do
;	this, all I/O ports end up going to the PCI bus. Therefore, we have
;	to set up some Programmable Address Registers (PARs) to enable
;	the ports we use into the general purpose I/O space.
;
	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR
	CLEAR_REGION_PAR PAR3		; Clear video register region
	MOV	BYTE PTR DS:[ADDDECTL], 013H	; Set HOLE bit
	IO_REGION_PAR 200H, 3AEH, PAR4	; Enable redirected ports
	IO_REGION_PAR 3E0H, 400H, PAR5	; Need these ports too
IFDEF	NEW_SUPERIO
	IO_REGION_PAR 015CH, 015DH,PAR7	; For super/IO data port
ENDIF
	PCI_REGION_PAR 0AH, 2, PAR0	; 0xa0000 -> 0xbffff screen buffers
	JMP	SHORT SCREND		; All done here
;
SCRFND:	COPY64K	SCR_SEG CPY_SEG		; Copy screen BIOS to CPY_SEG
	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR
	CLEAR_REGION_PAR PAR4		; Clear MMCR access
	CLEAR64K SCR_SEG		; Clear memory now visible
	COPY64K	 CPY_SEG SCR_SEG	; Copy screen BIOS back
	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR
	MEM_WPROT_PAR  0CH, 1,    PAR4	; Write-protect screen BIOS area
SCREND:	MOV	AL,05H			; Show we did it.
	OUT	80H,AL			; First LED
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Enable buffering
;
	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR
;                      ** DEATH **
; The following is commented out. If enabled, access to PROM space
; will cause the machine to die a horrible death and you will lose
; all your hair trying to find the problem.
;
;	MOV	BYTE PTR DS:[DBCTL],00011001B
;		                    ||||||||_____ Enable write buffer
;		                    |||||||______ Write buffer flush
;                                   ||||||_______ Write buffer watermark
;		                    ||||_________ Read ahead
;		                    |||__________ Reserved
	JMP	SHORT CFG_SUPER
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The following initializes the super I/O chip.
;
;	The configuration table consists of the index to write to the
;	index port, followed by the byte to write to the data port.
;
IFDEF	NEW_SUPERIO			; If we use the new Super I/O chip
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for the new Super I/O chip, National PC87309.
;
SUPIO_INDX	EQU	015CH		; Super I/O index port
SUPIO_DATA	EQU	015DH		; Super I/O data port
SUPERIO	LABEL	BYTE
	DB	02H, 01H		; Reset, load defaults
	DB	22H, 10H		; SIOCF2, Data rate select pin
	DB	07H, 00H		; Select logical device 0 (FDC)
	DB	31H, 00H		; I/O Range check disable
	DB	30H, 01H		; Enable FDC
	DB	60H, 03H		; MSB of base address
	DB	61H, 0F2H		; LSB of base address
	DB	70H, 06H		; Interrupt select IRQ6
	DB	71H, 02H		; Interrupt type (active high, edge)
	DB	74H, 01H		; DMA Channel 1 (actually 2)
	DB	07H, 01H		; Select logical device 1 (printer)
	DB	31H, 00H		; I/O Range check disable
	DB	30H, 01H		; Enable printer
	DB	70H, 07H		; Set IRQ 7 for printer
	DB	07H, 02H		; Select logical device 2 UART2
	DB	31H, 00H		; I/O Range check disable
	DB	30H, 01H		; Enable UART
	DB	07H, 03H		; Select logical device 3 UART1
	DB	31H, 00H		; I/O Range check disable
	DB	30H, 01H		; Enable UART
	DB	07H, 05H		; Select logical device 5 Mouse
	DB	31H, 00H		; I/O Range check disable
	DB	70H, 0CH		; Select IRQ for PS2 mouse
	DB	07H, 06H		; Select logical device 6 Keyboard
	DB	31H, 00H		; I/O Range check disable
	DB	30H, 01H		; Enable keyboard
CFG_LEN	EQU 	($ - SUPERIO) SHR 1
CFG_SUPER:
	CLI				; In case I started from DEBUG
	MOV	AX,CS			; Configuration table
	MOV	DS,AX			; Config table segment
	MOV	SI,OFFSET SUPERIO	; Point to table
	MOV	CX,CFG_LEN		; Length of the table
SUPER:	MOV	DX,SUPIO_INDX		; Configuration port
	LODSB				; Get index
	OUT	DX,AL			; Set index
	SLOW_IO				; Let index settle
	MOV	DX,SUPIO_DATA		; Set data port
	LODSB				; Get byte to write
	OUT	DX,AL			; Write data byte
	SLOW_IO				; Let data settle
	LOOP	SUPER			; Do all bytes
ELSE					; Development system
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for the Super I/O chip that came with the development
;	system (ALI 512x).
;
SUPIO_BASE	EQU	03F0H		; Super I/O base port
SUPERIO	LABEL	BYTE
	DB	07H, 00H		; Select logical device 0 (FDC)
	DB	30H, 01H		; Enable
	DB	74H, 00H		; DMA Channel 0 (actually 2)
	DB	07H, 03H		; Select logical device 3 (printer)
	DB	30H, 01H		; Enable
	DB	70H, 07H		; Set IRQ 7 for printer
	DB	07H, 04H		; Select logical device 4
	DB	30H, 01H		; Enable
	DB	07H, 05H		; Select logical device 5
	DB	30H, 01H		; Enable
	DB	07H, 07H		; Select logical device 7
	DB	72H, 0CH		; Select IRQ for PS2 mouse
CFG_LEN	EQU 	($ - SUPERIO) SHR 1
CFG_SUPER:
	CLI				; In case I started from DEBUG
	MOV	AX,CS			; Configuration table
	MOV	DS,AX			; Config table segment
	MOV	SI,OFFSET SUPERIO	; Point to table
	MOV	CX,CFG_LEN		; Length of the table
	MOV	DX,SUPIO_BASE		; Configuration port
	MOV	AL,51H			; First key value
	OUT	DX,AL			; Set first key value
	SLOW_IO				; Let it set up
	MOV	AL,23H			; Second key value
	OUT	DX,AL			; Set second key value
	SLOW_IO				; Let it set up
SUPER:	LODSB				; Get index
	OUT	DX,AL			; Set index
	SLOW_IO				; Let index settle
	INC	DX			; Set data port
	LODSB				; Get byte to write
	OUT	DX,AL			; Write data byte
	SLOW_IO				; Let data settle
	DEC	DX			; Back to index port
	LOOP	SUPER			; Do all bytes
	MOV	AL,0BBH			; Token to enter RUN mode
	OUT	DX,AL			; Out index/config port
	SLOW_IO				; Let it set up
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set the interrupt controllers for the required level-sensitive
;	settings for the interrupts using on the PCI bus. These are
;	IRQ2 (cascade), IRQ9, IRQ10, IRQ11, IRQ12, and IRQ15.
;
	MOV	AX,MCR_SEG		; MMCR Data segment
	MOV	DS,AX			; DS = MMCR
	MOV	BYTE PTR DS:[MPICMODE],00000100B
;                                      ||||||||_____ IRQ0
;                                      |||||||______ IRQ1
;                                      ||||||_______ IRQ2 (cascade)
;                                      |||||________ IRQ3
;                                      ||||_________ IRQ4
;                                      |||__________ IRQ5
;                                      ||___________ IRQ6
;                                      |____________ IRQ7
;
	MOV	BYTE PTR DS:[SL1PICMODE],11111110B
;                                        ||||||||____ IRQ8
;                                        |||||||_____ IRQ9
;                                        ||||||______ IRQ10
;                                        |||||_______ IRQ11
;                                        ||||________ IRQ12
;                                        |||_________ IRQ13
;                                        ||__________ IRQ14
;                                        |___________ IRQ15
;
	MOV	BYTE PTR DS:[PICICR],00000000B		; Global enable
;                                    ||||||||______ Master PIC
;                                    |||||||_______ Slave 1 PIC
;                                    ||||||________ Slave 2 PIC
;                                    |||||_________ Reserved bits
;                                    ||____________ NMI enable
;                                    |_____________ NMI done
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Remove the MMCR alias and zero the RAM that existed in that
;	area. This way, an operating system won't accidentally
;	step on our configuration registers.
;
	MOV	DX,CBAR			; Configuration base address
	MOV	EAX,0CBH		; Remove configuration alias
	OUT	DX,EAX			; 32 bit register
	CLEAR64K MCR_SEG		; Where MMCR previously existed
	CLEAR64K CPY_SEG		; Clear workspace
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	At this point we are supposed to have a basic PC/AT. Now we
;	continue with the real BIOS initialization.
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
