;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
TITLE	A BIOS For the Intel(r)/PC-AT Architecture
PAGE	,132
SUBTTL	System definitions
;
;	File:  PBIOS.ASM     Created 10-FEB-2000    Richard B. Johnson
;
;	This is a BIOS for an Intel machine. It is designed to be just
;	smart enough to boot the machine. Some of the PC/AT functionality
;	is not implemented. Further none of the "compatibility locations"
;	are implemented. In other words, procedures in the BIOS do not
;	exist at predefined physical locations. The interrupt table has
;	the appropriate vectors in the correct places, and the BIOS data
;	segment has the data locations in the correct places, but if some
;	dumb application does 'CALL FAR PTR F000:1234", expecting to
;	execute a procedure, all bets are off and the application will
;	certainly crash.
;
;	This is built and tested under MS-DOS. For instance, when the
;	floppy-disk interface is tested, just patch the vector, issue
;	the DOS "terminate an stay resident" function call, then
;	you can test what just replaced the default PROM code.
;
;	Presently this must be built using Micro$oft MASM or Borland
;	TASM.  To build, you execute:
;
;		TASM PBIOS;
;		LINK/T BIOS;
;
;	You can load this into memory under DOS to test. First you
;	must make certain that there is no memory manager installed in
;	"\CONFIG.SYS". Then you do:
;
;		DEBUG BIOS.COM
;		M 100 5000 9000:100
;		G=9000:100
;	Or:
;		DEBUG PBIOS.HEX
;		M 0 FFFF 9000:0
;		U 9000:FFF0		Examine start address
;		G=9000:(start address)
;
;	PBIOS will pretend that there is only 512k of low memory
;	available so it will boot a DOS formatted floppy in drive
;	"A:", leaving the resident BIOS at 9000:100 alone.
;
;	There is a utility, FIXUP.EXE, which makes a PROM image by
;	extending the length of the file to 64k and putting the proper
;	jump instruction 16 bytes from the end. This can be burned
;	into a PROM (or NVRAM) and the machine should boot with it
;	installed.
;
;	This code must be assembled using either Microsoft MASM or
;	Borland TASM. Extensive use of the 'PTR' argument prevents
;	assembly errors and warnings.
;
;	This BIOS will live at F000:0. This is where a 64k PROM or
;	the NVRAM equivalent must live. Upon startup, the CPU will
;	branch to FFFF:0. There will be a jump instruction to make
;	the CPU go to the real BIOS startup. On some machines, the
;	CPU really starts in 32-bit mode, 0xfffffff0, 16 bytes from
;	the end of the address space. It is aliased to the same
;	16-bit address and the first jump actually loads the 16-bit
;	code segment register.
;
;	I wanted to do timeouts better than some BIOS that use spin-
;	counts. Therfore I used a memory location called INTV_TIM
;	for a down-counter. This location may have to be changed
;	if it is found to conflict with something.
;
;	Many port names and other variable names are taken from those
;	published in the IBM Technical Reference manuals:
;
;	IBM Technical Reference Personal Computer XT  6322508
;	IBM Technical Reference Personal Computer AT  6280070
;
;	Compatability information and specifications are obtained from
;	System BIOS for IBM PC/XT/AT Computers and Compatibles,
;	Phoenix Technologies Ltd.	ISBN 0-201-51806-6
;
;	Hardware device specifications are obtained from their respective
;	data sheets. None of this source-code was copied from any other
;	source-code.
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
INCLUDE VERSION.INC
INCLUDE PBIOS.INC
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
SUBTTL	Real mode startup code
PAGE	,132
;
;	So I can test this in a running 'DOS' environment, I start
;	at 100H. Conditional assembly makes it start at 0.
;
;	Rules for startup procedures are that all segments are preserved.
;	However, all other registers except the stack are trashed.
;
PSEG	SEGMENT	USE16 PARA PUBLIC 'BIOS'
	ASSUME  CS:PSEG, DS:PSEG, ES:PSEG, SS:PSEG
IFDEF	COM
	ORG	100H
START:	JMP	BEGIN
ELSE
	ORG	0
START:
ENDIF
SIGN0	DB	'Analogic(tm) SC520 BIOS version ',VERS
SIGN_LEN	EQU	($ - SIGN0)
CENTER_LOGO	EQU	(COLS - SIGN_LEN) / 2
PCRLF	DB	CR,LF,0
SIGN1	DB	'Copyright(c) 2000-2004 Analogic Corporation'
CPYR_LEN	EQU	($ - SIGN1)
CENTER_CPYR	EQU	(COLS - CPYR_LEN) / 2
	DB	CR,LF,LF,0
BOOTM	DB	'Booting .....', CR,LF,0
FAILM	DB	'Can''t boot from the default drive.'
USRINFO	DB	CR,LF,'Enter ^C to retry, ^A to test RAM ..... ',0
CTLFAL	DB	'second interrupt controller or CMOS timer failure ',0
DIVZERO	DB	CR,LF,'Integer divide by zero error at ',0
BADOPC	DB	CR,LF,'Invalid opcode error at ',0
PARITY	DB	CR,LF,'Non-maskable interrupt at ',0
CTLDEL	DB	CR,LF,'Ctrl-Alt-Del (Reboot)',CR,LF,0
PKBRD	DB	'Checking keyboard ..... ',0
PCHKSUM	DB	'Check ROM checksum .... ',0
CHKCMOS	DB	'Checking CMOS timer ... ',0
CHKTIM	DB	'Checking main timer ... ',0
CHKTM2	DB	'Checking timer 2 ...... ',0
CKFLOP	DB	'Check FDC operation ... ',0
CHKPROT	DB	'Check virtual mode .... ',0
CHKEXTR	DB	'Check extended RAM .... ',0
CHKSCR	DB	'Check screen memory.... ',0
SYSRAM	DB	'System RAM found ...... ',0
EXTRAM	DB	'Extended RAM found .... ',0
CHKPCI	DB	'Check PCI devices ..... ',0
ROMMOD	DB	'ROM module scan ....... ',0
CMOSTIM	DB	'CMOS time ............. ',0
MATHCO	DB	'Math coprocessor ...... ',0
PRINTR	DB	'Printer ports found ... ',0
COMPRT	DB	'RS-232C ports found ...'
SPACES	DB	' ',0
KBYTES	DB	' Kilobytes',CR,LF,0
PFOUND	DB	'found',0
PNOT	DB	'not ',0
CHKTOK	DB	'okay ',0
CKFAIL	DB	'failed ',0
CMOSBAD	DB	'CMOS checksum is bad. It is ',0
CMOSSHD	DB	', should be ',0
PCTIME	DB	CR,LF,'Set time (hh:mm:ss) [ ',0
PCDATE	DB	CR,LF,'Set date (mm-dd-yyyy) [ ',0
PNOCHG	DB	'] [Enter] to accept: ',0
BOOTDEV	DB	CR,LF,'Boot device [F]loppy or [N]VRAM [',0
ANYKEY	DB	CR,LF,'Hit [Enter] to reboot: [ ]',BS,BS,0
PBYTES	DB	' bytes',CR,LF,0
CKLOWR	DB	'Checking low RAM ... ',0
CKADDR	DB	'Address ',0
DATWAS	DB	' was ',0
DASHLD	DB	' should have been ',0
IFDEF	TEST_FLOPPY
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for testing the floppy diskette code
;
PREST	DB	'        Reset', CR,LF,0	; 0
PSTAT	DB	'   Get status', CR,LF,0	; 1
PREAD	DB	'         Read', CR,LF,0	; 2
PWRIT	DB	'        Write', CR,LF,0	; 3
PVERF	DB	'       Verify', CR,LF,0	; 4
PFORM	DB	'       Format', CR,LF,0	; 5
PUNKN	DB	'      Unknown', CR,LF,0	; ?
PRPAR	DB	'   Parameters', CR,LF,0	; 8
PTYPE	DB	'   Drive type', CR,LF,0        ; 15h
PCHAN	DB	' Media change', CR,LF,0	; 16h
PSETD	DB	'Set disk type', CR,LF,0	; 17h
PSFRM	DB	'  Format type', CR,LF,0	; 18h
PRSLT	DB	'       Result', CR,LF,0
PDRIV	DB	'        Drive : ',0
PSECT	DB	'      Sectors : ',0
PTART	DB	' Start sector : ',0
PTRAK	DB	'     Cylinder : ',0
PHEAD	DB	'         Head : ',0
PADDR	DB	'      Address : ',0
PSTEX	DB	'       Status : ',0
PCARY	DB	'CY'		; CY with following space
PSPACE	DB	' ',0		; Space only
SHOTAB:	DW	PREST		; 0
	DW	PSTAT		; 1
	DW	PREAD		; 2
	DW	PWRIT		; 3
	DW	PVERF		; 4
	DW	PFORM		; 5
	DW	PUNKN		; 6
	DW	PUNKN		; 7
	DW	PRPAR		; 8
	DW	PUNKN		; 9
	DW	PUNKN		; 0A
	DW	PUNKN		; 0B
	DW	PUNKN		; 0C
	DW	PUNKN		; 0D
	DW	PUNKN		; 0E
	DW	PUNKN		; 0F
	DW	PUNKN		; 10
	DW	PUNKN		; 11
	DW	PUNKN		; 12
	DW	PUNKN		; 13
	DW	PUNKN		; 14
	DW	PTYPE		; 15
	DW	PCHAN		; 16
	DW	PSETD		; 17
	DW	PSFRM		; 18
	DW	PUNKN		; 19
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
PATTERN	DB	00000001B
	DB	00000011B
	DB	00000111B
	DB	00000110B
	DB	00000100B
	DB	00000000B
PATTERN_LEN EQU $ - PATTERN
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The following four lines should not be changed. FIXUP looks
;	for this code to determine the address of the initial JMP
;	instruction 16 bytes from the end of the PROM at offset 0FFF0H.
;
ALIGN	4
BEGIN:	CLI				; This should remain off
	CLD				; Do this only once
	NOP				; Signature
	NOP				; Signature
IFDEF	COM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is used to test the PCI BIOS in a MS-DOS environment.
;
IFDEF	TEST_PCI
	JMP TESTPCI
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The following tests some of the PCI access functionality in
;	the MS-DOS environment.
;
;
TPCI_READB	DB	'Test PCI read configuration BYTE',CR,LF,0
TPCI_READW	DB	'Test PCI read configuration WORD',CR,LF,0
TPCI_READL	DB	'Test PCI read configuration LONG',CR,LF,0
TPCI_CLASS	DB	'Test PCI class code',CR,LF,0
TPCI_DEVIC	DB	'Test PCI find device',CR,LF,0
TPCI_HBIOS	DB	'Host BIOS ',0
TPCI_TBIOS	DB	'Test BIOS ',0
TPCI_CMPFL	DB	'Compare failed at ',0
;
TESTPCI:
	MOV	AX,CS			; Get current code segment
	MOV	DS,AX			; Set data segment
	MOV	ES,AX			; Set extra segment
	MOV	SS,AX			; Set stack segment
	MOV	SP,100H			; Set stack
	STI				; Enable interrupts
	PUSH	ES			; Save
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; ES = 0
	MOV	DI,(80H SHL 2)		; Temporary PCI BIOS entry
	MOV	AX,OFFSET INT_1AH	; Our local procedure
	STOSW				; Write OFFSET
	MOV	AX,CS			; Get segment
	STOSW				; Write segment
	POP	ES			; Restore segment
;
	MOV	SI,OFFSET TPCI_READB	; Point to 'read BYTE' prompt
	CALL	PRINT			; Print it
	MOV	SI,OFFSET TPCI_HBIOS	; Point to host prompt
	CALL	PRINT			; Print it
	MOV	BH,0			; Bus number
	MOV	BL,0			; Device/function
	MOV	DI,0			; First register
THBYTE:	MOV	AX,0B108H
	INT	1AH
	MOV	AL,CL
	CALL	HEXB			; Write to screen
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	INC	DI			; Ready next
	CMP	DI,16H			; Check limits
	JC	SHORT THBYTE
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_TBIOS	; Point to test prompt
	CALL	PRINT			; Print it
	MOV	BH,0			; Bus number
	MOV	BL,0			; Device/function
	MOV	DI,0			; First register
TTBYTE:	MOV	AX,0B108H
	INT	80H
	MOV	AL,CL
	CALL	HEXB			; Write to screen
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	INC	DI			; Ready next
	CMP	DI,16H			; Check limits
	JC	SHORT TTBYTE
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_READW	; Point to 'read WORD' prompt
	CALL	PRINT			; Print it
	MOV	SI,OFFSET TPCI_HBIOS	; Point to host prompt
	CALL	PRINT			; Print it
	MOV	BH,0			; Bus number
	MOV	BL,0			; Device/function
	MOV	DI,0			; First register
THWORD:	MOV	AX,0B109H
	INT	1AH
	MOV	AX,CX
	CALL	HEXW			; Write to screen
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	ADD	DI,2			; Ready next
	CMP	DI,16H			; Check limits
	JC	SHORT THWORD
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_TBIOS	; Point to test prompt
	CALL	PRINT			; Print it
	MOV	BH,0			; Bus number
	MOV	BL,0			; Device/function
	MOV	DI,0			; First register
TTWORD:	MOV	AX,0B109H
	INT	80H
	MOV	AX,CX
	CALL	HEXW			; Write to screen
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	ADD	DI,2			; Ready next
	CMP	DI,16H			; Check limits
	JC	SHORT TTWORD
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_READL	; Point to 'read LONG' prompt
	CALL	PRINT			; Print it
	MOV	SI,OFFSET TPCI_HBIOS	; Point to host prompt
	CALL	PRINT			; Print it
	MOV	BH,0			; Bus number
	MOV	BL,0			; Device/function
	MOV	DI,0			; First register
THLONG:	MOV	AX,0B10AH
	INT	1AH
	MOV	EAX,ECX
	CALL	HEXD			; Write to screen
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	ADD	DI,4			; Ready next
	CMP	DI,16H			; Check limits
	JC	SHORT THLONG
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_TBIOS	; Point to test prompt
	CALL	PRINT			; Print it
	MOV	BH,0			; Bus number
	MOV	BL,0			; Device/function
	MOV	DI,0			; First register
TTLONG:	MOV	AX,0B10AH
	INT	80H
	MOV	EAX,ECX
	CALL	HEXD			; Write to screen
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	ADD	DI,4			; Ready next
	CMP	DI,16H			; Check limits
	JC	SHORT TTLONG
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_CLASS	; Point to prompt
	CALL	PRINT			; Write to screen
	MOV	SI,OFFSET TPCI_HBIOS	; Point to host prompt
	CALL	PRINT			; Print it
	MOV	ECX,00010000H		; Mass storage
	MOV	SI,0			; First register
	MOV	AX,0B103H		; PCI Class code
	INT	1AH			; Get host function
	PUSH	BX			; Save result
	MOV	AX,BX			; Get value
	CALL	HEXW			; Print it
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	POP	BX			; Restore result
	MOV	DI,0			; First register
	MOV	AX,0B10AH		; Read a DWORD
	INT	1AH
	PUSH	ECX			; Save result
	MOV	EAX,ECX
	CALL	HEXD			; Write to screen
	MOV	SI,OFFSET PCRLF		; Point to end of line
	CALL	PRINT			; Print it
	MOV	SI,OFFSET TPCI_TBIOS	; Point to test prompt
	CALL	PRINT			; Print it
	MOV	ECX,00010000H		; Mass storage
	MOV	SI,0			; First register
	MOV	AX,0B103H		; PCI Class code
	INT	80H			; Get host function
	PUSH	BX
	MOV	AX,BX			; Get value
	CALL	HEXW			; Print it
	MOV	AH, DUMB_TERM		; Dumb terminal mode
	MOV	AL,' '			; Space
	INT	SCREEN			; Write to screen
	POP	BX
	MOV	DI,0			; First register
	MOV	AX,0B10AH		; Read a DWORD
	INT	80H
	PUSH	ECX			; Save result
	MOV	EAX,ECX
	CALL	HEXD			; Write to screen
	MOV	SI,OFFSET PCRLF		; Point to end of line
	CALL	PRINT			; Print it
;
	MOV	SI,OFFSET TPCI_DEVIC	; Point to prompt
	CALL	PRINT			; Write to screen
	MOV	SI,OFFSET TPCI_HBIOS	; Point to host prompt
	CALL	PRINT			; Print it
	POP	ECX			; Device/vendor
	MOV	EDX,ECX			; Copy
	SHR	ECX,10H			; Move into place
	MOV	SI,0			; Device index
	MOV	AX,0B102H		; Find device
	INT	1AH			; Do it
	MOV	AX,BX			; Get result
	CALL	HEXW			; Print it
	MOV	SI,OFFSET PCRLF		; Point to end of line
	CALL	PRINT			; Print it
	MOV	SI,OFFSET TPCI_TBIOS	; Point to test prompt
	CALL	PRINT			; Print it
	POP	ECX			; Device/vendor
	MOV	EDX,ECX			; Copy
	SHR	ECX,10H			; Move into place
	MOV	SI,0			; Device index
	MOV	AX,0B102H		; Find device
	INT	80H			; Do it
	MOV	AX,BX			; Get result
	CALL	HEXW			; Print it
	MOV	SI,OFFSET PCRLF		; Point to end of line
	CALL	PRINT			; Print it
;
;	Read bytes from config registers into memory at 9000:0
;
	MOV	AX,9000H		; Where COMMAND.COM exists
	MOV	ES,AX			; Set dest segment
	XOR	DI,DI			; Offset zero
	XOR	BX,BX			; Device/function
	MOV	CX,100H			; Bytes to read
RDPCI0:	PUSH	CX			; Save count
	MOV	AX,0B108H		; Read byte
	INT	80H
	MOV	AL,CL			; Get returned byte
	STOSB				; Save byte
	POP	CX			; Restore count
	LOOP	RDPCI0			; Read all bytes
;
;	Write bytes from memory at 9000:0 into configuration registers
;
	MOV	AX,9000H
	MOV	DS,AX			; DS = our data
	XOR	SI,SI			; Offset zero
	XOR	BX,BX			; Device/function
	MOV	CX,100H			; Bytes to write
WRPCI0:	PUSH	CX			; Save count
	MOV	DI,SI			; Register offset
	LODSB				; Get byte
	MOV	CL,AL			; Byte to write
	MOV	AX,0B10BH		; Write byte
	INT	80H
	POP	CX			; Restore count
	LOOP	WRPCI0			; Write all bytes
;
;	Read bytes from configuration registers into memory at 9000:0100
;
	MOV	AX,9000H + (100H SHR 4)	; Next buffer
	MOV	ES,AX			; Set segment
	XOR	DI,DI			; Offset of next buffer
	XOR	BX,BX			; Device/function
	MOV	CX,100H			; Bytes to read
RDPCI1:	PUSH	CX			; Save count
	MOV	AX,0B108H		; Read byte
	INT	80H
	MOV	AL,CL			; Get returned byte
	STOSB				; Save byte
	POP	CX			; Restore count
	LOOP	RDPCI1			; Read all bytes
;
;	Compare the bytes at 9000:0000 and 9000:0100
;
	MOV	AX,9000H
	MOV	DS,AX			; DS = our data
	XOR	SI,SI
	XOR	DI,DI
	MOV	CX,100H
	REPZ	CMPSB
	MOV	AX,CS
	MOV	DS,AX
	MOV	ES,AX
	OR	CX,CX
	JZ	SHORT CMPOK
	MOV	SI,OFFSET TPCI_CMPFL
	CALL	PRINT
	MOV	AX,100H
	SUB	AX,CX
	CALL	HEXW
	MOV	SI,OFFSET PCRLF		; Point to end of line
	CALL	PRINT			; Print it
CMPOK:	MOV	AX,4C00H		; MS-DOS exit function
	INT	21H			; Return to MS-DOS
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for testing the whole BIOS in a MS-DOS environment.
;
	MOV	AX,CS			; Get current code segment
	MOV	DS,AX			; Set DS
	MOV	AX,TEST_SEG		; Relocation to here
	MOV	ES,AX			; Set ES
	MOV	SI,OFFSET START		; Code to copy
	MOV	DI,SI			; Where to put it
	MOV	CX,(0FFFFH - (OFFSET START - PSEG)) SHR 1
	REP	MOVSW			; Copy the code
	DB	0EAH			; Jump
	DW	OFFSET RSTART		; Offset
	DW	TEST_SEG		; Segment
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
	MOV	AL,0FFH			; Get a mask
	OUT	MSK_CTL0,AL		; Turn OFF first controller
	OUT	MSK_CTL1,AL		; Turn OFF second controller
	MOV	AL,NMI_OFF		; Turn off NMI
	OUT	CMOS_INDEX,AL		; CMOS index register
	MOV	AL,8FH			; NMI offset
	OUT	CMOS_INDEX,AL		; Set index
	IN	AL,CMOS_DATA		; Read/reset NMI
	MOV	AL,NMI_OFF		; Base port
	OUT	CMOS_INDEX,AL		; Set index
	IN	AL,CMOS_DATA		; Read/reset
	MOV	EAX,CR0			; Don't check, just reset
	AND	EAX,(NOT PROTECTED)	; Reset the bit
	OR	EAX,CR0_CD		; Disable caching
	MOV	CR0,EAX			; Make sure it's real
	MOV	EAX,3
	MOV	TR5,EAX			; Flush the cache
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	See if this was a return from protected mode. This is hardly a
;	'return'. The CPU reset (crashed) because the protected-mode
;	descriptor, passed by the caller, was bad.
;
	MOV	AL,(SHUTDOWN OR NMI_OFF) ; Reason for shutdown
	OUT	CMOS_INDEX,AL		; Set CMOS index register
	SLOW_IO				; Let it take effect
	IN	AL,CMOS_DATA		; Get result
	SLOW_IO
	MOV	AH,AL			; Save for later
	MOV	AL,NMI_OFF		; Keep parity OFF
	OUT	CMOS_INDEX,AL		; Reset index
	SLOW_IO
	CMP	AH,PROT_RET		; Return from protected mode?
	JNZ	SHORT NOPROT		; No
	MOV	AL,SHUTDOWN		; Yes
	OUT	CMOS_INDEX,AL		; Turns on NMI
	SLOW_IO				; Let it take effect
	XOR	AL,AL			; Zero
	OUT	CMOS_DATA,AL		; Clear shutdown byte
	SLOW_IO				; Let it take effect
	OUT	CMOS_INDEX,AL		; Set index back to zero
	SLOW_IO
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Disable A20. This could be on as a result of the crash in
;	protected mode.
;
	MOV	AL,0FH
	OUT	84H,AL			; Fast A<20> may be on.
	MOV	AL,0
	OUT	85H,AL
	SET_A20_OFF			; Execute MACRO
	MOV	EAX,CR0			; Don't check, just reset
	AND	EAX,(NOT CR0_CD)	; Enable caching
	MOV	CR0,EAX			; Make sure it'son
	MOV	AX,BIOS_SEG		; Get BIOS data segment
	MOV	DS,AX			; Set data segment
	MOV	ES,AX			; Set extra segment
	XOR	BX,BX			; Set index
	MOV	SP,WORD PTR [BX+POST_RES_LOW]	; Get user's saved stack
	MOV	SS,WORD PTR [BX+POST_RES_HIGH]	; Get user's stack segment
	POP	AX			; Interrupt control mask
	OUT	MSK_CTL1,AL		; Restore second controller mask
	MOV	AL,AH			; Recover for first controller
	OUT	MSK_CTL0,AL		; Restore first controller mask
	POP	DS			; Restore user's data segment
	POP	ES			; Restore user's extra segment
	POPA				; Restore GP registers
	STI				; Allow interrupts
	STC				; Show an error from virtual mode
	MOV	AH,2			; Exception interrupt error
	RETF	2			; Return to caller with flags
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now set up the hardware-specific machine if this was a cold
;	boot from a reset. Otherwise, set up the generic PC/AT equipment
;	and boot the machine.
;
SUBTTL	Machine-specific configuration and setup code.
PAGE	,132
NOPROT:	INCLUDE	SC520.ASM		; Machine-specific startup
SUBTTL	PC/AT compatible configuration, setup, and boot code.
PAGE	,132
RSTART:	CLI				; Possible warm boot, keep off
	MOV	AX,BIOS_SEG		; Get BIOS data segment
	MOV	DS,AX			; Set data segment
	MOV	ES,AX			; Set extra segment
	MOV	FS,AX			; Reload these too
	MOV	GS,AX
	MOV	AX,STACK_SEG		; Startup stack
	MOV	SS,AX			; Set stack segment
	MOV	SP,BIOS_STACK		; Set stack pointer
	MOV	EAX,CR0			; Get control register
	OR	EAX,CR0_CD		; Disable caching
	MOV	CR0,EAX			; Set control register
	MOV	BP,OFFSET NEXT		; We may not have a writable stack
	CALL	SET_MEMORY		; Set up memory
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now we call various functions to set up the important parts
;	of the machine.
;
NEXT:	CALL	DISAB_PCI		; Disable PCI bridge
	CALL	CLEAR_BIOS		; Clear BIOS ROM space
	CALL	SET_CMOS		; Set CMOS defaults
	CALL	MOTOR_OFF		; Turn off FDC motor
	CALL	SET_INT_TABLE		; Set table to point to dummy
	CALL	SET_INT_CONTROL		; Set up interrupt control
	CALL	SET_INIT_INTERS		; Set initial interrupts
	CALL	SET_TIMER0		; Set timer channel 0
	CALL	INIT_PCI_RES		; Initialize PCI resources
	CALL	SET_PRIVATE		; Set our private vector
	STI				; Enable interrupts, but not NMI
IFNDEF	COM				; Not while testing in DOS
	CALL	SET_PCI			; Set up the PCI bus
ENDIF
	CALL	SET_KEYBOARD		; Set the keyboard
	CALL	SET_RS232C		; Set RS-232C port(s)
	CALL	SET_RS232C_INT		; Set RS-232C interrupts
	CALL	SET_SCREEN		; Set up screen BIOS
	XOR	AL,AL			; Zero
	OUT	CMOS_INDEX,AL		; Turn on NMI
	CALL	CLS			; Clear the screen
	CALL	SIGNON			; Write sign-on message
	CALL	CHK_SCR_MEM		; Check screen memory
	CALL	CHK_CSUM		; Check checksum
	CALL	SET_TOOLS		; Set Interrupt 0x15 stuff
	CALL	SET_PRINTER		; Set printer port(s)
	CALL	SET_TIME		; Set the time
	CALL	SET_TIME_SERV		; Set CMOS time-service vector
	CALL	CHK_CMOS		; Verify CMOS and second controller
	CALL	CHK_TIMER0		; Check timer-0 wait routine
	CALL	SET_TIMER2		; Set up timer channel 2
	CALL	SHO_PCI			; Show the PCI bus
	CALL	SET_FLOPPY		; Set up floppy drive
	CALL	SET_HARDISK		; Set up any hard disk
	CALL	SET_EQUIP		; Set equipment-list vector
	CALL	SET_MEMSIZE		; Set memory size vector
	CALL	TEST_VMB		; Test virtual block move
	CALL	CHK_EXTR		; Check extended RAM
	CALL	SET_COMPAT_DATA		; Set compatibility data
	CALL	SET_MODULES		; Find BIOS modules and install
	CALL	SET_BOOT		; Set the boot vector
	CALL	CHK_INP			; Check for user input
	CALL	CLEAR_RAM		; Clean up work-space
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now try to boot the machine.
;
	MOV	SI,OFFSET BOOTM		; Point to message
	CALL	PRINT			; Print the message
	INT	BOOT			; Boot the machine
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Boot failed. Print a message and wait.
;
	MOV	SI,OFFSET FAILM		; Point to message
WPRMPT:	CALL	PRINT			; Print the message
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Wait on a failure with keyboard echo and everything running
;	to help look for the problem.
;
WAITF:	MOV	AH,0			; Wait-for-key function
	INT	KBD_INP			; Keyboard user interrupt
	CMP	AL,'C'-64		; Control-C ?
	JNZ	SHORT NOCTLC		; No, just echo right now
	JMP	RSTART			; Try again.
NOCTLC:	CMP	AL,'A'-64		; Control-A?
	JNZ	NOCTLA			; Nope
	MOV	SI,OFFSET PCRLF		; Write end of line
	CALL	PRINT			; Print to screen
DOTEST:	CALL	TEST_RAM		; Yes, test RAM
	MOV	AH,1			; Check keyboard
	INT	KBD_INP			; Anything there?
	JZ	DOTEST			; No
	MOV	AH,0			; Get key
	INT	KBD_INP			; Keyboard user interrupt
	MOV	SI,OFFSET USRINFO	; Prompt
	JMP	SHORT WPRMPT		; Yes, abort
NOCTLA:	XOR	BX,BX			; Set page zero
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Video BIOS
	SET_TIMER 2			; Set timer
TIM:	WAIT_TIMER TIM			; Wait for timeout
	JNZ	SHORT TIM		; Wait for change
	JMP	SHORT WAITF		; Loop to keyboard again
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Random number generator. A pseudo-random integer returns in
;	register AX.
;
RND:	PUSH	DS
	MOV	AX,RND_SEG		; Where we keep the seed
	MOV	DS,AX			; Set segment
	MOV	AX,WORD PTR DS:[0]	; Get last result
	ROR	AX,3			; Interesting sequence
	ADD	AX,MAGIC		; Conv number
	MOV	WORD PTR DS:[0],AX	; Save for next time
	POP	DS
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This tests low RAM until an error or until aborted.
TEST_RAM:
	PUSH	DS
	PUSH	ES
	MOV	SI,OFFSET CKLOWR	; Point to prompt
	CALL	PRINT			; Write to screen
	CLD				; Forwards
	CALL	RND			; Get a random number
;
;	This just reads and writes bytes.
;
	MOV	DX,1000H
TSTR0:	MOV	ES,DX			; Set extra segment
	XOR	DI,DI			; Offset zero
	MOV	CX,0FFFFH		; Number of bytes
	REP	STOSB			; Write these bytes
	ADD	DX,1000H		; Ready next segment
	CMP	DX,0A000H		; Check limits
	JC	SHORT TSTR0		; Continue
	MOV	DX,1000H
TSTR1:	MOV	CX,0FFFFH		; Max bytes
	MOV	DS,DX			; Set data segment
	XOR	DI,DI			; Offset zero
XORT1:	XOR	BYTE PTR [DI],AL	; Flip bits
	JNZ	TSTRF			; Bad RAM
	LEA	DI,[DI+1]		; Ready next
	LOOP	XORT1			; Continue
	ADD	DX,1000H		; Ready next segment
	CMP	DX,0A000H		; Check limits
	JC	SHORT TSTR1		; Continue
;
;	This reads and writes words.
;
	MOV	DX,1000H		; Start at first 64k again
TSTR2:	MOV	ES,DX			; Set extra segment
	XOR	DI,DI			; Offset zero
	MOV	CX,08000H		; Number of words
	REP	STOSW			; Write words
	ADD	DX,1000H		; Ready next segment
	CMP	DX,0A000H		; Check limits
	JC	SHORT TSTR2		; Continue
;
	MOV	DX,1000H		; Start at first 64k again
TSTR3:	MOV	CX,8000H		; Max words
	MOV	DS,DX			; Set extra segment
	XOR	DI,DI			; Offset zero
XORT3:	XOR	WORD PTR [DI],AX	; Flip bits
	JNZ	TSTRF			; Failed
	LEA	DI,[DI+2]		; Ready next word
	LOOP	XORT3			; Continue
	ADD	DX,1000H		; Ready next segment
	CMP	DX,0A000H		; Check limits
	JC	SHORT TSTR3		; Continue
;
;	This reads and write words at odd addresses.
;
	MOV	DX,1000H		; Start at first 64k again
TSTR4:	MOV	ES,DX			; Set extra segment
	MOV	DI,1			; Offset 1 (odd address) 
	MOV	CX,07FFFH		; Number of words
	REP	STOSW			; Write words
	ADD	DX,1000H		; Ready next segment
	CMP	DX,0A000H		; Check limits
	JC	SHORT TSTR4		; Continue
;
	MOV	DX,1000H		; Start at first 64k again
TSTR5:	MOV	CX,7FFFH		; Number of words
	MOV	DS,DX			; Set data segment
	MOV	DI,1			; Offset 1 (odd address)
XORT5:	XOR	WORD PTR [DI],AX	; Flip bits
	JNZ	SHORT TSTRF		; Test failed
	LEA	DI,[DI+2]		; Ready next
	LOOP	XORT5			; Continue
	ADD	DX,1000H		; Ready next segment
	CMP	DX,0A000H		; Check limits
	JC	SHORT TSTR5		; Continue
;
	MOV	SI,OFFSET CHKTOK	; Show it's okay
	CALL	PRINT			; Write to screen
	JMP	SHORT TSTRX		; Exit okay
;
TSTRF:	PUSH	AX			; Save what it should be
	PUSH	WORD PTR [DI]		; Save what it is
	PUSH	DI			; Save location
	MOV	SI,OFFSET CKFAIL	; Point to prompt
	CALL	PRINT			; Print message
	MOV	SI,OFFSET CKADDR	; Point to prompt
	CALL	PRINT			; Write to screen
	MOV	AX,DS			; Get failing segment
	CALL	HEXW			; Write word
	MOV	AL,':'			; Insert delimiter
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	XOR	BX,BX			; Page zero
	INT	SCREEN			; Write to screen
	POP	AX			; Failing location offset
	CALL	HEXW			; Write offset
	MOV	SI,OFFSET DATWAS	; Text to print
	CALL	PRINT			; Write screen
	POP	AX			; Restore data
	CALL	HEXW			; Write to screen
	MOV	SI,OFFSET DASHLD	; Text to print
	CALL	PRINT			; Write to screen
	POP	AX			; Word that it should be
	CALL	HEXW			; Write hex ASCII
	MOV	AH,0			; Wait for keyboard
	INT	KBD_INP			; Call function
;
TSTRX:	MOV	SI,OFFSET PCRLF		; End of line
	CALL	PRINT			; Write to screen
	POP	ES
	POP	DS
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This clears all RAM from 1000:0000 to 9000:FFFF. It is called
;	just before boot to clean up all the work-space.
;
CLEAR_RAM:
	PUSH	ES
	MOV	DX,1000H		; Segment to start
	XOR	AX,AX			; Zero word
	CLD				; Forwards
CLR64K:	MOV	ES,DX			; Set segment
	XOR	DI,DI			; Offset zero
	MOV	CX,8000H		; 64k words
	REP	STOSW			; Clear RAM
	ADD	DX,1000H		; Next segment
	CMP	DX,0A000H		; Check limits
	JC	CLR64K			; Continue
	POP	ES
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes the signon message at the top of the screen.
;
ALIGN	4
SIGNON:	MOV	CX,CENTER_LOGO		; To center text
	CALL	SPACE			; Space it over
	MOV	SI,OFFSET SIGN0		; Point to signon message
	CALL	PRINT			; Write signon message
	MOV	CX,CENTER_CPYR		; To center text
	CALL	SPACE			; Space it over
	MOV	SI,OFFSET SIGN1		; Point to signon message
	JMP	SHORT PRINT		; Write message, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This prints the message, addressed by CS:SI to the current
;	cursor location. Note: All BIOS messages are in the code
;	segment. We don't own any RAM for them.
;
ALIGN	4
PRINT:	PUSH	DS			; Save segment
	MOV	BX,CS			; Get code segment
	MOV	DS,BX			; DS = CS
	XOR	BX,BX			; Screen page zero
PRNT0:	LODSB				; Get byte
	OR	AL,AL			; See if done
	JZ	SHORT PREXT		; All done
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Screen BIOS
	JMP	SHORT PRNT0		; Continue
PREXT:	POP	DS			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This prints the number of spaces in CX to the output screen.
;	It is used to center or indent text.
;
ALIGN	4
SPACE:	XOR	BX,BX
SPACE0:	MOV	AX,0E20H		; Dumb terminal mode and space
	INT	SCREEN			; BIOS video output
	LOOP	SPACE0
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This checks for possible keyboard input and, if an escape or tab
;	key was entered, prompts for parameter changes.
;
ALIGN	4
CHK_INP:
	SET_TIMER 10H
CHKWT:	MOV	AH,1			; Check for key-input
	INT	KBD_INP			; Call our BIOS
	JNZ	SHORT GOTKEY		; Got key
	WAIT_TIMER CHKWT
	RET				; No input
;
GOTKEY:	MOV	AH,0			; Get key
	INT	KBD_INP			; Call our BIOS
	CMP	AL,ESC_KEY		; Look for escape-key
	JZ	SHORT IS_ESC		; It is an escape key
	CMP	AL,TAB_KEY		; For very dumb terminals
	JZ	SHORT IS_ESC		; Pretend it's an escape key
	JMP	SHORT CHK_INP		; Start all over again
;
IS_ESC:	MOV	AH,1			; Check for key-input
	INT	KBD_INP			; Call our BIOS
	JZ	SHORT EATEN		; No more keys
	MOV	AH,0			; Get key
	INT	KBD_INP			; Call our BIOS
	JMP	SHORT IS_ESC		; Clear kerboard buffer
EATEN:	MOV	SI,OFFSET PCTIME	; Point to time-prompt
	CALL	PRINT			; Output to screen
	CALL	SHOW_TIME		; Display
	MOV	SI,OFFSET PNOCHG	; Point to prompt
	CALL	PRINT			; Output to screen
	MOV	BX,SETUP_BUF		; Steal buffer space
	MOV	CX,SETUP_BUF_LEN	; 16 bytes
	CALL	CONIN			; Get response
	CMP	BYTE PTR [BX],CR	; Check for nothing
	JZ	SHORT USER1		; Get next response
	MOV	SI,BX			; Where the data is
	CALL	ASC2BIN			; Get hours
	CALL	BIN2BCD			; Convert to BCD
	MOV	CH,AL			; Hours in CH for function call
	CALL	ASC2BIN			; Get minutes
	CALL	BIN2BCD			; Convert to BCD
	MOV	CL,AL			; Hours in CL for function call
	CALL	ASC2BIN			; Get seconds
	CALL	BIN2BCD			; Convert to BCD
	MOV	DH,AL			; Seconds in DH for function call
	XOR	DL,DL			; Say no DST
	MOV	AH,3			; Set realtime clock
	INT	TOD			; Set the clock
	JMP	SHORT EATEN		; Try again
;
USER1:	MOV	SI,OFFSET PCDATE	; Point to date-prompt
	CALL	PRINT			; Output to screen
	CALL	SHOW_DATE		; Display
	MOV	SI,OFFSET PNOCHG	; Point to prompt
	CALL	PRINT			; Output to screen
	MOV	BX,SETUP_BUF		; Steal buffer space
	MOV	CX,SETUP_BUF_LEN	; 16 bytes
	CALL	CONIN			; Get response
	CMP	BYTE PTR [BX],CR	; Check for nothing
	JZ	SHORT USER2		; Get next response
	MOV	SI,BX			; Where the data is
	CALL	ASC2BIN			; Get month
	CALL	BIN2BCD			; Convert to BCD
	MOV	DH,AL			; Month in DH for function call
	CALL	ASC2BIN			; Get day
	CALL	BIN2BCD			; Convert to BCD
	MOV	DL,AL			; Day in DL for function call
	CALL	ASC2BIN			; Year
	PUSH	DX			; Save what we've done
	XOR	DX,DX			; Clear
	MOV	CX,100			; Need to get century
	DIV	CX			; AL = century, DL = year
	CALL	BIN2BCD			; Convert century to BCD
	MOV	CH,AL			; Set century
	MOV	AL,DL			; Get year
	CALL	BIN2BCD			; Convert to BCD
	MOV	CL,AL			; Save year
	POP	DX			; Restore
	MOV	AH,5			; Set realtime clock date
	INT	TOD			; Set the clock
	JMP	SHORT USER1		; Try again
USER2:	MOV	SI,OFFSET BOOTDEV	; Point to prompt
	CALL	PRINT			; Write to screen
	MOV	AL,CMOS_BOOTF		; Spare byte in CMOS
	CALL	GET_CMOS		; Get value
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	XOR	BX,BX			; Page zero
	INT	SCREEN			; Write to screen
	MOV	SI,OFFSET PNOCHG	; Point to prompt
	CALL	PRINT			; Output to screen
	MOV	BX,SETUP_BUF		; Steal buffer space
	MOV	CX,SETUP_BUF_LEN	; 16 bytes
	CALL	CONIN			; Get response
	CMP	BYTE PTR [BX],CR	; Check for nothing
	JZ	SHORT USER3		; Get next response
	MOV	AL,[BX]			; Get byte from buffer
	AND	AL,95			; Make upper-case
	MOV	AH,AL			; Put into high
	MOV	AL,CMOS_BOOTF		; Offset to write
	CALL	PUT_CMOS		; Write data to CMOS
	JMP	SHORT USER2		; Try again
USER3:	MOV	SI,OFFSET ANYKEY	; Point to prompt
	CALL	PRINT			; Write to screen
	XOR	AX,AX			; Wait for key function
	INT	KBD_INP			; Wait for input
	CALL	GET_CMOS_CHKSUM		; Get new checksum
	MOV	DX,AX			; Save
	MOV	AH,AL			; Low byte of checksum
	MOV	AL,2FH			; Offset of low byte
	CALL	PUT_CMOS		; Write to chip
	MOV	AH,DH			; High byte of checksum
	MOV	AL,2EH			; Offset of high byte
	CALL	PUT_CMOS		; Write to CMOS chip
	JMP	RSTART			; Do it over again
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets buffered keyboard input. It handles back-space
;	correction and carriage return termination only. It is
;	used to get user input for setting parameters. Note that
;	this is not designed to properly back-space over a tab!
;
;	Upon entry, DS:BX must point to a buffer (not checked), CX
;	must contain the buffer length.
;
;	Upon exit, a null-terminated string will be present in the
;	buffer and CX will contain the byte-count, which includes
;	the null.
;
ALIGN	4
CONIN:	PUSH	BX
	MOV	DX,BX			; Save first location
CONIN0:	XOR	AX,AX			; Wait for character
	INT	KBD_INP			; Call kerboard BIOS
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Write any/all characters to the screen including a back-space
;	unless we are at the beginning of the buffer. If we are at
;	the beginning, we write a space, then a back-space.
;
	CMP	BX,DX			; Already at the start?
	JNZ	SHORT NOSTRT		; No.
	CMP	AL,BS			; Is it a back-space?
	JNZ	SHORT NOSTRT		; No.
	PUSH	BX			; Save registers
	PUSH	CX
	XOR	BX,BX			; Page zero
	MOV	AL,' '			; Print a space
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Video BIOS
	MOV	AL,BS			; Print a back-space
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Video BIOS
	POP	CX			; Restore registers
	POP	BX
	JMP	SHORT CONIN0		; Continue
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Not at the start, write the character, even if it's a back-space.
;
NOSTRT:	PUSH	AX			; Save registers destroyed
	PUSH	BX
	PUSH	CX
	XOR	BX,BX			; Page zero
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Video BIOS
	POP	CX			; Restore registers
	POP	BX
	POP	AX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now see if it was a back-space. A back-space at the beginning of
;	the buffer was already handled so we don't back-space over a
;	prompt nor bump the pointer out-of-bounds.
;
	CMP	AL,BS			; Is it a back-space
	JNZ	SHORT NOBS		; Nope
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay. It was a back-space. We need to write a blank over the
;	already back-spaced cursor, then back-space again. Then we
;	will fix the buffer location and byte-count.
;
	PUSH	BX			; Save registers we destroy
	PUSH	CX
	XOR	BX,BX			; Page zero
	MOV	AL,' '			; Print a space
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Video BIOS
	MOV	AL,BS			; Write back-space again
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Video BIOS
	POP	CX			; Restore registers
	POP	BX
	DEC	BX			; Bump pointer
	INC	CX			; Bump key-count
	JMP	SHORT CONIN0		; Continue
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	No back-space. Put the character in the buffer, adjust the pointer,
;	then check the buffer limit (byte-count).
;
NOBS:	MOV	[BX],AL			; Save the key
	INC	BX			; Ready next
	DEC	CX			; Byte-count
	JZ	SHORT CONEX		; End of the buffer
	CMP	AL,CR			; End of the line?
	JZ	SHORT CONEX		; Yes
	JMP	SHORT CONIN0		; Get more
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	We reached the end-of-line or the end of the buffer. Fix up
;	the string and return.
;
CONEX:	MOV	BYTE PTR [BX],0		; Terminate the string
	MOV	CX,BX			; Present offset
	SUB	CX,BX			; CX = byte-count
	POP	BX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This displays the time as 'hh:mm:ss ' with a trailing space.
;	It uses a buffer allocated on the stack.
;
ALIGN	4
SHOW_TIME:
	PUSH	BP
	MOV	BP,SP			; Save stack pointer
	SUB	SP,10H			; 16 bytes for a buffer
	MOV	DI,SP			; ES:DI = buffer
	ADD	DI,2			; Leave 1 word
	PUSH	DS			; Save segment
	PUSH	ES			; Save extra also
	MOV	AX,SS			; Current stack
	MOV	ES,AX
	MOV	DS,AX			; ES = ES= SS
	MOV	BX,DI			; Save this location
	MOV	AH,2			; Read time
	INT	TOD			; Get time from CMOS
	MOV	AL,CH			; Get hours
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AL,':'			; Delimiter
	STOSB				; Into output string
	MOV	AL,CL			; Get minutes
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AL,':'			; Delimiter
	STOSB				; Into output string
	MOV	AL,DH			; Get seconds
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AX,0020H		; Delimiter
	STOSW				; Into output string
	MOV	SI,BX			; DS:SI = buffer beginning
PTIME:	LODSB				; Get buffer byte
	OR	AL,AL			; Check for end
	JZ	SHORT PTIMEX		; End of string
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	XOR	BX,BX			; Page zero
	INT	SCREEN			; Write to output
	JMP	SHORT PTIME		; Continue
PTIMEX:	POP	ES			; Restore extra seg
	POP	DS			; Restore segment
	MOV	SP,BP			; Restore, collapse local buffer
	POP	BP
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This displays the date as 'mm-dd-yyyy ' with a trailing space.
;	It uses a buffer allocated on the stack.
;
ALIGN	4
SHOW_DATE:
	PUSH	BP
	MOV	BP,SP			; Save stack pointer
	SUB	SP,10H			; 16 bytes for a buffer
	MOV	DI,SP			; ES:DI = buffer
	ADD	DI,2			; Leave 1 word
	PUSH	DS			; Save segment
	PUSH	ES			; Save extra also
	MOV	AX,SS			; Current stack
	MOV	ES,AX
	MOV	DS,AX			; ES = ES= SS
	MOV	BX,DI			; Save this location
	MOV	AH,4			; Read date
	INT	TOD			; Get time from CMOS
	MOV	AL,DH			; Get month
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AL,'-'			; Delimiter
	STOSB				; Into output string
	MOV	AL,DL			; Get day
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AL,'-'			; Delimiter
	STOSB				; Into output string
	MOV	AL,CH			; Get century
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AL,CL			; Get year
	CVT_ASC				; Convert
	STOSW				; Put into output string
	MOV	AX,0020H		; Delimiter
	STOSW				; Into output string
	MOV	SI,BX			; DS:SI = buffer beginning
PDATE:	LODSB				; Get buffer byte
	OR	AL,AL			; Check for end
	JZ	SHORT PDATEX		; End of string
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	XOR	BX,BX			; Page zero
	INT	SCREEN			; Write to output
	JMP	SHORT PDATE		; Continue
PDATEX:	POP	ES			; Restore extra seg
	POP	DS			; Restore segment
	MOV	SP,BP			; Restore, collapse local buffer
	POP	BP
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Given an input string addressed by DS:SI, the routine returns
;	the unsigned binary equivalent of the ASCII digits in AX. The
;	conversion ends at the first character that is not an ASCII
;	digit. Upon exit, DS:SI remains one character past the terminating
;	digit. This allows multiple calls to extract various fields from
;	delimited strings like "mm/dd/yyyy" or "hh:mm:ss". The actual
;	de-limiter is not important.
;
ALIGN	4
ASC2BIN:
	PUSH	CX			; Save registers we destroy
	PUSH	DX
	XOR	DX,DX			; Use as an accumulator
ASC2IN:	LODSB				; Get character
	SUB	AL,'0'			; Remove ASCII bias
	JC	SHORT ASC2EX		; Conversion done
	CMP	AL,10			; Check upper limit
	JNC	SHORT ASC2EX		; Conversion done
	XOR	AH,AH			; Clear
	MOV	CX,DX			; Save accumulated value
	SHL	DX,2			; Times 4
	ADD	DX,CX			; Times 5
	SHL	DX,1			; Times 10
	ADD	DX,AX			; Accumulate
	JMP	SHORT ASC2IN		; Continue
ASC2EX:	MOV	AX,DX			; Return accumulated value
	POP	DX			; Restore registers
	POP	CX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This checksums our code. This not only allows us to determine
;	if a PROM or NVRAM was written properly, but also to verify
;	that nothing is being written in the code-segment while this
;	is executing in RAM for testing.
ALIGN	4
CHK_CSUM:
	MOV	SI,OFFSET PCHKSUM	; Point to prompt
	CALL	PRINT			; Write to screen
	PUSH	DS			; Save segment
	MOV	AX,CS			; Get where we are
	MOV	DS,AX			; Set segment
	XOR	DX,DX			; Clear
	XOR	SI,SI			; Clear
	XOR	AX,AX			; Clear
	XOR	DI,DI
	XOR	CX,CX			; 64k of data
CHKCSM:	LODSB				; Get memory byte
	ADD	DX,AX			; Accumulate
	ADC	DI,0			; Take care of word overflow
	LOOP	CHKCSM			; Continue
	POP	DS			; Restore segment
	PUSH	DX			; Save low word
	PUSH	DI			; Save high Word
	MOV	SI,OFFSET CHKTOK	; Assume checksum was okay
	OR	DL,DL			; Low byte must be zero
	JZ	SHORT CHKRSP		; Good
	MOV	SI,OFFSET CKFAIL	; Bad checksum
CHKRSP:	CALL	PRINT			; Print results
	POP	AX			; Restore high word
	CALL	HEXW			; Print it
	POP	AX			; Restore low word
	CALL	HEXW			; Print it
	MOV	SI,OFFSET PCRLF		; Write end-of-line
	JMP	PRINT			; Write to screen, implied return
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This checks extended RAM using our 32-bit extended mode
;	function.
;
ALIGN	4
CHK_EXTR:
	MOV	SI,OFFSET CHKEXTR	; Point to prompt
	CALL	PRINT			; Write to screen
	MOV	AX,EXT_RAM_SIZE		; Function code
	INT	PRIVATE			; Execute private interrupt
	PUSH	AX			; Save at least status
	JNC	SHORT CHK_OK		; It didn't fail
	MOV	SI,OFFSET CKFAIL	; Show it failed
	CALL	PRINT			; Write 'failed' to screen
	POP	AX			; Get status
	CALL	HEXW			; Write word to screen
	MOV	SI,OFFSET PCRLF		; End of line
	JMP	PRINT			; Write to screen, implied return
CHK_OK:	PUSH	DX			; Save high word also
	MOV	SI,OFFSET CHKTOK	; Pount to prompt
	CALL	PRINT			; Write to screen
	POP	DX			; Restore high word
	POP	AX			; Restore low word
	PUSH	AX			; Save low word
	PUSH	DX			; Save high word
	CALL	BIN32ASC		; Print to screen
	MOV	SI,OFFSET PBYTES	; Point to 'bytes'
	POP	DX			; Restore high word
	POP	AX			; Restore low word
	SHL	EDX,10H			; Into high
	AND	EAX,0FFFFH		; Clear high junk
	OR	EDX,EAX			; Make DWORD
	SHR	EDX,10			; Make kbytes
	MOV	AH,DL			; Low kbytes
	MOV	AL,17H			; Low kbytes CMOS location
	CALL	PUT_CMOS		; Put into CMOS
	MOV	AH,DH			; High kbytes
	MOV	AL,18H			; High kbytes CMOS location
	CALL	PUT_CMOS		; Put into CMOS
	CALL	GET_CMOS_CHKSUM		; Get checksum
	MOV	DL,AL			; Save low checksum
	MOV	AL,2EH			; High checksum location
	CALL	PUT_CMOS		; Write high checksum
	MOV	AH,DL			; Saved low checksum
	MOV	AL,2FH			; Save low CMOS offset
	CALL	PUT_CMOS		; Put into CMOS
	JMP	PRINT			; Write to screen, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets CMOS data. Upon entry, the offset of the data is in
;	register AL. Upon exit, the retrieved data is in register AL.
;
ALIGN	4
GET_CMOS:
	PUSHF				; Save whatever
	CLI				; No interrupts
	OUT	CMOS_INDEX,AL		; Set index register
	SLOW_IO				; Wait for it to take effect
	IN	AL,CMOS_DATA		; Get result
	SLOW_IO				; Slow down I/O
	MOV	AH,AL			; Save
	XOR	AL,AL			; Zero
	OUT	CMOS_INDEX,AL		; Set back to zero
	SLOW_IO				; Wait to take effect
	MOV	AL,AH			; Restore output value
	POPF				; Restore
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This puts CMOS data. Upon entry, the offset of the data is in
;	register AL, and the data to be written is in AH.
;
ALIGN	4
PUT_CMOS:
	PUSHF				; Save whatever
	CLI				; No interrupts
	OUT	CMOS_INDEX,AL		; Set index register
	SLOW_IO				; Wait for it to take effect
	MOV	AL,AH			; Data to be written
	OUT	CMOS_DATA,AL		; Write CMOS data
	SLOW_IO				; Slow down I/O
	XOR	AL,AL			; Zero
	OUT	CMOS_INDEX,AL		; Set back to zero
	SLOW_IO				; Wait to take effect
	POPF				; Restore
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets the current CMOS checksum. It is not clear how much
;	of the CMOS data are actually supposed to be checksummed.
;	I find that several computers checksum everything from offset
;	10h up to offset 3Fh. Others only checksum to the checksum low
;	word at offset 2eh. IBM says the checksum is from 10H - 2DH.
;
;	So, I will take a hack at it. The checksum returns in register
;	AX. No other registers are destroyed.
;
ALIGN	4
GET_CMOS_CHKSUM:
	PUSHF					; Save flags
	CLI					; No interrupts
	PUSH	BX
	PUSH	CX
	MOV	CX,(2EH - 10H)			; Length to checksum
	XOR	BX,BX				; Accumulator
	MOV	AH,10H				; CMOS offset to start
CHKSUM:	MOV	AL,AH				; Get CMOS offset
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; I/O delay
	IN	AL,CMOS_DATA			; Get value
	SLOW_IO					; I/O delay
	ADD	BL,AL				; Accumulate
	ADC	BH,0				; Take care of CY
	INC	AH				; Ready next CMOS offset
	LOOP	CHKSUM				; Do all offsets
	MOV	AX,BX				; Return Checksum in AX
	POP	CX
	POP	BX
	POPF					; Restore interrupts/flags
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes a 16-bit integer to the screen with leading-zero
;	suppression. The input integer is register AX. Other registers
;	are preserved.
;
ALIGN	4
BIN2ASC:
	PUSH	BX
	PUSH	CX
	PUSH	DX
	MOV	CX,5			; 65536 (5 digits)
	MOV	BX,10			; Will divide by 10
BIN20:	XOR	DX,DX			; Clear high word
	DIV	BX			; Successive divisions
	PUSH	DX			; Save remainder
	LOOP	BIN20			; Continue
	MOV	CX,4			; First 4 digits have leading zeros
	XOR	BX,BX			; Screen page zero
	XOR	DX,DX			; Leading zero flag
BIN21:	POP	AX			; Get last/first
	OR	DX,AX			; Set / test flag
	JZ	SHORT BIN22		; leading zero, don't print
	ADD	AL,'0'			; ASCII bias
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Write to screen
BIN22:	LOOP	BIN21			; Do all.
	POP	AX			; Get last/first
	ADD	AL,'0'			; ASCII bias
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Write to screen
	POP	DX
	POP	CX
	POP	BX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes a 32-bit binary number to the screen with leading-zero
;	suppression. Upon input, the number is in DX:AX.
;
;	The alternate entry point gets the entire input value in EAX.
;
ALIGN	4
BIN32ASC:
	SHL	EDX,10H			; Move input high word into high
	OR	EAX,EDX			; Combine into EAX
BIN32EAX:
	MOV	EBX,10			; Divisor
	XOR	CX,CX			; Counter
DOMORE:	XOR	EDX,EDX			; Ready for divide
	DIV	EBX
	PUSH	DX			; Put result onto the stack
	INC	CX			; Count pushes
	OR	EAX,EAX			; Check for zero
	JNZ	SHORT DOMORE		; Continue
;
	XOR	BX,BX			; Screen page zero
	XOR	DX,DX			; Leading zero flag
	DEC	CX			; Last zero must be printed
	JZ	SHORT BIN25		; That's all there was
BIN23:	POP	AX			; Get last/first
	OR	DX,AX			; Set / test flag
	JZ	SHORT BIN24		; Leading zero, don't print
	ADD	AL,'0'			; ASCII bias
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Write to screen
BIN24:	LOOP	BIN23			; Do all.
BIN25:	POP	AX			; Get last/first
	ADD	AL,'0'			; ASCII bias
	MOV	AH,DUMB_TERM		; Dumb terminal mode
	INT	SCREEN			; Write to screen
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This shows the floppy disk activity. It is for testing.
;	Production code will have this conditional-assembled out.
;
IFDEF	TEST_FLOPPY
ALIGN	4
SHOW:	PUSHA
	PUSH	DS
	PUSH	CS
	POP	DS				; DS = CS
	MOV	DX,AX				; Command code
	MOV	DI,0				; Use for status
	ADC	DI,0				; DI = 1 if a carry
	MOV	SI,OFFSET PRSLT			; Assume a result code
	OR	AX,AX				; See if we were right
	JNZ	SHORT SHOW1			; Yes
	MOV	AX,[BP-AX_SAV]			; Command code
	MOV	BL,AH				; BL = command code
	XOR	BH,BH				; Clear
	CMP	BX,19H				; Greatest command code
	JC	SHORT SHOW0			; It's okay
	MOV	BX,19H				; Substitute
SHOW0:	SHL	BX,1				; Table has words
	ADD	BX,OFFSET SHOTAB		; Start of table
	MOV	SI,[BX]				; Get string
SHOW1:	CALL	PRINT				; Print function code
	MOV	SI,OFFSET PDRIV			; Drive
	CALL	PRINT				; Print sectors
	MOV	AL,BYTE PTR [BP-DX_SAV]		; Get drive number
	MOV	SI,OFFSET PCRLF			; End
	CALL	HEXB				; Write sectors
	CALL	PRINT
	MOV	SI,OFFSET PSECT			; Sectors
	CALL	PRINT				; Print sectors
	MOV	AL,BYTE PTR [BP-AX_SAV]		; Get sectors
	CALL	HEXB				; Write sectors
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT
	MOV	SI,OFFSET PTART			; Starting sector
	CALL	PRINT
	MOV	AL,BYTE PTR [BP-CX_SAV]		; Get starting sector
	CALL	HEXB				; Write to output
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT
	MOV	SI,OFFSET PTRAK			; Point to cylinder
	CALL	PRINT
	MOV	AX,[BP-CX_SAV]			; Get sector/track
	MOV	AL,AH				; Select track
	CALL	HEXB				; Write to screen
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT				; Print prompt
	MOV	SI,OFFSET PHEAD			; Point to head
	CALL	PRINT				; Print prompt
	MOV	AX,[BP-DX_SAV]			; Head/drive
	MOV	AL,AH				; Get head
	CALL	HEXB				; Write to screen
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT				; Print prompt
	MOV	SI, OFFSET PADDR		; Point to address string
	CALL	PRINT				; Print prompt
	MOV	AX,[BP-ES_SAV]			; Get segment
	OR	DX,DX				; Input params?
	JZ	SHORT SEGOK			; Yes
	MOV	AX,[BP-DMA_PAG]			; Bo, get DTA page
	SHL	AX,0CH				; Convert to segment
SEGOK:	CALL	HEXW				; Print the segment
	MOV	AL,':'				; ASCII
	MOV	AH,DUMB_TERM			; Dumb terminal
	XOR	BX,BX				; Page zero
	INT	SCREEN				; Video BIOS
	MOV	AX,[BP-BX_SAV]			; Get offset
	OR	DX,DX				; Input params?
	JZ	SHORT OFFOK			; Yes
	MOV	AX,[BP-DMA_OFF]			; No, get DTA offset
OFFOK:	CALL	HEXW				; Print the segment
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT
	OR	DX,DX				; Is it incomming?
	JZ	SHORT SHOW5			; Yes.
	MOV	SI,OFFSET PSTEX			; Point to status string
	CALL	PRINT				; Write to screen
	OR	DI,DI				; Check for CY flag
	JZ	SHORT SHOW3			; Not set
	MOV	SI, OFFSET PCARY		; Point to carry string
	CALL	PRINT				; Print to screen
	MOV	AX,[BP-AX_SAV]			; Show what we return
	CALL	HEXW				; Print to screen
	MOV	SI, OFFSET PSPACE		; Print spaces
	CALL	PRINT
SHOW3:	PUSH	DS				; Save segment
	MOV	AX,BIOS_SEG			; Point to BIOS segment
	MOV	DS,AX				; DS = BIOS
	MOV	CX,7				; Possible status bytes
	MOV	SI,NEC_STATUS			; Where status should be
SHOW4:	PUSH	CX				; Save count
	LODSB					; Get status byte
	CALL	HEXB				; Write byte to screen
	PUSH	SI				; Save index
	MOV	SI,OFFSET PSPACE		; Space to print
	CALL	PRINT				; Print it
	POP	SI				; Restore index
	POP	CX				; Restore count
	LOOP	SHOW4				; Do all bytes
	POP	DS				; Restore segment
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT
	MOV	SI,OFFSET PCRLF			; End
	CALL	PRINT
SHOW5:	POP	DS
	POPA
	RET
ENDIF
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This checksums a BIOS module. Upon entry, SI should point
;	to the BIOS module segment. It checks for the appropriate
;	signature, checksums the module, and upon return AX = 0 if
;	it's a good module.
;
ALIGN	4
CHKSUM_MOD:
	PUSH	SI			; Save input value
	PUSH	DS			; Save segment
	MOV	DS,SI			; Set segment
	XOR	SI,SI			; Offset zero
	OR	AX,0FFFFH		; Assume bad module
	CMP	WORD PTR [SI], 0AA55H	; See if it's a module
	JNZ	SHORT ISMOD2		; It's not a module
	MOV	CH,[SI+2]		; Get blocks to checksum
	SHL	CH,1			; 512-byte blocks
	XOR	CL,CL			; Low byte is zero
	XOR	AH,AH			; Clear high byte
ISMOD1:	LODSB				; Get memory byte
	ADD	AH,AL			; Accumulate
	LOOP	ISMOD1			; Do all the bytes
	MOV	AL,AH			; Extend
ISMOD2:	POP	DS			; Restore segment
	POP	SI			; Restore input value
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is machine-specific. There is no way in the world that I
;	can know what bits have to be set in what secret registers in
;	order to enable memory. Right now, I am going to clear all the
;	memory I can reach in each 64K. Since my stack is there,
;	I will play a trick to be able to get back. It is presumed
;	that when this routine is through, we have memory enabled.
;
ALIGN	4
SET_MEMORY:
	MOV	EAX,CR0		; Get control register
	AND	EAX,(NOT (CR0_CD OR CR0_NW))	; Enable caching
	MOV	CR0,EAX		; Set control register
	WBINVD			; Flush the cache
	POP	AX		; Throw away our return address
	MOV	BX,ES		; Save segment
	MOV	SI,CS		; Get our segment
	CMP	SI,SCR_BASE	; Check if we are in PROM
	JC	SHORT NOPROM	; No, leave alone
	MOV	SI,SCR_BASE	; Set lowest PROM (actually screen registers)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	First clear everything. The first write to newly powered-on RAM
;	might not actually succeed.
;
NOPROM:	XOR	DX,DX		; Cheap zero
	MOV	DI,DX		; Into DI
	MOV	AX,DX		; Zero
ZEROS:	MOV	ES,DX		; ES:SI = 0:0 to start
	MOV	CX,8000H	; Number of words in a segment
	REP	STOSW		; Clear memory
	ADD	DX,1000H	; The next segment address
	CMP	DX,SI		; Last segment
	JC	SHORT ZEROS	; Set pattern in the lower 640++
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay, now I will write a 'unique' pattern. This is not a
;	so-called memory test. We just want to find the extent of
;	writable RAM. I assume that the machine was designed with
;	unique decoding of RAM.
;
	XOR	DX,DX		; Cheap zero
	MOV	DI,DX		; Into DI
	MOV	AX,0AA55H	; Usual pattern
PATTRN:	MOV	ES,DX		; ES:SI = 0:0 to start
	MOV	CX,8000H	; Number of words in a segment
	REP	STOSW		; Put a pattern in memory
	ADD	DX,1000H	; The next segment address
	CMP	DX,SI		; Last segment
	JC	SHORT PATTRN	; Set pattern in the lower 640++
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now I see how far the RAM write succeeded. The pattern is still
;	in the AX register.
;
	XOR	DX,DX		; Cheap zero
	MOV	DI,DX		; Into DI
CHKPAT:	MOV	ES,DX		; ES:SI = 0:0 to start
	MOV	CX,8000H	; Number of words in a segment
	REPZ	SCASW		; Check the segment
	JNZ	SHORT GOTEND	; No more writable RAM
	ADD	DX,1000H	; The next segment address
	CMP	DX,SI		; Last segment
	JC	SHORT CHKPAT	; Check pattern in the lower 640++
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	At this time, DX is 0x1000 higher in value than the last writable
;	RAM. I save that value in SI, then clear the RAM again.
;
GOTEND:	MOV	SI,DX		; Last segment available
	XOR	DX,DX		; Cheap zero
	MOV	DI,DX		; Into DI
	MOV	AX,DX		; Use zeros to clear
CLEAR:	MOV	ES,DX		; ES:SI = 0:0 to start
	MOV	CX,8000H	; Number of words in a segment
	REP	STOSW		; Clear RAM
	ADD	DX,1000H	; The next segment address
	CMP	DX,SI		; Last segment
	JC	SHORT CLEAR	; Leave RAM zeroed
	SHR	SI,6		; Convert to Kbytes
	MOV	AX,BIOS_SEG	; AX = BIOS_DATA
	MOV	ES,AX		; ES = BIOS_DATA
	MOV	ES:[MEM_KBYTES],SI; Save kbytes of low memory
	MOV	ES,BX		; Restore segment
	PUSH	BP		; Return address was in BP
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This clears any space that BIOS ROM modules may use later in the
;	boot sequence. They are A000H, B000H, C000H, D000H, and E000H.
;	64k is cleared in each area. No checks are made for writable
;	ram (there may be an ISA screen-card at some address).
;
CLEAR_BIOS:
	PUSH	ES
	MOV	DX,0A000H		; First area to clear
	XOR	AX,AX			; Clear word
CLR_AR:	MOV	ES,DX			; Set segment
	XOR	DI,DI			; ES:DI = area to clear
	MOV	CX,8000H		; 64k in words
	REP	STOSW			; Clear
	ADD	DX,1000H		; Ready next
	CMP	DX,0F000H		; Check limits
	JC	SHORT CLR_AR		; Continue
	POP	ES
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up the interrupt controllers to their default settings.
;	This will be necessary so the timer-tick will work and I can get
;	a floppy drive to run.
;
ALIGN	4
SET_INT_CONTROL:
	PUSHF
	CLI
	MOV	AL,11H		; To enter initialization mode
	OUT	20H,AL		; Set initialization mode
	MOV	AL,8		; ICW2 Mapped 8 to 16
	OUT	21H,AL
	MOV	AL,4		; Slave on IRQ2
	OUT	21H,AL
	MOV	AL,1		; Master expects normal EOI
	OUT	21H,AL
	MOV	AL,0FFH		; OCW1
	OUT	21H,AL		; Set mask for all interrupts
	MOV	AL,11H		; Initialization mode
	OUT	0A0H,AL		; Second controller port
	MOV	AL,70H		; Mapped 70 to 77
	OUT	0A1H,AL		; Set
	MOV	AL,2		; Second is a slave
	OUT	0A1H,AL		; Set
	MOV	AL,1		; Slave expects normal EOI
	OUT	0A1H,AL		; Set
	MOV	AL,11111111B	; OCW1 (Mask all interrupts)
;                  ||||||||____ IRQ8
;                  |||||||_____ IRQ9
;                  ||||||______ IRQ10
;                  |||||_______ IRQ11
;                  ||||________ IRQ12
;                  |||_________ IRQ13
;                  ||__________ IRQ14
;                  |___________ IRQ15
;
	OUT	MSK_CTL1,AL	; Set mask
	MOV	AL,11111011B	; IRQ2 to second controller
;                  ||||||||____ IRQ0
;                  |||||||_____ IRQ1
;                  ||||||______ IRQ2
;                  |||||_______ IRQ3
;                  ||||________ IRQ4
;                  |||_________ IRQ5
;                  ||__________ IRQ6
;                  |___________ IRQ7
;
	OUT	MSK_CTL0,AL	; Set first controller mask
	MOV	AL,NOS_EOI	; Non Specific EOI
	OUT	ACK_CTL0,AL	; Reset first controller
	OUT	ACK_CTL1,AL	; Reset second controller
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up vectors in the interrupt table. All the interrupt
;	vectors, hardware and software, will point to one default routine
;	at the beginning. Vectors of 80H and greater are cleared to zero.
;
ALIGN	4
SET_INT_TABLE:
	PUSHF				; Save flags
	PUSH	ES			; Save segment
	CLI				; No interrupts
	XOR	AX,AX			; AX = 0
	MOV	ES,AX			; ES = 0
	MOV	DI,AX			; DI = 0
	MOV	CX,80H			; Max entries
SETINT:	MOV	AX,OFFSET DUMMY		; Get offset
	STOSW				; Put into interrupt table
	MOV	AX,CS			; Get segment
	STOSW				; Put into interrupt table
	LOOP	SETINT			; Do all vectors
	MOV	CX,100H			; Max entries
	XOR	AX,AX			; Rest of the table
	REP	STOSW			; From 80H, clear
	POP	ES			; Restore segment
	POPF				; Restore flags
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up timer-channel 0 so the BIOS timer-tick will run.
;	Interrupts are all masked OFF so we won't be interrupted.
;
ALIGN	4
SET_TIMER0:
	PUSHF
	CLI
	PUSH	ES			; Save segment
	MOV	AL,00110110B
;		   ||||||||_________ 16-bit binary
;		   |||||||__________ Mode 3
;		   ||||_____________ Write LSB/MSB
;		   ||_______________ Select timer 0

	OUT	(TIMER + 3), AL		; Write timer mode register
	SLOW_IO				; Slow down I/O
	XOR	AL,AL			; Cheap zero
	OUT	TIMER, AL		; LSB
	SLOW_IO				; Slow down I/O
	OUT	TIMER,AL		; MSB
	SLOW_IO				; Slow down I/O
	XOR	AX,AX			; Cheap zero
	MOV	ES,AX			; ES = 0
	MOV	DI,(IRQ0 SHL 2)		; 4 bytes per entry
	MOV	AX,OFFSET TIMER0_ISR	; Point to our timer
	STOSW				; Put in the offset
	MOV	AX,CS			; Get Segment
	STOSW				; Put in the segment
	MOV	DI,(USER_TICK SHL 2)	; 4 bytes per entry
	MOV	AX,OFFSET INT_1CH	; Point to our timer
	STOSW				; Put in the offset
	MOV	AX,CS			; Get Segment
	STOSW				; Put in the segment
	IN	AL,MSK_CTL0		; Interrupt mask
	SLOW_IO				; Slow down I/O
	AND	AL,11111110B		; Enable first interrupt
;                  ||||||||____ IRQ0
;                  |||||||_____ IRQ1
;                  ||||||______ IRQ2
;                  |||||_______ IRQ3
;                  ||||________ IRQ4
;                  |||_________ IRQ5
;                  ||__________ IRQ6
;                  |___________ IRQ7
;
	OUT	MSK_CTL0,AL		; Reset enable mask
	SLOW_IO				; Slow down I/O
	MOV	AL,NOS_EOI		; ACK
	OUT	ACK_CTL0,AL		; Out ACK register
	SLOW_IO				; Slow down I/O
	POP	ES
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets and tests timer-channel 2. It sets it to about 1 kHz.
;
ALIGN	4
SET_TIMER2:
	MOV	SI,OFFSET CHKTM2	; Point to prompt
	CALL	PRINT			; Write to screen
	MOV	SI,OFFSET CHKTOK	; Point to 'Okay'
	CLI
	MOV	AL,10110110B
;		   ||||||||_________ 16-bit binary
;		   |||||||__________ Mode 3
;		   ||||_____________ Write LSB/MSB
;		   ||_______________ Select timer 2

	OUT	(TIMER + 3), AL		; Write timer mode register
	SLOW_IO				; Slow down I/O
	MOV	AL,0A9H			; LSB for 1 kHz
	OUT	(TIMER + 2), AL		; LSB
	SLOW_IO				; Slow down I/O
	MOV	AL,04H			; MSB for 1kHz
	OUT	(TIMER + 2),AL		; MSB
	SLOW_IO				; Slow down I/O
	IN	AL,(TIMER + 2)		; Get timer count again
	SLOW_IO				; Slow down I/O
	MOV	AH,AL			; Save
	STI				; Allow interrupts
	SET_TIMER 1			; Set up wait time
CHKT2:	IN	AL,(TIMER + 2)		; Get next count
	SLOW_IO				; Slow down I/O
	CMP	AL,AH			; Check against last
	JNZ	SHORT T2OK		; It's okay
	WAIT_TIMER CHKT2		; Wait for a timeout
	MOV	SI,OFFSET CKFAIL	; Show bad
T2OK:	PUSH	AX			; Save test value
	CALL	PRINT			; Write results
	POP	AX			; Restore test value
	CALL	HEXW			; Write AX to the screen
	MOV	SI,OFFSET PCRLF		; End of line
	JMP	PRINT			; write result, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the screen for its minimal VGA mode with 80 columns
;	and 25 lines. It relies on a complete BIOS module being present
;	at the correct VGA BIOS location. Note that some assemblers
;	can't do "CALL FAR PTR 0C000H:3", so we fake it.
;
ALIGN	4
SET_SCREEN:
	PUSHF
	CLI				; No interrupts
	PUSH	DS			; Save segment registers
	PUSH	ES
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; ES = 0
	MOV	DI, (SCREEN SHL 2)	; Screen vector
	MOV	AX,OFFSET INT_10H	; Offset to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Segment to patch
	STOSW				; Patch segment
;
	MOV	SI,VGA_BIOS		; Screen BIOS should be here
	CALL	CHKSUM_MOD		; Checksum
	OR	AX,AX			; See if it's there
	JNZ	SHORT SMODE		; No screen-card
	MOV	AX,SS			; Some boards need DS=ES=SS, BP=SP
	MOV	ES,AX			; Restore ES
	MOV	DS,AX			; Restore DS
	MOV	BP,SP			; Set up for poor 'C' code
	PUSH	CS			; Current segment
	PUSH	OFFSET SCRDON		; Where to go later
	PUSH	SI			; Segment of screen card
	PUSH	3			; Offset to 'call'
	RETF				; Far 'return' to screen BIOS
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay, probably a PCI screen card. Let's see if we can find one.
;
SMODE:	XOR	SI,SI			; Will never be 1, help debug
	MOV	ECX,00030000H		; Display controller
	CALL	PCI_CLASS_CODE		; Try to find it
	JC	SCRDON			; Not found
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay we found a compatible controller. Now try to enable it.
;
	XOR	BH,BH			; Clear high byte
	SHR	BX,3			; Device ID into position
	MOV	SI,BX			; SI = device ID
	READ_PCI SI, PCI_CMD_STA	; Command/status register
	OR	EAX,00000007H		; Bus Master/Memory/IO
	WRITE_PCI SI, PCI_CMD_STA, EAX	; Enable the device
;
	MOV	EAX,0FFFFFFFEH		; All bits set except enable bit
	WRITE_PCI SI, PCI_ROMADDR, EAX	; Expansion ROM address
	READ_PCI  SI, PCI_ROMADDR	; Read the result
	CALL	ALLOC_PCI_ADDR		; Allocate a buffer
	OR	EAX,EAX			; Any expansion ROM?
	JZ	SCRDON			; Nothing there
	MOV	BX,CPY_ROM_SEG		; Where we build our table
	MOV	DS,BX			; DS = CPY_ROM_SEG
	MOV	DWORD PTR DS:[CPY_ROM_LEN],ECX
	MOV	DWORD PTR DS:[CPY_ROM_SRC],EAX
	MOV	DWORD PTR DS:[CPY_ROM_DST],(VGA_BIOS SHL 4)
	OR	EAX,00000001H		; Enable the decode
	WRITE_PCI SI, PCI_ROMADDR, EAX	; Enable expansion ROM
	MOV	AH,CPY_ROM_BIOS		; Function code
	INT	PRIVATE			; Do it.
	WRITE_PCI SI, PCI_ROMADDR, 0	; Disable expansion ROM
;
	MOV	SI,VGA_BIOS		; Screen BIOS should be here
	CALL	CHKSUM_MOD		; Checksum
	OR	AX,AX			; See if it's there
	JNZ	SHORT SCRDON		; No screen-card
	MOV	AX,BIOS_SEG		; Restore any segments
	MOV	DS,AX			; DS = BIOS_SEG
	MOV	ES,AX			; ES = BIOS_SEG
;
	DB	09AH			; Call 'far ptr '
	DW	03H			; Offset
	DW	VGA_BIOS		; Segment
;
SCRDON:	PARK_PCI			; Park on the bridge
	POP	ES			; Restore segment registers
	POP	DS
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This finds and relocates any SCSI or similar hard-disk module
;	that has an internal BIOS. The SCAN modules  routine installs it.
;
;	Let's see if we can find a PCI hard disk with a BIOS module.
;
ALIGN	4
SET_HARDISK:
	PUSHF
	CLI				; No interrupts
	PUSH	DS			; Save segment registers
	PUSH	ES
	XOR	SI,SI			; Will never be 1, help debug
	MOV	ECX,00010000H		; Mass storage controller
	CALL	PCI_CLASS_CODE		; Try to find it
	JC	DSKDON			; Not found
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay we found a compatible controller. Now try to enable it.
;
	XOR	BH,BH			; Clear high byte
	SHR	BX,3			; Move ID into position
	MOV	SI,BX			; SI = device
	READ_PCI SI, PCI_CMD_STA	; Command/status register
	OR	EAX,00000007H		; Bus Master/Memory/IO
	WRITE_PCI SI, PCI_CMD_STA, EAX	; Enable the device
	MOV	EAX,0FFFFFFFEH		; All bits set except enable bit
	WRITE_PCI SI, PCI_ROMADDR, EAX	; Expansion ROM address
	READ_PCI  SI, PCI_ROMADDR	; Read the result
	CALL	ALLOC_PCI_ADDR		; Allocate a buffer
	OR	EAX,EAX			; Any expansion ROM?
	JZ	SHORT DSKDON		; Nothing there
	MOV	BX,CPY_ROM_SEG		; Where we build our table
	MOV	DS,BX			; DS = CPY_ROM_SEG
	MOV	DWORD PTR DS:[CPY_ROM_LEN],ECX
	MOV	DWORD PTR DS:[CPY_ROM_SRC],EAX
	MOV	DWORD PTR DS:[CPY_ROM_DST],(DSK_BIOS SHL 4)
	OR	EAX,00000001H		; Enable the decode
	WRITE_PCI SI, PCI_ROMADDR, EAX	; Enable expansion ROM
	MOV	AH,CPY_ROM_BIOS		; Function code
	INT	PRIVATE			; Do it.
	WRITE_PCI SI, PCI_ROMADDR, 0	; Disable expansion ROM
DSKDON:	PARK_PCI			; Park on the bridge
	POP	ES			; Restore segment registers
	POP	DS
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This clears the screen and puts the cursor at the home position.
;
ALIGN	4
CLS:	PUSH	ES			; Save segment
	MOV	AX,SCR_SEGMENT
	MOV	ES,AX			; ES = Screen
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,0720H		; Blank word
	REP	STOSW			; Write to screen
	XOR	DX,DX			; Row/column = 0:0
	MOV	BX,DX			; Screen page zero
	MOV	AX,0200H		; Set cursor
	INT	SCREEN			; Screen BIOS
	POP	ES			; Restore
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
SUBTTL	PCI Control and configuration code
PAGE	,132
;
;	The PCI bus code is grouped into this area. PCI stands for
;	Peripheral Component Interconnect. It is a nice idea. However
;	many/most/all board vendors don't follow the specification.
;	For instance, it is an absolute rule that 32-bit decodes be
;	used on the 32-bit PCI bus. Most boards still have to be put
;	at special addresses so their aliases, caused by incomplete
;	decodes, don't interfere.
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This allocates PCI-bus memory-space. Upon entry EAX contains
;	the DWORD read from the PCI-bus memory address register. This
;	can be zero (no memory to reserve). Upon exit, EAX contains
;	the value to be written back to this register, including possibly
;	zero.
;
ALIGN	4
ALLOC_PCI_ADDR:
	PUSH	EBX				; Save nonvolatile registers
	PUSH	DS				; Save segment
	PUSH	PCI_OBJ_ALLOC			; Where we save last result
	POP	DS				; DS = PCI_OBJ_ALLOC
	TEST	EAX,1				; IO port?
	JZ	SHORT NOTIOP			; Not an IO port
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This allocates an I/O port for the device. They are allocated on
;	16-byte boundaries even if we don't need the space.
;
	AND	EAX,(NOT 3)			; Mask port/reserved bits
	JZ	SHORT NOALOC			; No space required
	MOV	ECX,EAX				; Start with first address bit
	NEG	ECX				; ECX = length required
	CMP	ECX,000000010H			; At least 16 bytes?
	JNC	SHORT WAS16			; Yes
	MOV	ECX,000000010H			; At least 16 bytes
WAS16:	MOV	EBX,ECX				; Save allocation length
	MOV	EAX,DWORD PTR DS:[PCI_PRT]	; Get present value
	ADD	EAX,ECX				; New offset value
	DEC	EAX				; Adjust for previous math
	NEG	ECX				; Create a mask
	AND	EAX,ECX				; N-byte boundary
	ADD	EBX,EAX				; EBX = for this allocation
	MOV	DWORD PTR DS:[PCI_PRT],EBX	; Ready next
	JMP	SHORT NOALOC			; We are done
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This allocates I/O memory-space. A defect (feature) of the
;	PCI addressing scheme is that a N-byte allocation must start
;	on a N-byte boundary. This means that a 4 megabyte allocation
;	for a PCI screen-card wastes nearly 4 megabytes of address-space.
;
NOTIOP:	AND	EAX,0FFFFFFF0H			; Clear non-address bits
	JZ	SHORT NOALOC			; No memory required
	MOV	ECX,EAX				; Start with first address bit
	NEG	ECX				; ECX = length required
	MOV	EBX,ECX				; Save allocation length
	MOV	EAX,DWORD PTR DS:[PCI_MEM]	; Get present value
	ADD	EAX,ECX				; New offset value
	DEC	EAX				; Adjust for previous math
	NEG	ECX				; Create a mask
	AND	EAX,ECX				; N-byte boundary
	ADD	EBX,EAX				; EBX = for this allocation
	MOV	DWORD PTR DS:[PCI_MEM],EBX	; Ready next
NOALOC:	NEG	ECX				; Return with length
	POP	DS				; Restore segment
	POP	EBX
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This initializes memory and port allocation space for PCI
;	device resource allocation.
;
ALIGN	4
INIT_PCI_RES:
	PUSH	DS			; Save segment
	MOV	AX,PCI_OBJ_ALLOC	; Segment address for storage
	MOV	DS,AX			; Set segment
	MOV	DWORD PTR DS:[PCI_MEM],(INSTALLED_MEM SHL 14H) ; For PCI
	MOV	DWORD PTR DS:[PCI_MEM], 0D0000000H
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Find the lowest port address (above PCI_OFF) that doesn't have
;	any devices responding at possible aliases.
;
	MOV	EDX,0000F002H		; Highest possible even port
FPRT:	SUB	DX,02H			; Keep on a WORD boundary
	IN	AX,DX			; See if anything there
	SLOW_IO				; Bus settle time for slow devices
	INC	AX			; 0FFFFH -> 0000H
	JNZ	SHORT PRTFND		; We found an alias
	CMP	DX,PCI_OFF		; Lowest port
	JNC	FPRT			; Continue
PRTFND:	ADD	DX,02H			; Last good port
	MOV	DWORD PTR DS:[PCI_PRT], EDX
;	MOV	DWORD PTR DS:[PCI_PRT], 0F000H
	POP	DS			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This allocates an interrupt for a device found on the PCI bus.
;	It became necessary to use a machine-specific table-driven
;	method because the hardware engineers just put anything anywhere,
;	wherever the PC traces would reach.
;	There was a rule: IRQ = (pin - 1 + slot %4) + 9. Forget it.
;	I haven't even found a PCI-Bus card that uses any pin number
;	other than 1.  This would put everything on IRQ9 which is,
;	in fact, what happens on my Compaq.
;
;
;                   --- Pin ---
;                 1    2    3    4
;
ALIGN	4
IRQTAB	DB	00H, 00H, 00H, 00H		; Bus device 0
	DB	09H, 0AH, 0BH, 0CH		; Bus device 1
	DB	0AH, 0BH, 0CH, 09H		; Bus device 2
	DB	0BH, 0CH, 09H, 0AH		; Bus device 3
	DB	0CH, 09H, 0AH, 0BH		; Bus device 4
	DB	09H, 0AH, 0BH, 0CH		; Bus device 5
	DB	0AH, 0BH, 0CH, 09H		; Bus device 6
	DB	0BH, 0CH, 09H, 0AH		; Bus device 7
	DB	0CH, 09H, 0AH, 0BH		; Bus device 8
	DB	09H, 0AH, 0BH, 0CH		; Bus device 9
	DB	0AH, 0BH, 0CH, 09H		; Bus device 10
	DB	0BH, 0CH, 09H, 0AH		; Bus device 11
	DB	0CH, 09H, 0AH, 0BH		; Bus device 12
	DB	09H, 0AH, 0BH, 0CH		; Bus device 13
	DB	0AH, 0BH, 0CH, 09H		; Bus device 14
	DB	0BH, 0CH, 09H, 0AH		; Bus device 15 Tundra chip
	DB	0AH, 0BH, 0CH, 09H		; Bus device 16 Ieee1394
	DB	09H, 0AH, 0BH, 0CH		; Bus device 17 AMD ethernet
	DB	0CH, 09H, 0AH, 0BH		; Bus device 18 Video
	DB	0BH, 0CH, 09H, 0AH		; Bus device 19
	DB	0CH, 09H, 0AH, 0BH		; Bus device 20
	DB	09H, 0AH, 0BH, 0CH		; Bus device 21
	DB	0AH, 0BH, 0CH, 09H		; Bus device 22
	DB	0BH, 0BH, 09H, 0AH		; Bus device 23
	DB	0CH, 09H, 0AH, 0BH		; Bus device 24
	DB	09H, 0AH, 0BH, 0CH		; Bus device 25
	DB	0AH, 0BH, 0CH, 09H		; Bus device 26
	DB	0BH, 0CH, 09H, 0AH		; Bus device 27
	DB	0CH, 09H, 0AH, 0BH		; Bus device 28
	DB	09H, 0AH, 0BH, 0CH		; Bus device 29
	DB	0AH, 0BH, 0CH, 09H		; Bus device 30
	DB	0BH, 0CH, 09H, 0AH		; Bus device 31
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Upon entry, this function has the PCI 'pin' number in register
;	AH, and the bus device number register SI. Upon exit, it returns
;	the 'pin' number in AH, and the IRQ number in AL. This is not
;	range-checked.
;
ALIGN	4
GET_IRQ:
	PUSH	SI			; Save register
	PUSH	DS			; Save segment
	MOV	BX,CS			; Where the table exists
	MOV	DS,BX			; DS = where IRQTAB is
	SHL	SI,2			; 4 bytes per table entry
	ADD	SI,OFFSET IRQTAB	; Start of the table
	MOV	BX,AX			; Get pin in BH
	DEC	BH			; Make zero-based
	SHR	BX,8			; Put into low byte
	ADD	SI,BX			; Sum pin offset
	MOV	AL,BYTE PTR [SI]	; Get the IRQ number
	POP	DS			; Restore segment
	POP	SI			; Restore register
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This attempts to set up the PCI bus. This does  a PCI
;	snoop, finding PnP devices, allocating memory, and enabling
;	them. Note that this machine has only one bridge so we
;	don't have to check for bridges behind bridges.
;
ALIGN	4
SET_PCI:
	XOR	SI,SI			; Start at device zero, the bridge.
	READ_PCI SI, 0			; See if the bridge is there
	INC	EAX			; 0xffffffff becomes zero
	JNZ	SHORT BRIDGE		; There is a bridge
	RET				; Forget it, no bridge
;
BRIDGE:	READ_PCI  SI, PCI_CMD_STA	; Get command/status register
	OR	AX,0107H		; Enable I/O, Memory, Master, B2B
	WRITE_PCI SI, PCI_CMD_STA, EAX	; Enable it.
;
	READ_PCI  SI, PCI_AMD		; AMD Specific PCI retry count
	MOV	AL,80H			; Recommended value
	WRITE_PCI SI, PCI_AMD, EAX	; Set timeout value
;
	READ_PCI  SI,PCI_LAT_CACHE	; Latency, etc.
	MOV	AX,4008H		; Latency / cache line size
	WRITE_PCI SI, PCI_LAT_CACHE,EAX	; Set it (if possible)
;
PCIDEV:	INC	SI			; Ready next device
	READ_PCI SI, 0			; Get device ID
	INC	EAX			; Anybody home? 0xffffffff becomes 0
	JZ	PCINXT			; Nope
;
	READ_PCI  SI, PCI_CMD_STA	; Get command/status register
	OR	AX,0107H		; Enable I/O, Memory, Master, B2B
	WRITE_PCI SI, PCI_CMD_STA, EAX	; Enable it.
;
	READ_PCI  SI,PCI_LAT_CACHE	; Latency, etc.
	MOV	AX,4008H		; Latency / cache line size
	WRITE_PCI SI, PCI_LAT_CACHE,EAX	; Set it (if possible)
;
	MOV	DI,PCI_BASE0		; First base address offset
PCIRES:	MOV	EAX,0FFFFFFFFH		; All bits set
	WRITE_PCI SI, DI, EAX		; Try to set all bits
	READ_PCI  SI, DI		; Get the results
	CALL	ALLOC_PCI_ADDR		; Allocate resources
	WRITE_PCI SI, DI, EAX		; Write base address (could be zero)
	INC	DI			; Ready next base address
	CMP	DI,PCI_CARDBUS		; Check limits
	JC	PCIRES			; Next possible address
;
	READ_PCI  SI, PCI_INTER		; Interrupt pin
	MOV	ECX,EAX			; Save whatever
	XOR	CX,CX			; Clear low word
	TEST	AX,0F00H		; Isolate interrupt pin
	JZ	SHORT PCINOI		; No interrupt required
	CALL	GET_IRQ			; Get IRQ value from table
PCINOI:	OR	EAX,ECX			; Could be writable
	WRITE_PCI SI, PCI_INTER, EAX	; Set IRQ (could be zero)
PCINXT:	CMP	SI,PCI_MAX		; End of devices?
	JC	PCIDEV			; Nope
	PARK_PCI			; Park on the bridge
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This disables all the PCI devices that it finds. This is necessary
;	for a 'warm boot' startup.
;
DISAB_PCI:
	MOV	SI,1			; Don't start with the bridge
CHKDEV:	READ_PCI SI, 0			; Check for a device
	INC	EAX			; -1 becomes zero
	JZ	SHORT NXTDEV		; Try the next one
	READ_PCI  SI, PCI_CMD_STA	; Read command / status
	XOR	AX,AX			; Clear enables
	WRITE_PCI SI, PCI_CMD_STA, EAX	; Disable
NXTDEV:	INC	SI			; Ready next
	CMP	SI,PCI_MAX		; Check limit
	JC	SHORT CHKDEV		; Continue
	READ_PCI 0, 0			; Read the bridge
	XOR	AX,AX			; Clear enables
	WRITE_PCI 0, 0, EAX		; Disable the bridge
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now just show what we have got.
;
ALIGN	4
SHO_PCI:
	MOV	SI,OFFSET CHKPCI	; Point to the prompt
	CALL	PRINT			; Write to screen
	XOR	SI,SI			; First device
PCI0:	READ_PCI SI, 0			; Device/Vendor
	CMP	EAX,0FFFFFFFFH		; See if anything
	JZ	SHORT PCI1		; No device
	CALL	HEXD			; Print hex DWORD
	MOV	AX,0E20H		; Dumb terminal + space
	INT	SCREEN			; Write to screen
PCI1:	INC	SI			; Ready next device
	CMP	SI,PCI_MAX		; Max PCI devices?
	JC	SHORT PCI0		; Do all devices
	PARK_PCI			; Park on the bridge
	MOV	SI,OFFSET PCRLF		; End of line
	JMP	PRINT			; Print, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function table for the PCI BIOS-16 function-codes.
;
PCI_FUNCTS	DW	INVALID_FUNC		; Function 0
		DW	PCI_BIOS_PRESENT	; Function 1
		DW	FIND_PCI_DEVICE		; Function 2
		DW	PCI_CLASS_CODE		; Function 3
		DW	INVALID_FUNC		; Function 4
		DW	INVALID_FUNC		; Function 5
		DW	GENERATE_SPECIAL_CYCLE	; Function 6
		DW	INVALID_FUNC		; Function 7
		DW	READ_CONFIG_BYTE	; Function 8
		DW	READ_CONFIG_WORD	; Function 9
		DW	READ_CONFIG_DWORD	; Function 0A
		DW	WRITE_CONFIG_BYTE	; Function 0B
		DW	WRITE_CONFIG_WORD	; Function 0C
		DW	WRITE_CONFIG_DWORD	; Function 0D
		DW	SHOW_IRQ		; Function 0E
		DW	SET_IRQ			; Function 0F
TOO_HIGH	LABEL	WORD
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the PCI BIOS-16 entry point. It is a 'near' procedure
;	called from the INT 1AH interface.
;
PCI_ENTRY:
	PUSH	AX				; Dummy place-holder
	PUSH	AX				; Save used registers
	PUSH	BX
	XOR	BH,BH				; Clear high byte
	MOV	BL,AL				; Function code
	SHL	BX,1				; Table has words
	ADD	BX,OFFSET PCI_FUNCTS		; Beginning of table
	CMP	BX,OFFSET TOO_HIGH		; Check limits
	JC	SHORT LIMOK			; It's okay
	MOV	BX,OFFSET PCI_FUNCTS		; Substitute invalid function
LIMOK:	MOV	AX,WORD PTR CS:[BX]		; Address of function
	MOV	BX,SP				; Current stack
	MOV	WORD PTR SS:[BX+4],AX		; Save address
	POP	BX				; Restore used registers
	POP	AX
	RET					; 'Return' to called function
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	INVALID_FUNC:
;		Return flag for invalid function call
;
;	ENTRY: NONE
;
;	EXIT: AH = FUNC_NOT_SUPPORTED
;	CF = 1
;
INVALID_FUNC:
	MOV	AH,FUNC_NOT_SUPPORTED
	STC					; Set error return
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
; PCI_BIOS_PRESENT:
;
;   This function allows the caller to determine both whether the PCI
;   BIOS interface function set is present and what current interface
;   version level is.
;
; ENTRY:  AH = PCI_FUNCTION_ID		(B1h)
;	  AL = PCI_BIOS_PRESENT 	(01h)
;
; EXIT:   EDX = " ICP"
;
;	   AH = Present Status, 00h = BIOS Present
;	   AL = Hardware mechanism
;	   BH = Interface level major version
;	   BL = Interface level minor version
;	   CL = Number of last PCI bus in the system
;	   CF = Present Status, set  = No BIOS Present,
;
PCI_BIOS_PRESENT:
	MOV	AX,0011H		; Mechanism #1 and special cycle
	MOV	BX,0201H		; BH = Major, BL = Minor version
	MOV	EDX,' ICP'		; PCI backwards
	XOR	CL,CL			; Maximum PCI bus, clears CY
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
; FIND_PCI_DEVICE:
;
;   Given the Vendor identifier and Device identifier in the DX and CX
;   registers respectively. this function must provide the corresponding Bus
;   Number and Slot Number, if such a device exists in the system.
;
; ENTRY : AH = PCI_FUNCTION_ID		(B1h)
;	  AL = FIND_PCI_DEVICE		(02H)
;	  CX = Device ID (0...65535)
;	  DX = Vendor ID (1...65534)
;	  SI = Device Index (0...N)
;
; EXIT	: BH = Bus Identification Number (0...255)
;	  BL = Device Number in upper 5 bits
;	       Function Number in lower 3 bits
;	  CF = Completion Status, set = error, cleared = success.
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : DEVICE_NOT_FOUND	(86h)
;			    BAD_DEVICE_ID	(82h)
;			    BAD_VENDOR_ID	(83h)
;
FIND_PCI_DEVICE:
		PUSH	AX
		PUSH	CX
		PUSH	DX
		PUSH	SI
		SHL	ECX,10H			; Device into high
		OR	CX,DX			; Vendor into low
SCAN_DEV:	READ_PCI SI, 0			; Read PCI device header
		CMP	ECX,EAX			; Did we find it?
		JZ	SHORT DEV_FOUND		; It was found
		INC	SI			; Ready next device
		CMP	SI,PCI_MAX		; End of devices?
		JC	SHORT SCAN_DEV		; Not yet
		POP	SI
		POP	DX			; Restore saved registers
		POP	CX
		POP	AX
		MOV	AH,DEVICE_NOT_FOUND	; Show it's not found
		STC				; Show error
		RET
;
DEV_FOUND:	MOV	BX,SI			; Device number
		SHL	BX,3			; Move into place
		POP	SI
		POP	DX			; Restore saved registers
		POP	CX
		POP	AX
		XOR	AL,AL			; Show good, clear CY
		RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;  PCI_CLASS_CODE:
;
;  Given the class-code as an entry parameter, this function must return
;  the device number, function number, and bus number if such a device
;  is found on the system.
;
;  ENTRY: AH  = PCI_FUNCTION_ID 	(B1h)
;	  AL  = FIND_PCI_CLASS_CODE	(03H)
;	  ECX = Class Code (in lower 3 bytes)
;	  SI = Device Index (0...N)
;
;  EXIT:  BH = Bus Identification Number (0...255)
;	  BL = Device Number in upper 5 bits
;	       Function Number in lower 3 bits
;	  CF = Completion Status, set = error, cleared = success.
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code :  DEVICE_NOT_FOUND	(86h)
;
PCI_CLASS_CODE:	PUSH	AX
		PUSH	CX
		PUSH	DX
		PUSH	SI
		SHL	ECX,8H			; Shift class code
SCAN_CLASS:	READ_PCI SI, PCI_CLASS		; Read the device
		MOV	BX,AX			; Save possible result
		AND	EAX,0FFFFFF00H		; Clear revision number
		CMP	EAX,ECX			; See if we found it
		JZ	SHORT CLA_FOUND		; It's found
		INC	SI			; Ready next device
		CMP	SI,PCI_MAX		; Check limits
		JC	SHORT SCAN_CLASS	; Ready next
;
		POP	SI
		POP	DX			; Restore saved registers
		POP	CX
		POP	AX
		MOV	AH,DEVICE_NOT_FOUND	; Show it's not found
		STC				; Show error
		RET
;
CLA_FOUND:	SHL	SI,3			; Move into place
		MOV	BX,SI			; Return result
		POP	SI
		POP	DX			; Restore saved registers
		POP	CX
		POP	AX
		XOR	AL,AL			; Show good, clear CY
		RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;  GENERATE_SPECIAL_CYCLE:
;
;  This function allows for generation of PCI special cycles. The current
;  bridge does not actually support special cycles, but they seem to
;  'work'.
;
;  ENTRY:  AH = PCI_FUNCTION_ID 	 (B1h)
;	   AL = GENERATE SPECIAL_CYCLE	 (06h)
;	   BH = Bus Number (0...255)
;	  EDX = Special Cycle Data
;
;  EXIT:  CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : FUNC_NOT_SUPPORTED	(81h)
;
GENERATE_SPECIAL_CYCLE:
	PUSH	AX
	PUSH	EDX				; Special cycle data
	PUSH	EDX				; Save again
	MOV	EAX,8000FF00H			; Device/function all 1s
	MOV	DX,PCI_INDEX			; PCI index port
	OUT	DX,EAX				; Set index port
	SLOW_IO
	MOV	DX,PCI_DATA			; Data port
	POP	EAX				; Special cycle data
	OUT	DX,EAX				; Write special-cycle data
	SLOW_IO
	POP	EDX
	POP	AX
	XOR	AH,AH				; Show good
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;    This checks the arguments supplied by the PCI BIOS caller.
;    We only support one PCI bus, i.e., we don't check for busses
;    behind other bridges, even if they exist.
;
CHK_ARGS:
	XOR	AH,AH			; Clear possible error code
	OR	BH,BH			; Check bus ID
	JZ	SHORT ID_OK		; Bus zero is okay
	MOV	AH,FUNC_NOT_SUPPORTED	; Show we support only 1 bus
	JMP	SHORT EXT_ARGS		; Return with error code
ID_OK:	CMP	DI,255			; Check register limits
	JBE	SHORT REG_OK		; It's okay
	MOV	AH,BAD_REGISTER_NUMBER	; Show bad value
	JMP	SHORT EXT_ARGS		; Return with error code
REG_OK:	MOV	AL,BL			; Get device number
	SHR	AL,3			; Move into low
	CMP	AL,PCI_MAX		; Check limits
	JBE	SHORT EXT_ARGS		; It's okay
	MOV	AH,BAD_SLOT_ID		; Wrong device
EXT_ARGS:
	OR	AH,AH			; Check for an error
	JZ	SHORT CHK_EXT		; No errors
	STC				; Show error
CHK_EXT:
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;   READ_CONFIG_BYTE:
;
;   This function allows reading individual bytes from the configuration
;   space of a specific device. In this code we only read whole DWORDS
;   (32 bits) from the PCI registers. Therefore, there is some code necessary
;   to select the current byte to be returned. Byte reads and writes from
;   the PCI registers do not work very well on the SC520 embedded bridge.
;
;   ENTRY: AH = PCI_FUNCTION_ID		(B1h)
;	  AL = READ_CONFIG_BYTE 	(08h)
;	  BH = Bus identification Number (0...255)
;	  BL = Device Number in upper 5 bits
;	       Function Number in lower 3 bits
;	  DI = Register Number (0...255)
;
; EXIT	: CL = Byte Read
;	  CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : BAD_BUS_ID		(84h)
;			    BAD_SLOT_ID 	(85h)
;
READ_CONFIG_BYTE:
	CALL	CHK_ARGS		; Check callers arguments
	JNC	R1OK			; They were okay
	RET
;
R1OK:	PUSH	AX
	PUSH	BX
	PUSH	DX
	PUSH	DI
;
	MOV	AX,DI			; Get register number offset
	AND	AX,3			; AX = shift value 0 -> 3
	SHL	AX,3			; Times 8, 8 bits/byte
	PUSH	AX			; Save shift value
	SHR	DI,2			; Divide by 4 (register number)
	XOR	EAX,EAX			; Start with zero
	MOV	AX,BX			; Bus, device, function
	SHL	EAX,8			; Move into place
	SHL	DI,2			; Back two bits
	OR	AX,DI			; Set DWORD register number
	OR	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Set to index register
	OUT	DX,EAX			; Start transaction
	SLOW_IO
	MOV	DX,PCI_DATA		; Set to data port
	IN	EAX,DX			; Get data
	SLOW_IO
	POP	CX			; Shift value
	SHR	EAX,CL			; Shift into place
	MOV	CL,AL			; Return with the byte
;
	POP	DI
	POP	DX
	POP	BX
	POP	AX
	XOR	AH,AH			; Show good
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;    READ_CONFIG_WORD:
;
;    This function allows reading individual words from the configuration
;    space of a specific device. The Register Number parameter must be a
;    multiple of two although we try to return the correct word even if
;    the input value is wrong. In this code we only reads whole DWORDS
;   (32 bits) from the PCI registers. Therefore, there is some code necessary
;   to select the current word to be returned. Word reads and writes from
;   the PCI registers do not work very well on the SC520 embedded bridge.
;
;   ENTRY: AH = PCI_FUNCTION_ID		(B1h)
;	   AL = READ_CONFIG_WORD 	(09h)
;	        READ_CONFIG_WORD32	(89h)
;          BH = Bus identification Number (0...255)
;	   BL = Device Number in upper 5 bits
;	        Function Number in lower 3 bits
;          DI = Register Number (0,2,4,...254)
;
;    EXIT: CX = Word Read
;          CF = Completion Status, set = error, reset = success
;
;          if CF clear --> AH = SUCCESSFUL	(00h)
;          if CF set,
;          AH = Error Code: BAD_BUS_ID		(84h)
;			    BAD_SLOT_ID 	(85h)
;			    BAD_REGISTER_NUMBER (87h)
;
READ_CONFIG_WORD:
	CALL	CHK_ARGS		; Check callers arguments
	JNC	R3OK			; They were okay
	RET
;
R3OK:	PUSH	AX
	PUSH	BX
	PUSH	DX
	PUSH	DI
	MOV	AX,DI			; Get register number offset
	AND	AX,3			; AX = shift value 0 -> 3
	SHL	AX,3			; Times 8. (16 bits / word)
	PUSH	AX			; Save shift value
	SHR	DI,2			; Divide by 4 (register number)
	XOR	EAX,EAX			; Start with zero
	MOV	AX,BX			; Bus, device, function
	SHL	EAX,8			; Move into place
	SHL	DI,2			; Back two bits
	OR	AX,DI			; Set DWORD register number
	OR	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Set to index register
	OUT	DX,EAX			; Start transaction
	SLOW_IO
	MOV	DX,PCI_DATA		; Set to data port
	IN	EAX,DX			; Get data
	SLOW_IO
	POP	CX			; Shift value
	SHR	EAX,CL			; Shift into place
	MOV	CX,AX			; Return with the word
;
	POP	DI
	POP	DX
	POP	BX
	POP	AX
	XOR	AH,AH			; Show good
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;    READ_CONFIG_DWORD:
;    This function allows reading individual DWORDs from the configuration
;    space of a specific device. The Register Number parameter must be a
;    multiple of four (i.e., bit 0 and 1 must be set to 0).
;
;    ENTRY: AH = PCI_FUNCTION_ID	(B1h)
;           AL = READ_CONFIG_DWORD	(0Ah)
;           BH = Bus identification Number (0...255)
;           BL = Device Number in upper 5 bits
;           Function Number in lower 3 bits
;           DI = Register Number (0,4,8,...252)
;
;    EXIT: ECX = Dword read
;          CF  = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : BAD_BUS_ID		(84h)
;			    BAD_SLOT_ID 	(85h)
;			    BAD_REGISTER_NUMBER (87h)
;
READ_CONFIG_DWORD:
	TEST	DI,3			; Lowest two bits clear
	JZ	SHORT R4OK		; Okay so far
	MOV	AH,BAD_REGISTER_NUMBER	; Show bad value
	STC				; Show error
	RET
;
R4OK:	CALL	CHK_ARGS		; Check callers arguments
	JNC	R5OK			; They were okay
	RET
;
R5OK:	PUSH	AX
	PUSH	BX
	PUSH	DX
	PUSH	DI
	MOV	AX,DI			; Get register number offset
	SHR	DI,2			; Divide by 4 (register number)
	XOR	EAX,EAX			; Start with zero
	MOV	AX,BX			; Bus, device, function
	SHL	EAX,8			; Move into place
	SHL	DI,2			; Back two bits
	OR	AX,DI			; Set DWORD register number
	OR	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Set to index register
	OUT	DX,EAX			; Start transaction
	SLOW_IO
	MOV	DX,PCI_DATA		; Set to data port
	IN	EAX,DX			; Get data
	SLOW_IO
	MOV	ECX,EAX			; Return with the DWORD
;
	POP	DI
	POP	DX
	POP	BX
	POP	AX
	XOR	AH,AH			; Show good
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;    WRITE_CONFIG_BYTE:
;
;    This function allows writing individual bytes from the configuration
;    space of a specific device. This function writes DWORDS only to
;    the PCI registers. Therefore, the appropriate DWORD that contains
;    the byte to be written is first read. It is then modified and the
;    nodified DWORD is written back.
;
;
;  ENTRY: AH = PCI_FUNCTION_ID		(B1h)
;	  AL = WRITE_CONFIG_BYTE	(0Bh)
;	  BH = Bus identification Number (0...255)
;	  BL = Device Number in upper 5 bits
;	       Function Number in lower 3 bits
;	  DI = Register Number (0...255)
;	  CL = Byte Value to Write
;
;  EXIT: CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : BAD_BUS_ID		(84h)
;			    BAD_SLOT_ID 	(85h)
;
WRITE_CONFIG_BYTE:
	CALL	CHK_ARGS		; Check callers arguments
	JNC	R6OK			; They were okay
	RET
;
R6OK:	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	DI
	PUSH	CX			; Save byte to write
;
	MOV	AX,DI			; Get register number offset
	AND	AX,3			; AX = shift value 0 -> 3
	SHL	AX,3			; Times 8, 8 bits/byte
	PUSH	AX			; Save shift value
	SHR	DI,2			; Divide by 4 (register number)
	XOR	EAX,EAX			; Start with zero
	MOV	AX,BX			; Bus, device, function
	SHL	EAX,8			; Move into place
	SHL	DI,2			; Back two bits
	OR	AX,DI			; Set DWORD register number
	OR	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Set to index register
	OUT	DX,EAX			; Start transaction
	SLOW_IO
	MOV	DX,PCI_DATA		; Set to data port
	IN	EAX,DX			; Get data
	SLOW_IO
	POP	CX			; Shift value
	MOV	EBX,0FFH		; Byte in lowest position
	SHL	EBX,CL			; Move into position
	NOT	EBX			; Make mask
	AND	EAX,EBX			; Mask off bits we will change
	POP	BX			; Byte to write
	AND	EBX,0FFH		; Keep only original value
	SHL	EBX,CL			; Move into position
	OR	EAX,EBX			; Modify complete
	OUT	DX,EAX			; Set new value
	SLOW_IO
;
	POP	DI
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	XOR	AH,AH			; Show good
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
; WRITE_CONFIG_WORD:
;
;   This function allows writing individual words from the configuration
;   space of a specific device. The Register Number parameter must be a
;   multiple of two (i.e., bit 0 must be set to 0). Since we read and
;   write only 32-bit values (DWORDS), the DWORD that contains the WORD
;   to be written is first read. The appropriate WORD is shifted into
;   place, then the entire DWORD is written back.
;
;  ENTRY: AH = PCI_FUNCTION_ID		(B1h)
;	  AL = WRITE_CONFIG_WORD	(0Ch)
;	  BH = Bus identification Number (0...255)
;	  BL = Device Number in upper 5 bits
;	       Function Number in lower 3 bits
;	  DI = Register Number (0,2,4,...254)
;	  CX = Word Value Write
;
;  EXIT: CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : BAD_BUS_ID		(84h)
;			    BAD_SLOT_ID 	(85h)
;			    BAD_REGISTER_NUMBER (87h)
;
WRITE_CONFIG_WORD:
	TEST	DI,1			; Needs to be even
	JZ	SHORT R7OK		; Okay so far
	MOV	AH,BAD_REGISTER_NUMBER	; Show bad value
	STC				; Show error
	RET
;
R7OK:	CALL	CHK_ARGS		; Check callers arguments
	JNC	R8OK			; They were okay
	RET
;
R8OK:	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	DI
	PUSH	CX			; Save word to write
	MOV	AX,DI			; Get register number offset
	AND	AX,3			; AX = shift value 0 -> 3
	SHL	AX,3			; Times 8. (16 bits / word)
	PUSH	AX			; Save shift value
	SHR	DI,2			; Divide by 4 (register number)
	XOR	EAX,EAX			; Start with zero
	MOV	AX,BX			; Bus, device, function
	SHL	EAX,8			; Move into place
	SHL	DI,2			; Back two bits
	OR	AX,DI			; Set DWORD register number
	OR	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Set to index register
	OUT	DX,EAX			; Start transaction
	SLOW_IO
	MOV	DX,PCI_DATA		; Set to data port
	IN	EAX,DX			; Get data
	SLOW_IO
	POP	CX			; Shift value
	MOV	EBX,0FFFFH		; Set mask bits
	SHL	EBX,CL			; Put mask into place
	NOT	EBX			; Make mask
	AND	EAX,EBX			; Clear bits
	POP	BX			; Word to write
	AND	EBX,0FFFFH		; Mask high bits
	SHL	EBX,CL			; Move into place
	OR	EAX,EBX			; Set new WORD
	OUT	DX,EAX			; Write new configuration
	SLOW_IO
;
	POP	DI
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	XOR	AH,AH			; Show good
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;   WRITE_CONFIG_DWORD:
;   This function allows writing individual DWORDs from the configuration
;   space of a specific device. The Register Number parameter must be a
;   multiple of four (i.e., bit 0 and 1 must be set to 0).
;
;  ENTRY: AH  = PCI_FUNCTION_ID 	(B1h)
;	  AL  = WRITE_CONFIG_DWORD	(0Dh)
;	  BH  = Bus identification Number (0...255)
;	  BL  = Device Number in upper 5 bits
;		Function Number in lower 3 bits
;	  DI  = Register Number (0,4,8,...252)
;	  ECX = Dword value to write
;
;  EXIT:  CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : BAD_BUS_ID		(84h)
;			    BAD_SLOT_ID 	(85h)
;			    BAD_REGISTER_NUMBER (87h)
;

WRITE_CONFIG_DWORD:
	TEST	DI,3			; Two low bits must be off
	JZ	SHORT R9OK		; Okay so far
	MOV	AH,BAD_REGISTER_NUMBER	; Show bad value
	STC				; Show error
	RET
;
R9OK:	CALL	CHK_ARGS		; Check callers arguments
	JNC	RAOK			; They were okay
	RET
;
RAOK:	PUSH	AX
	PUSH	BX
	PUSH	DX
	PUSH	DI
	MOV	AX,DI			; Get register number offset
	SHR	DI,2			; Divide by 4 (register number)
	XOR	EAX,EAX			; Start with zero
	MOV	AX,BX			; Bus, device, function
	SHL	EAX,8			; Move into place
	SHL	DI,2			; Back two bits
	OR	AX,DI			; Set DWORD register number
	OR	EAX,80000001H		; Set access bit
	MOV	DX,PCI_INDEX		; Set to index register
	OUT	DX,EAX			; Start transaction
	SLOW_IO
	MOV	DX,PCI_DATA		; Set to data port
	MOV	EAX,ECX			; DWORD to write
	OUT	DX,EAX			; Write data
	SLOW_IO
;
	POP	DI
	POP	DX
	POP	BX
	POP	AX
	XOR	AH,AH			; Show good
	RET

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
; SHOW_IRQ:
;	This function returns the PCI interrupt routing options available on
;   on the system motherboard and also the current state of what interrupts
;   are currently exclusively assigned to PCI
;
; ENTRY : AH  = PCI_FUNCTION_ID 	(B1h)
;	  AL  = SHOW_IRQ	(0Eh)
;	  BX  = Initialized to 0000H
;	  DS  = Segment or Selector for BIOS data
;	  ES  = Segment of Selector for routing table parameter
;	  DI  = Offset for routing table parameter
;
; EXIT	: CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : BUFFER_TO_SMALL    	(89h)
;			    FUNC_NOT_SUPPORTED	(81h)
;

SHOW_IRQ:
	MOV	AH,FUNC_NOT_SUPPORTED
	STC
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
; SET_IRQ:
;	This function causes the specified hardware interrupt(IRQ) to be
;   connected to the specified interrupt pin of a PCI device.
;
; ENTRY : AH  = PCI_FUNCTION_ID 	(B1h)
;	  AL  = SET_IRQ		(0Fh)
;	  BX  = BH - Bus number , BL - Device(upper 5 bits) and function
;	  CL  = IntPin parameter. Vaild values 0Ah..0DH
;	  CH  = Irq number. Valid values 0..0FH
;	  DS  = Segment of Selector for BIOS data
;
; EXIT	: CF = Completion Status, set = error, reset = success
;
;	  if CF clear --> AH = SUCCESSFUL	(00h)
;	  if CF set,
;	  AH = Error Code : SET_FAILED    	(88h)
;			    FUNC_NOT_SUPPORTED	(81h)
;
SET_IRQ:
	MOV	AH,FUNC_NOT_SUPPORTED
	STC
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=- End of PCI code -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
SUBTTL	Support code for compatible operation
PAGE	,132
;
;	This checks screen memory and leaves the screen as it was. We
;	need to detect a funky screen card.
;
ALIGN	4
CHK_SCR_MEM:
	PUSH	DS
	PUSH	ES			; Save extra segment
	MOV	SI,OFFSET CHKSCR	; Point to prompt
	CALL	PRINT			; Write message
	MOV	AX,SETUP_BUF		; Scratch space
	MOV	ES,AX			; ES = SETUP_BUF
	XOR	DI,DI			; ES:DI = SETUP_BUF
	MOV	AX,SCR_SEGMENT		; Screen segment
	MOV	DS,AX			; DS = SCR_SEGMENT
	MOV	SI,DI			; DS:SI = SCR_SEGMENT
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	REP	MOVSW			; Copy contents
;
	MOV	AX,SCR_SEGMENT
	MOV	ES,AX			; ES = Screen
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,0AA55H		; Test word
	REP	STOSW			; Write to screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	XOR	DI,DI			; ES:DI = screen
	REPZ	SCASW			; See if it got there
	JNZ	SHORT SCRF		; It failed
;
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,055AAH		; Test word
	REP	STOSW			; Write to screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	XOR	DI,DI			; ES:DI = screen
	REPZ	SCASW			; See if it got there
	JNZ	SHORT SCRF		; If failed
;
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,05555H		; Test word
	REP	STOSW			; Write to screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	XOR	DI,DI			; ES:DI = screen
	REPZ	SCASW			; See if it got there
	JNZ	SHORT SCRF		; If failed
;
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,0AAAAH		; Test word
	REP	STOSW			; Write to screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	XOR	DI,DI			; ES:DI = screen
	REPZ	SCASW			; See if it got there
	JNZ	SHORT SCRF		; If failed
;
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,0A5A5H		; Test word
	REP	STOSW			; Write to screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	XOR	DI,DI			; ES:DI = screen
	REPZ	SCASW			; See if it got there
	JNZ	SHORT SCRF		; If failed
;
	XOR	DI,DI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	MOV	AX,05A5AH		; Test word
	REP	STOSW			; Write to screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	XOR	DI,DI			; ES:DI = screen
	REPZ	SCASW			; See if it got there
;
SCRF:	PUSHF
	MOV	AX,SETUP_BUF		; Where we saved screen contents
	MOV	DS,AX			; DS = SETUP_BUF
	XOR	SI,SI			; DS:SI = SETUP_BUF
	MOV	DI,SI			; ES:DI = screen
	MOV	CX,SCR_REGEN_LEN	; All have at least this
	REP	MOVSW			; Copy screen contents back
	POPF
	POP	ES			; Restore segment
	POP	DS
	MOV	SI,OFFSET CHKTOK	; Assume it's okay
	JZ	SHORT SCROK		; It was okay
	MOV	SI,OFFSET CKFAIL	; Point to failed prompt
SCROK:	CALL	PRINT			; Write to screen
	MOV	SI,OFFSET PCRLF		; End of line
	JMP	PRINT			; Implied return
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Illegal OPCODE trap.
;
ALIGN	4
INT_06:
	MOV	SI,OFFSET BADOPC
	CALL	PRINT
	JMP	RETADR
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the keyboard busy loop.
;
ALIGN	4
BUSY:	STI
	NOP
	NOP
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up the keyboard for its default characteristics,
;	interrupt driven. Do we need a keyboard to boot? Yes, we need
;	to set the boot device and the CMOS date/time.
;
ALIGN	4
SET_KEYBOARD:
	MOV	SI,OFFSET PKBRD		; Point to prompt
	CALL	PRINT			; Write to screen
	CLI
	MOV	BX,KBD_HEAD		; Get keyboard head pointer
	MOV	WORD PTR [BX],KBD_BUF	; Start of keyboard buffer
	MOV	BX,KBD_TAIL		; Get keyboard tail pointer
	MOV	WORD PTR [BX],KBD_BUF	; Start of keyboard buffer
	MOV	BX,KBD_LED		; Keyboard LED bits
	MOV	BYTE PTR [BX],0		; Clear LED status
	PUSH	ES			; Save segment
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; Set segment to 0
	MOV	DI,(KBD_INP SHL 2)	; User keyboard functions
	MOV	AX, OFFSET INT_16H	; Routine to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Get segment
	STOSW				; Patch segment
	MOV	DI,(IRQ1 SHL 2)		; Hardware interrupt
	MOV	AX, OFFSET KBD_ISR	; ISR to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Get segment
	STOSW				; Patch segment
	MOV	DI,(KBD_BZY SHL 2)	; Keyboard busy loop
	MOV	AX,OFFSET BUSY		; Address of procedure
	STOSW				; Set offset
	MOV	AX,CS			; Current segment
	STOSW				; Set segment
	IN	AL,MSK_CTL0		; Interrupt mask
	SLOW_IO				; Slow down I/O
	AND	AL,11111101B		; Enable second interrupt
;                  ||||||||____ IRQ0
;                  |||||||_____ IRQ1
;                  ||||||______ IRQ2
;                  |||||_______ IRQ3
;                  ||||________ IRQ4
;                  |||_________ IRQ5
;                  ||__________ IRQ6
;                  |___________ IRQ7
;
	OUT	MSK_CTL0,AL		; Reset enable mask
	SLOW_IO				; Slow down I/O
	IN	AL,PORT_A		; Prime the pump
	SLOW_IO				; Slow down I/O
	IN	AL,STATUS_PORT		; Get this too
	SLOW_IO				; Slow down I/O
	MOV	AL,NOS_EOI		; ACK
	OUT	ACK_CTL0,AL		; Out ACK register
	SLOW_IO				; Slow down I/O
	POP	ES			; Restore segment
	STI
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set the repeat rate and enable the keyboard.
;	Flash the keyboard LEDs and leave the keyboard enabled.
;
	PUSH	ES			; Save segment
	MOV	CX,CS			; Get code segment
	MOV	ES,CX			; DS = CS
	XOR	BX,BX			; Offset zero
	MOV	AL,60H			; Set to take raw command
	OUT	STATUS_PORT,AL		; Becomes the command port
	MOV	AL,41H			; AT keyboard and interrupt enable
	CALL	SEND_8042		; Send to normal command port
	MOV	AL,KBD_SPEED		; Send keyboard speed command
	CALL	SEND_8042		; Send to keyboard controller
	MOV	AL,KBD_REPEAT		; Send the repeat rate
	CALL	SEND_8042		; Send to the controller
	MOV	AL,KBD_ENABLE		; Enable the keyboard
	CALL	SEND_8042		; Send to the controller
	MOV	CX,4			; Pattern count
PATRN:	PUSH	CX			; Save pattern count
	MOV	SI,OFFSET PATTERN	; Led Pattern
	MOV	CX,PATTERN_LEN		; Length of the pattern
DANCE:	MOV	AX, WORD PTR ES:[SI]	; Get bit pattern
	INC	SI			; Ready next
	CALL	DO_LEDS			; Light the leds
	SET_TIMER 01H			; One timer-tick
PAUSE:	WAIT_TIMER PAUSE		; Wait for a timer-tick
	LOOP	DANCE			; Do all the LEDs
	POP	CX			; Restore pattern count
	LOOP	PATRN
	POP	ES
	MOV	AH,1			; Get status, no wait
	INT	KBD_INP
	PUSH	AX			; Save result
	MOV	SI,OFFSET CHKTOK	; Point to okay prompt
	CALL	PRINT			; Write message
	POP	AX			; Keyboard status
	CALL	HEXW			; Write response
	MOV	SI,OFFSET PCRLF		; End of line
	JMP	PRINT			; Write output, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up the boot vector.
;
ALIGN	4
SET_BOOT:
	PUSH	ES			; Save segment
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; Set segment to 0
	MOV	DI,(BOOT SHL 2)		; Boot vector
	MOV	AX, OFFSET INT_19H	; Routine to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Get segment
	STOSW				; Patch segment
	POP	ES			; Restore segment
	MOV	AL,SHUTDOWN		; CMOS shutdown byte
	MOV	AH,04H			; Show POST end, bootsystem
	JMP	PUT_CMOS		; Write to CMOS (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set up the BIOS time-of day services for software interrupt
;	1AH.
;
ALIGN	4
SET_TIME_SERV:
	PUSH	ES			; Save segment
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; Set segment to 0
	MOV	DI,(TOD SHL 2)		; BIOS time-of-day services
	MOV	AX, OFFSET INT_1AH	; Routine to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Get segment
	STOSW				; Patch segment
	POP	ES			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the equipment-list vector.
;
ALIGN	4
SET_EQUIP:
	PUSH	ES			; Save segment
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; Set segment to 0
	MOV	DI,(EQUIP SHL 2)	; Equipment list services
	MOV	AX, OFFSET INT_11H	; Routine to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Get segment
	STOSW				; Patch segment
	POP	ES			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the memory size vector.
;
ALIGN	4
SET_MEMSIZE:
	PUSH	ES			; Save segment
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; Set segment to 0
	MOV	DI,(MEMSIZE SHL 2)	; Memory size service
	MOV	AX, OFFSET INT_12H	; Routine to patch
	STOSW				; Patch offset
	MOV	AX,CS			; Get segment
	STOSW				; Patch segment
	POP	ES			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up certain defaults in the CMOS RAM. This is
;	necessary because many operating systems read CMOS directly,
;	which they shouldn't.
;
SET_CMOS:
	PUSHF
	CLI
	XOR	AH,AH			; Zero, soft reset
	MOV	AL,0FH			; Shutdown byte
	CALL	PUT_CMOS		; Write to CMOS
	MOV	AH,40H			; One 1.44 Mb drive
	MOV	AL,10H			; CMOS offset
	CALL	PUT_CMOS		; Write to CMOS
	MOV	AH,0FFH			; No HD
	MOV	AL,11H			; CMOS offset
	CALL	PUT_CMOS		; No fixed disk 0
	MOV	AH,00H			; Zero
	MOV	AL,12H			; CMOS offset
	CALL	PUT_CMOS		; No fixed disk 1
	MOV	AH,0FFH			; No HD
	MOV	AL,19H			; CMOS offset
	CALL	PUT_CMOS		; No fixed disk 1
	MOV	AH,0FFH			; No HD
	MOV	AL,1AH			; CMOS offset
	CALL	PUT_CMOS		; No fixed disk 1
	MOV	AH,00100011B		; Equipment
;		   ||||||||______ Diskette (zero if so!)
;		   |||||||_______ Math coprocessor
;		   ||||||________ Reserved
;		   ||||__________ Primary display, color, 80 columns
;		   ||____________ 0 = 1 diskette drive
	MOV	AL,14H			; Offset of equipment list
	CALL	PUT_CMOS		; Write to CMOS
	CALL	GET_CMOS_CHKSUM		; Return checksum in AX
	MOV	BL,AL			; Save low byte
	MOV	AL,2EH			; Offset of high byte
	CALL	PUT_CMOS		; Write CMOS
	MOV	AH,BL			; Get low byte in AH
	MOV	AL,2FH			; Offset of low byte
	CALL	PUT_CMOS		; Write CMOS
	POPF
	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up the floppy (if it exists). It also sets up the
;	INT 40 vector and the INT 13 vector, pretending that a hard disk
;	does not exist. The master DMA controller is also set so DMA
;	channel 2 has a path to the CPU. This is only set once.
;
ALIGN	4
SET_FLOPPY:
	PUSHF
	CLI
	MOV	AL,4
	OUT	DMA_MASTER+01AH, AL	; Write anything
	OUT	DMA_MASTER+010H, AL	; Disable DMA
	MOV	AL,0C0H			; Select channel 4
	OUT	DMA_MASTER+016H, AL	; Master DMA chan 4->7
	XOR	AL,AL			; Get a zero
	OUT	DMA_MASTER+010H, AL	; Enable
	OUT	DMA_MASTER+01EH, AL	; Unmask
	PUSH	ES
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; ES = 0
	MOV	DI,(DISKS SHL 2)	; Hard disk vector
	MOV	AX,OFFSET INT_DISK	; Local procedure
	STOSW				; Patch offset
	MOV	AX,CS			; Current code segment
	STOSW				; Patch segment
	MOV	DI,(FLOPPY SHL 2)	; FLoppy disk vector
	MOV	AX,OFFSET INT_DISK	; Local procedure
	STOSW				; Patch offset
	MOV	AX,CS			; Current code segment
	STOSW				; Patch segment
	MOV	DI,(FLOPPY_TAB SHL 2)	; Parameter table vector
	MOV	AX,OFFSET DISK_PARTAB	; Local procedure
	STOSW				; Patch offset
	MOV	AX,CS			; Current code segment
	STOSW				; Patch segment
	MOV	DI,(IRQ6 SHL 2)		; Floppy disk interrupt
	MOV	AX,OFFSET FDC_INTER	; Local procedure
	STOSW				; Patch offset
	MOV	AX,CS			; Current code segment
	STOSW				; Patch segment
	IN	AL,MSK_CTL0		; Get controller mask
	SLOW_IO				; Slow down I/O
	AND	AL,10111111B		; Unmask the floppy
;                  ||||||||____ IRQ0
;                  |||||||_____ IRQ1
;                  ||||||______ IRQ2
;                  |||||_______ IRQ3
;                  ||||________ IRQ4
;                  |||_________ IRQ5
;                  ||__________ IRQ6
;                  |___________ IRQ7
;
	OUT	MSK_CTL0,AL		; Enable the controller
	SLOW_IO				; Slow down I/O
	INT	IRQ6			; Execute the interrupt
	POP	ES
	POPF
	MOV	SI,OFFSET CKFLOP	; Point to message
	CALL	PRINT			; Write to screen
	XOR	DX,DX			; Drive 0
	XOR	AX,AX			; Reset drive system
	INT	FLOPPY			; Drive function
	MOV	SI,OFFSET CHKTOK	; Assume it's okay
	JNC	SHORT FLOPOK		; Correct, it's okay
	MOV	SI,OFFSET CKFAIL	; It failed
FLOPOK:	PUSH	AX			; Save return code
	CALL	PRINT			; Write the message
	POP	AX			; Restore return code
	CALL	HEXW			; Write to screen
	MOV	SI,OFFSET PCRLF		; Point to end-of-line
	JMP	PRINT			; Write to screen, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up the BIOS data area to be compatible with DOS.
;	It is necessary to boot DOS, but is not necessary for real
;	operating systems.
;
ALIGN	4
SET_COMPAT_DATA:
	XOR	BX,BX				; Zero the index
	MOV	BYTE PTR [BX+MEDIA_TYPE0],17H	; 1.44 Mb, known media
	MOV	BYTE PTR [BX+MEDIA_TYPE1],07H	; 1.44 Mb, unknown media
	MOV	DI,RS232C_0			; First location
	MOV	CX,4				; Max possible
	XOR	AX,AX				; What to look for
	REPNZ	SCASW				; See what we have
	SUB	DI,RS232C_0			; First location
	SHR	DI,1				; It was words
	DEC	DI				; One less
	MOV	DX,DI				; Save Number of COM ports
;
	MOV	DI,PRINT_0			; First location
	MOV	CX,3				; Max possible
	REPNZ	SCASW				; See what we have
	SUB	DI,PRINT_0			; First location
	SHR	DI,1				; It was words
	DEC	DI				; One less
	MOV	CX,DI				; Save
;
	MOV	CH,CL				; Set number of printers
	SHL	CH,6				; Place for printers
	MOV	DH,DL				; Number of COM ports
	SHL	DH,1				; Place for COM ports
	OR	DH,CH				; Combine
	MOV	AX,0000000000000001B		; Device list
;                  ||||||||||||||||_______ Boot diskette
;                  |||||||||||||||________ Coprocessor installed
;                  ||||||||||||||_________ Pointing device installed
;                  |||||||||||||__________ VGA area
;                  ||||||||||_____________ Number of diskette drives (-1)
;                  ||||||||_______________ Reserved
;                  |||||||________________ Number of COM ports
;                  |||||__________________ Reserved
;                  ||_____________________ Number of printers
	OR	AH,DH				; Set printer/com-port info
	MOV	SI,COPR_CTLW			; Point to control word
	FINIT					; Initiaize coprocessor
	FSTCW	[SI]				; Store control word
	CMP	WORD PTR [SI],37FH		; See if it worked
	JNZ	SHORT NOCOP			; No coprocessor
	FLD1					; Put 1.0 onto FPU stack
	FLDZ					; Put 0.0 onto FPU stack
	FDIV					; Divide by zero
	FSTP	QWORD PTR [SI]			; Store result
	OR	AL,00000010B			; Set coprocessor bit
NOCOP:	MOV	DX,[BX+DEVICE_LIST]		; Get VGA info
	AND	DX,0000000000111000B		; Save only VGA info
	OR	AX,DX				; Insert VGA info
	MOV	[BX+DEVICE_LIST],AX		; Set device-list
	MOV	AL,00100110B
;                  ||||||||_______________ Drive zero is change-line
;                  |||||||________________ Drive zero is multirate
;                  ||||||_________________ Drive zero is determined
;                  |||||__________________ Reserved
;                  ||||___________________ Drive 1 is change-line
;                  |||____________________ Drive 1 is multirate
;                  ||_____________________ Drive 1 is determined
;                  |______________________ Reserved
	MOV	[BX+DISK_INFO],AL		; Set diskette info
	MOV	WORD PTR [BX+BOOT_FLAG],1234H	; Set boot flag
	MOV	WORD PTR [BX+POST_RES_LOW],OFFSET DUMMY	; Dummy routine
	MOV	WORD PTR [BX+POST_RES_HIGH],CS		; Set segment
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now display what we've got.
;
	MOV	SI,OFFSET SYSRAM		; Point to prompt
	CALL	PRINT				; Write to screen
	INT	12H				; Get system (low) memory
	CALL	BIN2ASC				; Convert/print
	MOV	SI,OFFSET KBYTES		; Point to prompt
	CALL	PRINT				; Write to screen
	MOV	SI,OFFSET EXTRAM		; Point to prompt
	CALL	PRINT				; Write to screen
	MOV	AX,0E801H			; Extended RAM
	INT	TOOLS				; Get from our BIOS
	AND	EBX,0FFFFH			; Mask high bits
	SHL	EBX,06H				; Convert to kilobytes
	AND	EAX,0FFFFH			; Mask high junk
	ADD	EAX,EBX				; Combine results
	CALL	BIN32EAX			; Convert/print
	MOV	SI,OFFSET KBYTES		; Point to prompt
	CALL	PRINT				; Write to screen
	MOV	SI,OFFSET PRINTR		; Point to prompt
	CALL	PRINT				; Write to screen
	MOV	SI,PRINT_0			; The first address
	MOV	CX,3				; Maximum of 3 ports
MOREP:	LODSW					; Get possible address
	OR	AX,AX				; End of list?
	JZ	SHORT NOMOP			; Yes, no more
	CALL	HEXW				; Print the port address
	PUSH	SI				; Save pointer
	MOV	SI,OFFSET SPACES		; Print a space
	CALL	PRINT				; Print it
	POP	SI				; Restore pointer
	LOOP	MOREP				; Do all printers
NOMOP:	CMP	CX,3				; No printers?
	JNZ	SHORT SOMEP			; Some were found
	CALL	BIN2ASC				; Write '0' on the screen
SOMEP:	MOV	SI,OFFSET PCRLF			; End of line
	CALL	PRINT				; Write end of line
	MOV	SI,OFFSET COMPRT		; Point to prompt
	CALL	PRINT				; Write to screen
	MOV	SI,RS232C_0			; The first address
	MOV	CX,4				; Maximum of 4 ports
MOCOM:	LODSW					; Get possible address
	OR	AX,AX				; End of list?
	JZ	SHORT NOMOC			; Yes, no more
	CALL	HEXW				; Print the port address
	PUSH	SI				; Save pointer
	MOV	SI,OFFSET SPACES		; Print a space
	CALL	PRINT				; Print it
	POP	SI				; Restore pointer
	LOOP	MOCOM				; Do all COM ports
NOMOC:	CMP	CX,4				; No COM ports
	JNZ	SHORT SOMEC			; Some were found
	CALL	BIN2ASC				; Write '0' on the screen
SOMEC:	MOV	SI,OFFSET PCRLF			; End of line
	CALL	PRINT				; Write end of line
	MOV	SI,OFFSET CMOSTIM		; Point to string
	CALL	PRINT				; Print the string
	CALL	SHOW_TIME			; Display the time
	CALL	SHOW_DATE			; Display the date
	MOV	SI,OFFSET PCRLF			; End of line
	CALL	PRINT				; Write to screen
	MOV	SI,OFFSET MATHCO		; Point to string
	CALL	PRINT				; Write to screen
	MOV	BX,DEVICE_LIST			; Point to devices
	MOV	AX,[BX]				; Get device list
	TEST	AL,00000010B			; Check for coprocessor
;                  ||||||||_______ Boot diskette
;                  |||||||________ Coprocessor installed
;                  ||||||_________ Pointing device installed
;                  |||||__________ VGA area
;                  ||_____________ Number of diskette drives (-1)
;                  |______________ Reserved
;
	JNZ	SHORT COPOK			; We have a coprocessor
	MOV	SI,OFFSET PNOT			; Point to string
	CALL	PRINT				; Write to screen
COPOK:	MOV	SI,OFFSET PFOUND		; Point to string
	CALL	PRINT				; Write to screen
	CALL	GET_CMOS_CHKSUM			; Get CMOS checksum
	MOV	BX,AX				; Save
	MOV	AL,2EH				; Offset of high byte
	CALL	GET_CMOS			; Get from CMOS
	MOV	CH,AL				; Save as high byte
	MOV	AL,2FH				; Offset of low byte
	CALL	GET_CMOS			; Get from CMOS
	MOV	CL,AL				; CX = CMOS checksum
	CMP	BX,CX				; Check checksum
	JZ	SHORT CHKOK			; Was okay
	PUSH	BX				; Save calculated value
	PUSH	CX				; Save internal value
	MOV	SI,OFFSET PCRLF			; For previous prompt
	CALL	PRINT				; Write to screen
	MOV	SI,OFFSET CMOSBAD		; Point to string
	CALL	PRINT				; Print the string
	POP	AX				; Bad value
	CALL	BIN2ASC				; Write to screen
	MOV	SI,OFFSET CMOSSHD		; Point to string
	CALL	PRINT				; Write to screen
	POP	AX				; Calculated value
	CALL	BIN2ASC				; Write to screen
CHKOK:	MOV	SI,OFFSET PCRLF			; End of line
	JMP	PRINT				; To screen, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This tests the virtual block move in protected mode. First
;	we will test it with a bad GDT to verify that we can get
;	back when the processor resets. Then we test it with a block-move
;	to screen RAM in protected mode.
;
ALIGN	4
TEST_VMB:
	MOV	SI,OFFSET CHKPROT		; Point to prompt
	CALL	PRINT				; Write to screem
	PUSH	DS
	PUSH	ES
	MOV	AX,SETUP_BUF			; Use this segment for testing
	MOV	ES,AX				; ES = 5000H
	MOV	DS,AX
	XOR	DI,DI				; ES:DI = GDT
	MOV	SI,DI				; ES:SI = GDT
	XOR	AX,AX				; Fill word
	MOV	CX,20H				; 32 words
	REP	STOSW				; Clear
	MOV	CX,1000H			; Bytes to move
	MOV	AX,8000H
	MOV	DX,000BH
	MOV	WORD PTR [SI.GDT.SRC_LOW],AX	; 24 bit source address LOW
	MOV	BYTE PTR [SI.GDT.SRC_HIG],DL	; 24 bit source address HIGH
	MOV	BYTE PTR [SI.GDT.SRC_ACC],93H	; Source Access byte
	MOV	WORD PTR [SI.GDT.SRC_LEN],CX	; Length in BYTES
	MOV	WORD PTR [SI.GDT.SRC_NUL],0	; * M A N D A T O R Y *
;
	MOV	AX,0000H
	MOV	DX,0010H			; 1 Megabyte
	MOV	WORD PTR [SI.GDT.DES_LOW],AX	; 24 bit dest address LOW
	MOV	BYTE PTR [SI.GDT.DES_HIG],DL	; 24 bit dest address HIGH
	MOV	BYTE PTR [SI.GDT.DES_ACC],93H	; Destination access BYTE
	MOV	WORD PTR [SI.GDT.DES_LEN],CX	; Length in BYTES
	MOV	WORD PTR [SI.GDT.DES_NUL],0	; * M A N D A T O R Y *
	SHR	CX,1				; Convert to words
	MOV	AX,8700H			; Virtual block move
	INT	TOOLS				; Execute it
	JC	SHORT NOWAY			; There was a problem
;
	PUSH	ES				; Save segment
	MOV	AX,SCR_SEGMENT			; AX = screen segment
	MOV	ES,AX				; ES = screen segment
	XOR	DI,DI				; ES:DI = screen
	MOV	CX,1000H			; One page
	MOV	AX,1720H			; Attribute and space
	REP	STOSW				; Write to screen
	MOV	AX,BIOS_SEG			; BIOS data
	MOV	ES,AX				; ES = BIOS data
	MOV	BX,TIMER_LOW			; Timer tick
	MOV	AX,ES:[BX]			; Get timer tick
WAT0:	CMP	AX,ES:[BX]			; Check tick
	JZ	SHORT WAT0			; Wait until it changes
	POP	ES				; Restore segment
;
	XOR	DI,DI				; ES:DI = GDT
	MOV	SI,DI				; ES:SI = GDT
	XOR	AX,AX				; Fill word
	MOV	CX,20H				; 32 words
	REP	STOSW				; Clear
	MOV	CX,1000H			; Bytes to move

	MOV	AX,0000H
	MOV	DX,0010H			; 1 Megabyte
	MOV	WORD PTR [SI.GDT.SRC_LOW],AX	; 24 bit source address LOW
	MOV	BYTE PTR [SI.GDT.SRC_HIG],DL	; 24 bit source address HIGH
	MOV	BYTE PTR [SI.GDT.SRC_ACC],93H	; Source Access byte
	MOV	WORD PTR [SI.GDT.SRC_LEN],CX	; Length in BYTES
	MOV	WORD PTR [SI.GDT.SRC_NUL],0	; * M A N D A T O R Y *
;
	MOV	AX,8000H
	MOV	DX,000BH
	MOV	WORD PTR [SI.GDT.DES_LOW],AX	; 24 bit dest address LOW
	MOV	BYTE PTR [SI.GDT.DES_HIG],DL	; 24 bit dest address HIGH
	MOV	BYTE PTR [SI.GDT.DES_ACC],93H	; Destination access BYTE
	MOV	WORD PTR [SI.GDT.DES_LEN],CX	; Length in BYTES
	MOV	WORD PTR [SI.GDT.DES_NUL],0	; * M A N D A T O R Y *
	SHR	CX,1				; Convert to words
	MOV	AX,8700H			; Virtual block move
	INT	TOOLS				; Execute it
;
NOWAY:	POP	ES
	POP	DS
	MOV	SI,OFFSET CHKTOK		; Assume it's okay
	JNC	SHORT PROTOK			; Was okay
	MOV	SI,OFFSET CKFAIL		; It failed
PROTOK:	PUSH	AX				; Save return value
	CALL	PRINT				; Write to screen
	POP	AX				; Restore return value
	CALL	HEXW				; Write return value
	MOV	SI,OFFSET PCRLF			; End of the line
	JMP	PRINT				; To screen, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Interrupt descriptor table for the protected-move operation.
;	This will not be correct unless the code is running in PROM
;	at absolute address 0x000F+offset.
;
ALIGN	4
BIOS_IDT:
	DW	100H				; Length
	DW	OFFSET BIOS_IDT_DESC		; Linear address LOW word
	DW	0FH				; Linear address HIGH word
	DW	0
BIOS_IDT_DESC:
REPT	32
	DW	OFFSET TRAP
	DW	20H
	DW	8700H
	DW	0
ENDM
ALIGN	4
TRAP:	MOV	AX,SHUTDOWN		; Shutdown byte = 0
	CALL	PUT_CMOS		; Write to CMOS
	JMP	RSTART			; Warm boot
ALIGN	4
REAL_IDT_LOC:	DW	3FFH		; Real-mode IDT (dummy)
		DD	0
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is called from software interrupt 15H with function code
;	87H. It is used to copy a block of data from anywhere to anywhere
;	within the first 24 megabytes of memory.
;
;	Upon entry:
;	ES:SI points to a descriptor provided by the caller.
;	DB	8 DUP(0)			8 bytes of dummy
;	GDT_LOC	GDT<>				8 bytes
;	Source	GDT<>				8 Bytes
;	Dest	GDT<>				8 Bytes
;	BIOS CS	GDT<>				8 Bytes
;	BIOS SS GDT<>				8 Bytes
;
;	CX = Number of words to move (0 to 8000H).
;
;	Upon exit:
;	Success AH = 0, CF = 0, ZF = 1
;       Failure AH = Error code, CF = 1, ZF = 0
;	AH = 1 RAM parity. Parity error will be cleared
;       AH = 2 Exception interrupt error
;       AH = 3 Gate address for CS or SS failed
;       All registers except EAX are preserved.
ALIGN	4
VMB:	CLI					; No interrupts
	CLD					; Forwards
	PUSHA					; Need to save everything
	PUSH	ES				; Save ES
	PUSH	DS				; Save DS
	SET_A20_ON				; Set up for 24-bits
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Set the shutdown byte in case the input selector is a invalid one
;	which will cause the CPU to reset. Also save the current interrupt
;	controller masks and mask them off.
;
	IN	AL,MSK_CTL0			; Get current interrupt mask
	MOV	AH,AL				; Save in AH
	IN	AL,MSK_CTL1			; Get current interrupt mask
	PUSH	AX				; Save for later
	MOV	AX,BIOS_SEG			; Get BIOS data
	MOV	DS,AX				; DS = BIOS data
	XOR	BX,BX				; Clear
	MOV	WORD PTR [BX+POST_RES_HIGH],SS	; Save stack segment
	MOV	WORD PTR [BX+POST_RES_LOW],SP	; Save stack pointer
	MOV	AX,090FH			; Shutdown byte
	CALL	PUT_CMOS			; Save in NVRAM
	MOV	AL,NMI_OFF			; Disable parity
	OUT	CMOS_INDEX,AL			; Set index register
	MOV	AX,ES				; Current data segment
	MOV	DS,AX				; DS=ES=AX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Fill in caller's descriptor.
;
	MOV	WORD PTR [SI+HILIMIT+8],0CFH	; Granularity = 4096, 386
	MOV	WORD PTR [SI+LIMIT+8],0FFFFH	; Limit = 0xffff
	MOV	BYTE PTR [SI+ACCESS+8],92H	; Read/write
	MAKE_REAL DS, SI
	MOV	[SI+LOWBASE+8],AX		; Set low address
	MOV	[SI+HIBASE+8],DL		; Set high address
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Stack descriptor
;
	MOV	WORD PTR [SI+HILIMIT+28H],0FFH	; Granularity = 4096, 386
	MOV	WORD PTR [SI+LIMIT+28H],0FFFFH	; Set limit
	MOV	BYTE PTR [SI+ACCESS+28H],93H	; Set access control
	MAKE_REAL SS				; Fix up stack descriptor
	MOV	[SI+LOWBASE+28H],AX		; Set low address
	MOV	[SI+HIBASE+28H],DL		; Set high address
	LIDT	FWORD PTR CS:[BIOS_IDT]		; Set IDT
	LGDT	FWORD PTR DS:[SI+8H]		; Load GDT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	BIOS Descriptor
;
	MOV	WORD PTR [SI+HILIMIT+20H],0FFH	; Granularity
	MOV	WORD PTR [SI+LIMIT+20H],0FFFFH	; Set limit
	MOV	BYTE PTR [SI+ACCESS+20H],09BH	; Set access rights
	MOV	WORD PTR [SI+LOWBASE+20H],0	; Set address (0)
	MOV	BYTE PTR [SI+HIBASE+20H],0FH	; 0->FFFFH
	MOV	EAX,CR0				; Get current control word
	OR	EAX,PROTECTED			; Set protected mode bit
	MOV	CR0,EAX				; This is it
	JMP	SHORT VMB1			; Flush cache
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Set up segment descriptors
;
ALIGN	4
VMB1:	XOR	AH,AH				; Clear high byte
	MOV	AL,18H				; Destination GDT
	MOV	ES,AX				; Into ES
	MOV	AL,10H				; Source GDT
	MOV	DS,AX				; Into ds
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	All that work for this trivial little procedure to be executed
;	in protected mode! Note: the code is still executing in 16-bit
;	mode, the data however, is 32-bits wide and it starts (base)
;	where the converted caller's input 24-bit addresses started.
;	Therefore, a correctly running assembler will show 'MOVSD'
;	with a 0x66 prefix.
;
	XOR	SI,SI				; DS:SI = source
	MOV	DI,SI				; ES:DI = destination
	SHR	CX,1				; Words to DWORDS
	REP	MOVSD				; Copy DWORDS
	ADC	CX,0				; Possible odd word
	REP	MOVSW				; Copy if it exists
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now return to real mode, check for errors later.
;
	MOV	AX,28H				; Set DS, ES to point to stack
	MOV	DS,AX
	MOV	ES,AX				; DS=ES=SS
	MOV	EAX,CR0				; Machine status word
	AND	EAX,(NOT PROTECTED)
	MOV	CR0,EAX				; Return to real mode
	JMP	SHORT VMB2
ALIGN	4
VMB2:	LIDT	FWORD PTR CS:[REAL_IDT_LOC]	; Dummy to clear
	SET_A20_OFF				; Turn off A<20>
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Clear shutdown byte
;
	MOV	AX,000FH			; Reset CMOS
	CALL	PUT_CMOS			; Restore
	XOR	AL,AL				; Enable parity
	OUT	CMOS_INDEX,AL			; Set back
	POP	AX				; Throw away old mask
	POP	DS				; Recover regs
	POP	ES
	POPA
	IN	AL,PORT_B			; Check for errors
	MOV	AH,AL				; Save
	TEST	AL,MEM_CHK			; Check for parity error
	JZ	SHORT VMB3			; No parity error
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	There was a parity error.
;
	OR	AL,PAR_CHK_OFF			; Set parity off
	OUT	PORT_B,AL			; Disable parity
	SLOW_IO
	SLOW_IO
	MOV	AL,AH				; Get previous value
	OUT	PORT_B,AL			; Set parity back on
	MOV	AH,1				; Show parity error
	STC					; Set carry
	JMP	SHORT VMB4			; Quit
VMB3:	XOR	AX,AX				; Show no errors
VMB4:	STI					; Enable interrupts
	RETF	2
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets expansion RAM. It is saved in CMOS so must be read
;	from there. Function, GET_CMOS, saves the flags so the flags
;	will not be changed by this function.
;
ALIGN	4
GET_EXP_RAM:
	PUSH	BX				; Save register used
	MOV	AL,17H				; Low byte of expansion RAM
	CALL	GET_CMOS			; Get CMOS data.
	MOV	BL,AL				; Save low byte
	MOV	AL,18H				; High byte of expansion RAM
	CALL	GET_CMOS			; Get CMOS data
	MOV	AH,AL				; Move into high
	MOV	AL,BL				; Get low
	POP	BX				; Restore register used
	RETF	2				; Return with flags
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets extended RAM.
;
;  INT 15 - Phoenix BIOS v4.0 - GET MEMORY SIZE FOR >64M CONFIGURATIONS
;        AX = E801h
;  Return: CF clear if successful
;           AX = extended memory between 1M and 16M, in K (max 3C00h = 15MB)
;           BX = extended memory above 16M, in 64K blocks
;           CX = configured memory 1M to 16M, in K
;           DX = configured memory above 16M, in 64K blocks
;        CF set on error
;
ALIGN	4
GET_EXT_RAM:
	PUSH	DS
	MOV	BX,BIOS_SEG			; BIOS data
	MOV	DS,BX				; DS = BIOS data
	MOV	AX,DS:[PRINT_3]			; High RAM here 64k chunks
	AND	EAX,0FFFFH			; Mask high junk
	SHL	EAX,6				; Make 1k blocks
	MOV	EBX,EAX				; Copy
	CMP	EAX,03C00H			; More than 15 MB?
	JC	SHORT LT15			; No
	MOV	EAX,03C00H			; Yes
LT15:	SUB	EBX,EAX				; Subtract low mem
	SHR	EBX,6				; Make 64k blocks
	MOV	CX,AX				; Copy
	MOV	DX,BX				; Copy
	POP	DS
	CLC
	RETF	2				; Return with flags
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
ALIGN	4
CFG_TAB	DW	9				; Bytes in the table
	DB	0FCH				; It's an AT
	DB	000H				; AT Generic
	DB	1				; BIOS version
        DB      11100100B			; Features
;               ||||||||_____________ Reserved
;               |||||||______________ PC-Type I/O
;               ||||||_______________ Extended BIOS area used
;               |||||________________ Wait for external event
;               ||||_________________ Keyboard intercept supported
;               |||__________________ Realtime clock present
;               ||___________________ Second interrupt chip present
;               |____________________ DMA channel 3 used for fixed disk.
;
	DD	0				; Fill at the end
	DD	0				; Spare fill at the end
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This used to be the "tape-drive" port used to save and restore
;	programs written BASIC. Now it's the catch-all, including some
;	important protected-mode interfaces. The function-codes are
;	scattered all over the place so a command-branch table doesn't
;	look good. I could make a translate table but I'll just wing it
;	with a bunch of compares.
;
ALIGN	4
INT_15H:
	STI					; User function, enable
	CMP	AH,87H				; Protected-mode copy?
	JNZ	SHORT NOCPY			; No
	JMP	VMB				; Yes, implied return
NOCPY:	CMP	AX,0E801H			; New extended RAM?
	JNZ	SHORT NOE0			; No
	JMP	GET_EXT_RAM			; Yes, implied return
NOE0:	CMP	AH,88H				; Extended memory?
	JNZ	SHORT NO88			; No
	JMP	GET_EXP_RAM			; Get expansion RAM
NO88:	CMP	AH,0C0H				; Configuration data?
	JNZ	SHORT NOC0			; No
	MOV	BX,CS				; Current code segment
	MOV	ES,BX				; ES = offset of table
	MOV	BX,OFFSET CFG_TAB		; ES:BX = address of table
	JMP	SHORT EX15			; Done
NOC0:	CMP	AH,80H				; Device open
	JZ	SHORT EX15			; Done
	CMP	AH,81H				; Device close
	JZ	SHORT EX15			; Done
	CMP	AH,82H				; Program termination?
	JZ	SHORT EX15			; Done
	CMP	AH,84H				; Joystick?
	JNZ	SHORT NO84			; No joystick
	XOR	AX,AX				; Clear these, per spec.
	MOV	BX,AX
	MOV	CX,AX
	MOV	DX,AX
	STC					; Set CY
	RETF	2				; Return with carry
NO84:	CMP	AH,85H				; SysReq Key
	JZ	SHORT EX15			; Done
	CMP	AH,90H				; Device busy
	JZ	SHORT EX15			; Done
	XOR	AH,AH
	STC
EX15:	RETF	2

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The INT 15 vector is used by LILO to move data across the 16-bit
;	boundary during boot. It is also used for memory sizing, etc.
;	Therefore it's important.
;
ALIGN	4
SET_TOOLS:
	PUSHF					; Save
	CLI					; No interrupts
	PUSH	ES				; Save extra segment
	XOR	AX,AX				; Get zero
	MOV	ES,AX				; ES = 0
	MOV	DI,(TOOLS SHL 2)		; Point to vector
	MOV	AX,OFFSET INT_15H		; Vector to patch
	STOSW					; Patch offset
	MOV	AX,CS				; Current code segment
	STOSW					; Patch code segment
	POP	ES				; Restore segment
	POPF					; Restore flags
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This attempts to set any  8250 UART that may be addressed
;	by the register value in DX. It returns DX = base address
;	if the UART is found. The presence of a UART is reflected
;	in the flags. ZF=1 if there is no UART
;
ALIGN	4
INI8250:
	ADD	DX,IER				; Interrupt enable register
	XOR	AL,AL				; Get a zero
	OUT	DX,AL				; Interrupts off
	CALL	WAITUS				; Need to wait
	IN	AL,DX				; Read value
	CMP	AL,0FFH				; See if dead port
	JZ	SHORT NOPORT			; Not there
	ADD	DX,(LCR - IER)			; Offset to line control
	MOV	AL,10000000B			; Divisor latch access bit
	OUT	DX,AL				; Set DLAB
	SUB	DX,LCR				; Back to base port
	MOV	AX,12				; 9600 baud-rate
	OUT	DX,AX				; Set the baud - rate
	CALL	WAITUS				; Need to wait
	ADD	DX,LCR				; Offset to line control
	MOV	AL,00000011B			; 8 bits, 1 stop, no parity
	OUT	DX,AL				; Set baud-rate
	SLOW_IO					; Slow down I/O
	SUB	DX,LCR				; Back to base-port
NOPORT:	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This looks for RS-232C ports and sets them to 9600 baud.
;	It records their presence in the appropriate place.
;
ALIGN	4
COM1	DW	3F8H				; COM1 UART base address
COM2	DW	2F8H				; COM2 UART base address
COM3	DW	3E8H				; COM3 UART base address
COM4	DW	2E8H				; COM4 UART base address
ALIGN	4
SET_RS232C:
	PUSH	DS				; Save segment
	PUSH	ES				; Save segment
	XOR	AX,AX				; Get a zero
	MOV	ES,AX				; ES = 0
	MOV	DI,(SERIAL SHL 2)		; User vector
	MOV	AX,OFFSET INT_14H		; Offset of code to patch
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	POP	ES				; Restore segment
	MOV	AX,CS				; Current code segment
	MOV	DS,AX				; DS = CS
	MOV	SI,OFFSET COM1			; DS:SI = possible address
	MOV	DI,RS232C_0			; ES:DI = port address
	MOV	CX,4				; Possible ports
	XOR	AX,AX				; Get a zero
	REP	STOSW				; Clear the buffer
	MOV	DI,RS232C_0			; Point to beginning again
	MOV	CX,4				; Possible ports
RS232C:	PUSH	CX				; Save count
	LODSW					; Get UART address
	MOV	DX,AX				; Possible address
	CALL	INI8250				; Try to initialize
	JZ	SHORT NXTCOM			; Not valid, try next
	MOV	AX,DX				; Valid
	STOSW					; Save in destination
NXTCOM:	POP	CX				; Restore count
	LOOP	RS232C				; Do all ports
	POP	DS
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets up the RS-232C interrupt on the first COM port.
;
SET_RS232C_INT:
	PUSHF
	CLI
	PUSH	DS
	PUSH	ES
	MOV	AX,CS
	MOV	DS,AX				; DS = CS
	MOV	DX,[COM1]			; Get first com port
	PUSH	DX				; Save COM port address
	CALL	INI8250				; In case it's been changed
	XOR	AX,AX				; Zero
	MOV	ES,AX				; ES = 0
	MOV	DI,(IRQ4 SHL 2)			; IRQ4 vector
	MOV	AX,OFFSET RS232_ISR		; Interrupt service routine
	STOSW					; Offset into vector
	MOV	AX,CS				; Get code segment
	STOSW					; Segment into vector
	POP	DX				; Restore COM port address
	IN	AL,DX				; Get any junk
	ADD	DX,IER				; Interrupt enable
	MOV	AL,00000001B			; Interrupt enable
;	           ||||||||______ Interrupt on data ready
;	           |||||||_______ Interrupt on TX holding register ready
;	           ||||||________ Interrupt on line status change
;	           |||||_________ Interrupt on modem status change
;	           ||||__________ Reserved
	OUT	DX,AL				; Enable interrupts
	SUB	DX,IER				; Back to base port
	ADD	DX,LSR				; Line status
	IN	AL,DX				; Eat any junk
	SUB	DX,LSR				; Back to base port
	ADD	DX,MCR				; Modem control
	MOV	AL,00001011B			; Modem control bits
;	           ||||||||_____ DTR
;	           |||||||______ RTS
;	           ||||||_______ OUT1
;	           |||||________ OUT2
;                  ||||_________ LOOP
;                  |||__________ Reserved
;
	OUT	DX,AL				; Set modem control bits
;
	IN	AL,MSK_CTL0			; Get current mask
	AND	AL,11101111B			; IRQ 4
;	           ||||||||________ IRQ0
;	           |||||||_________ IRQ1
;                  ||||||__________ IRQ2
;	           |||||___________ IRQ3
;                  ||||____________ IRQ4
;	           |||_____________ IRQ5
;                  ||______________ IRQ6
;                  |_______________ IRQ7
	OUT	MSK_CTL0,AL			; Enable controller
	MOV	AX,BIOS_SEG			; Get BIOS segment
	MOV	DS,AX				; DS = BIOS data
	MOV	BX,KBD_HEAD			; Get keyboard head pointer
	MOV	WORD PTR [BX],KBD_BUF		; Start of keyboard buffer
	MOV	BX,KBD_TAIL			; Get keyboard tail pointer
	MOV	WORD PTR [BX],KBD_BUF		; Start of keyboard buffer
	POP	ES
	POP	DS
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the RS-232C interrupt service routine used when the
;	keyboard has been removed. It puts its data into the keyboard
;	buffer and therefore pretends that it's the keyboard.
;
RS232_ISR:
	PUSH	AX
	PUSH	DX
	PUSH	DS				; Save segment
	MOV	AX,CS				; Get code segment data
	MOV	DS,AX				; CS = data
	MOV	DX,[COM1]			; COM port
	MOV	AX,BIOS_SEG			; Get BIOS data
	MOV	DS,AX				; DS = BIOS data
	ADD	DX,LSR				; Offset to line status
	IN	AL,DX				; Read, throw away
	SUB	DX,LSR				; Back to base port
	ADD	DX,MSR				; Offset to modem status
	IN	AL,DX				; Read, throw away
	SUB	DX,MSR				; Back to base port
	ADD	DX,IIR				; Offset to interrupt status
	IN	AL,DX				; Read, throw away
	SUB	DX,IIR				; Back to base port
	IN	AL,DX				; Get character
	XOR	AH,AH				; Clear high byte
	CMP	AL,7FH				; Check for delete
	JNZ	NODEL				; Nope, it's good
	MOV	AL,8H				; Substitute a back-space
NODEL:	CALL	PUT_KEY				; Put into keyboard buffer
	MOV	AL,NOS_EOI			; End of interrupt
	OUT	ACK_CTL0,AL			; Acknowledge
	POP	DS
	POP	DX
	POP	AX
	IRET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Baud-rate table for 8250x UARTS.
;
ALIGN	4
BAUDTAB	DW	417H				; 110  baud
	DW	300H				; 150  baud
	DW	180H				; 300  baud
	DW	0C0H				; 600  baud
	DW	060H				; 1200 baud
	DW	030H				; 2400 baud
	DW	018H				; 4800 baud
	DW	00CH				; 9600 baud
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the serial communications vector. This is all poll-mode
;	stuff. Real programs use interrupts, but the PC/AT BIOS doesn't.
;
ALIGN	4
INT_14H:
	STI					; User function, enable
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	DS				; Save segment
	MOV	BX,BIOS_SEG			; Get BIOS data
	MOV	DS,BX				; DS = BIOS data
	MOV	BX,DX				; COM port
	SHL	BX,1				; Table has words
	ADD	BX,RS232C_0			; Beginning of the table
	MOV	DX,[BX]				; Get COM port base
	OR	DX,DX				; See if it exits
	JNZ	SHORT INT140			; It exists okay
INT14E:	STC					; Errors accumulate here
	JMP	INT14X				; Get out with CY set
INT140:
	CMP	AH,3				; Check function call
	JNBE	SHORT INT14E			; It's an error
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Presently, the COM port is in DX and the function code is
;	in AH. Both have been limit-checked.
;
	DEC	AH				; Check for initialize
	JNS	SHORT INT141			; Not initialize
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This initializes the COM port and returns some status.
;	Function code (0).
;
	PUSH	AX				; Save params
	MOV	BL,AL				; Copy, need baud-rate
	AND	BX,0000000011100000B		; Clear unused bits
	SHR	BX,4				; Shift 1 less than mask
	ADD	BX,OFFSET BAUDTAB		; Offset to rate table
	MOV	CX,CS:[BX]			; Get baud-rate divisor
	ADD	DX,IER				; Interrupt enable register
	XOR	AL,AL				; Get a zero
	OUT	DX,AL				; Interrupts off
	ADD	DX,(LCR - IER)			; Offset to line control
	MOV	AL,10000000B			; Divisor latch access bit
	OUT	DX,AL				; Set DLAB
	SUB	DX,LCR				; Back to base port
	MOV	AX,CX				; baud-rate
	OUT	DX,AX				; Set the baud - rate
	CALL	WAITUS				; Need to wait
	ADD	DX,LCR				; Offset to line control
	POP	AX				; Recover line parameters
	AND	AL,00011111B			; Mask baud-rate bits
;                     |||||_______ Word length
;                     |||_________ Stop bits
;                     ||__________ Parity enable
;                     |___________ Odd/even
;
	OUT	DX,AL				; Set baud-rate
	CALL	WAITUS				; Need to wait
	ADD	DX,(MCR - LCR)			; Get to modem control
	MOV	AL,00000011B			; Turn on most everything
;                      ||||________ DTR
;                      |||_________ RTS
;                      ||__________ Out 1
;                      |___________ Out 2
	OUT	DX,AL				; Write to port
	ADD	DX,(LSR - MCR)			; Need line-status register
	IN	AL,DX				; Get line status
	MOV	AH,AL				; Put in proper place
	ADD	DX,(MSR - LSR)			; Get modem status
	IN	AL,DX				; Get status
	JMP	SHORT INT14X
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
INT141:	DEC	AH				; Check for send character
	JNS	SHORT INT142			; Not send
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sends a character in AL, then returns some status.
;	Unlike the IBM/PC, as long as the chip exists, this can
;	never fail. I ignore the modem-control bits since in most
;	all hookups, only 3 wires are used for poll-mode RS-232C.
;	Communications programs deal directly with the hardware and
;	use interrupts. This allows one to do `DIR >COM1` without
;	hanging the machine.
;	Function code (1)
;
	ADD	DX,LSR				; Get to line-status
	MOV	AH,AL				; Save character
WURDY:	IN	AL,DX				; Check line status
	AND	AL,00100000B			; Holding register empty?
;                  ||||||||______ Data ready
;                  |||||||_______ Overrun error
;                  ||||||________ Parity error
;                  |||||_________ Framing error
;                  ||||__________ Break interrupt
;                  |||___________ Transmitter holding register empty
;                  ||____________ Transmitter shift register empty
;                  |_____________ Reserved for Intel stock dividend
;
	JNZ	SHORT COMRDY			; Yes
	CALL	WAITUS				; Wait a bit
	JMP	SHORT WURDY			; Continue
COMRDY:	SUB	DX,LSR				; Back to base port
	MOV	AL,AH				; Get saved byte
	OUT	DX,AL				; Write the character
	ADD	DX,LSR				; Need line-status register
	IN	AL,DX				; Get line status
	MOV	AH,AL				; Put in proper place
	ADD	DX,(MSR - LSR)			; Get modem status
	IN	AL,DX				; Get status
	JMP	SHORT INT14X
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
INT142:	DEC	AH				; Check for receive
	JNS	SHORT INT143			; Nope, must be status
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This reads a character into AL, then returns some status.
;	Function code (2). If the character is not ready, it returns
;	with the time-out bit set and the carry flag set.
;
	ADD	DX,LSR				; Line status register
	SET_TIMER 02H				; 2 timer-ticks
WTDAT:	IN	AL,DX				; Get UART status
	TEST	AL,00000001B			; Holding register empty?
;                  ||||||||______ Data ready
;                  |||||||_______ Overrun error
;                  ||||||________ Parity error
;                  |||||_________ Framing error
;                  ||||__________ Break interrupt
;                  |||___________ Transmitter holding register empty
;                  ||____________ Transmitter shift register empty
;                  |_____________ Reserved for Intel stock dividend
;
	JNZ	SHORT DATRDY			; We have data
	WAIT_TIMER WTDAT			; Wait until ready
;
;	Here, it timed out. No data are available.
;
	MOV	AH,AL				; Line status
	MOV	AL,2				; Restore function
	OR	AH,10000000B			; Show timeout
	STC					; Show an error
	JMP	SHORT INT14X
;
DATRDY:	SUB	DX,LSR				; Back to base port
	MOV	AH,AL				; Save line status
	IN	AL,DX				; Get character
	JMP	SHORT INT14X
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This just returns the device status.
;	Function code (3).
;
INT143:	ADD	DX,LSR				; Need line-status register
	IN	AL,DX				; Get line status
	MOV	AH,AL				; Put in proper place
	ADD	DX,(MSR - LSR)			; Get modem status
	IN	AL,DX				; Get status
;
INT14X:	POP	DS				; Restore segment
	POP	DX
	POP	CX
	POP	BX
	RETF	2
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This it the printer output vector. It is not even slightly
;	tested because these kinds of printers don't exist anymore.
;	However, standard disclaimer applies, 30 seconds or 30 feet,
;	whichever comes first.
;
ALIGN	4
INT_17H:
	STI					; User function, enable
	PUSH	BX				; Save registers used
	PUSH	CX
	PUSH	DX				; Save printer number
	PUSH	DS				; Save segment
	MOV	BX,BIOS_SEG			; BIOS data segment
	MOV	DS,BX				; DS = BIOS data
	MOV	BX,DX				; Get printer
	SHL	BX,1				; Table has words
	ADD	BX,PRINT_0			; First port
	MOV	DX,[BX]				; Printer base address
	OR	DX,DX				; See if it exists
	JNZ	SHORT PRN0			; Okay, it exists
	JMP	PRN5				; Exit with carry set
PRN0:	DEC	AH				; Check for 'print'
	JNS	SHORT PRN3			; Not to print
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Here we are supposed to print a character in AL. It seems to
;	work on an old Epson.
;
	PUSH	AX				; Save character
	OUT	DX,AL				; Just write the character
	INC	DX				; Status port
	SET_TIMER 02H				; Set timeout interval
PRN1:	IN	AL,DX				; Get status
	TEST	AL,10000000B			; Check for busy
	JNZ	SHORT PRN2			; Not busy
	WAIT_TIMER PRN1				; Wait for ready
	MOV	CL,AL				; Save status
	OR	CL,00000001B			; Set timout bits
	AND	CL,11111001B			; Turn OFF unused bits
	XOR	CL,01001000B			; Flip these bits
	POP	AX				; Recover original character
	JMP	SHORT PRN5			; Error exit
PRN2:	INC	DX				; Ready stobe-port
	CALL	WAITPRN				; Wait 1200 microseconds
	MOV	AL,00001101B			; Set stobe high
	OUT	DX,AL				; Set strobe
	CALL	WAITPRN				; Wait 1200 microseconds
	MOV	AL,00001100B			; Set strobe low
	OUT	DX,AL				; Set it
	DEC	DX				; Back to status port
	CALL	WAITPRN				; Wait 1200 microseconds
	IN	AL,DX				; Get status
	MOV	CL,AL				; Save status
	AND	CL,11111000B			; Turn OFF unused bits
	XOR	CL,01001000B			; Flip these bits
	POP	AX				; Recover character
	JMP	SHORT PRN6			; All done
PRN3:	DEC	AH				; Check for initialize
	JNS	SHORT PRN4			; Not initialize
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Here we are supposed to initialize the printer. My old Epson
;	makes some noise when this executes. Maybe it works?
;
	PUSH	AX				; Save
	ADD	DX,2				; Output port
	MOV	AL,00001000B			; Init line
	OUT	DX,AL				; Set init line low
	SET_TIMER 02H				; Set wait interval
WSET:	WAIT_TIMER WSET				; Wait to set up
	MOV	AL,00001100B			; No interrupts, no auto-lf
	OUT	DX,AL				; Set it
	CALL	WAITPRN				; Wait for printer
	DEC	DX				; Back to status port
	IN	AL,DX				; Get status
	MOV	CL,AL				; Save status
	AND	CL,11111000B			; Turn OFF unused bits
	XOR	CL,01001000B			; Flip these bits
	POP	AX				; Restore
	JMP	SHORT PRN6			; All done
PRN4:	DEC	AH				; Printer status?
	JNS	SHORT PRN5			; Not printer status
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Here we are supposed to get the printer status. MSD.EXE gets
;	something it understands.
;
	PUSH	AX				; Save whatever
	INC	DX				; Status port
	IN	AL,DX				; Get status
	MOV	CL,AL				; Save status
	AND	CL,11111000B			; Turn OFF unused bits
	XOR	CL,01001000B			; Flip these bits
	POP	AX				; Restore saved value
	JMP	SHORT PRN6			; All done
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for an input function out-of-range.
;
	MOV	CL,AH				; Decremented function
PRN5:	STC					; None of above (error)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
PRN6:	MOV	AH,CL				; Set status into AH
	POP	DS				; Restore segment
	POP	DX
	POP	CX
	POP	BX
	RETF	2
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Table of possible ports for a printer.
;
ALIGN	4
PPORTS	DW	3BCH				; Printer port 1
	DW	378H				; Printer port 2
	DW	278H				; Printer port 3
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This looks for any printer ports and records their presence in the
;	appropriate place. It also patches in the printer vector.
;
ALIGN	4
SET_PRINTER:
	PUSHF					; Save
	CLI					; No interrupts
	PUSH	ES				; Save segment
	XOR	AX,AX				; Get a zero
	MOV	ES,AX				; ES = 0
	MOV	DI,(PRINTER SHL 2)		; Vector to patch
	MOV	AX,OFFSET INT_17H		; Offset
	STOSW					; Patch offset
	MOV	AX,CS				; Segment
	STOSW					; Patch segment
	POP	ES				; Restore segment
	POPF					; Restore flags
	PUSH	DS				; Save data segment
	PUSH	ES				; Save extra segment
	MOV	AX,CS				; Current code segment
	MOV	DS,AX				; DS = CS
	MOV	BX,BIOS_SEG			; Get data segment
	MOV	ES,BX				; ES = bios data
	MOV	DI,PRINT_0			; ES:DI = printer base
	MOV	SI,OFFSET PPORTS		; DS:SI = possible ports
	MOV	CX,3				; Three possible ports
FINDP:	LODSW					; Get possible port
	MOV	DX,AX				; Address into DX
	IN	AL,DX				; Anybody home?
	CMP	AL,0FFH				; Dead port
	JZ	SHORT NEXTP			; Try next
	MOV	AL,10000000B			; Bit to write
WPRN:	OUT	DX,AL				; Write a bit
	PUSH	DS				; Save segment
	MOV	DS,BX				; DS = BIOS data
	SET_TIMER 1				; One timer tick
SPRN:	WAIT_TIMER SPRN				; Wait for a tick
	POP	DS				; Restore segment
	SHR	AL,1				; Next bit
	JNC	SHORT WPRN			; Do all bits
	MOV	AX,DX				; Recover port
	STOSW					; Save address
NEXTP:	LOOP	FINDP				; Try all addresses
	MOV	CX,3				; Three possible printers
	XOR	DX,DX				; Reset printer (0)
RSETP:	XOR	AX,AX				; Init printer
	INT	PRINTER				; Execute interrupt
	INC	DX				; Next port
	LOOP	RSETP				; Continue
	POP	ES				; Restore extra segment
	POP	DS				; Restore data segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Since some assemblers can's use structures I will define
;	the temporary time storage as:

BCD_SEC		EQU	0			; Offset of seconds
BCD_SEC_ALRM	EQU	1			; Offset of seconds alarm
BCD_MIN		EQU	2			; Offset of minutes
BCD_MIN_ALRM	EQU	3			; Offset of minutes alarm
BCD_HRS		EQU	4			; Offset of hours
BCD_HRS_ALRM	EQU	5			; Offset of hours alarm
BCD_DAY		EQU	6			; Day of the week
BCD_DATE	EQU	7			; Date
BCD_MON		EQU	8			; Month
BCD_YRS		EQU	9			; Year
CMOS_COPY	EQU	10			; Where CMOS copy starts
CMOS_BYTES	EQU	10			; Bytes to get from CMOS
TOT_SECONDS	EQU	(CMOS_BYTES * 2)	; Where to store result
BCD_CENT	EQU	(TOT_SECONDS + 2)	; Century
LEAP_YEAR	EQU	(BCD_CENT + 2)		; Where to put leap-year
TOT_ALOC	EQU	(LEAP_YEAR + 4)		; Total local store size
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	"Thirty days hath September, April, June, and November. All the
;	rest have thirty-one except February hath twenty-eight or leap-
;	years twenty-nine."
;
ALIGN	4
DAYS		DB	31			; January
		DB	28			; February
		DB	31			; March
		DB	30			; April
		DB	31			; May
		DB	30			; June
		DB	31			; July
		DB	31			; August
		DB	30			; September
		DB	31			; October
		DB	30			; November
		DB	31			; December
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the BIOS time / date to the time of the CMOS clock.
;
ALIGN	4
SET_TIME:
	PUSH	BP				; Save frame-pointer
	MOV	BP,SP				; Save stack in BP
	SUB	SP,TOT_ALOC			; Allocate local storage
	MOV	BX,SP				; Index points to SP
	PUSH	DS				; Save segments
	PUSH	ES
	MOV	AX,SS				; Co-locate segments
	MOV	DS,AX				; Set DS
	MOV	ES,AX				; Set ES
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	I get two copies of the CMOS time contents and compare them.
;	If they are the same, no changes occurred while reading.
;	I read until there are no changes. This allows me to access
;	an active timer.
;
GET0:	MOV	DI,BX				; Local data area
	MOV	CX,CMOS_BYTES			; Bytes to get from CMOS
	XOR	AH,AH				; Start index at zero
GET1:	MOV	AL,AH				; Set index
	OUT	CMOS_INDEX,AL			; Tell device
	SLOW_IO					; delay
	IN	AL,CMOS_DATA			; Get result
	STOSB					; Put in local buffer
	INC	AH				; Ready next CMOS location
	LOOP	GET1				; Get all CMOS data
	MOV	SI,DI				; Index for compare
	MOV	CX,CMOS_BYTES			; Bytes we have to get
	XOR	AH,AH				; Start index at zero
GET2:	MOV	AL,AH				; Get index
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Delay
	IN	AL,CMOS_DATA			; Get CMOS data
	STOSB					; Put in local buffer
	INC	AH				; Ready next CMOS location
	LOOP	GET2				; Get all CMOS data
	MOV	AL,32H				; BCD century
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Delay
	IN	AL,CMOS_DATA			; Get BCD century
	MOV	[BX+BCD_CENT],AL		; Save century
	XOR	AL,AL				; Index zero
	OUT	CMOS_INDEX,AL			; Reset index zero
	SLOW_IO					; Delay
	MOV	CX,CMOS_BYTES			; Bytes we have
	MOV	DI,BX				; Location of first copy
	REPZ	CMPSB				; Compare copies
	JNZ	SHORT GET0			; Not the same, do again
;
	MOV	AL, [BX+BCD_SEC]		; Get BCD seconds
	CALL	BCD2BIN				; Convert
	MOV	[BX+TOT_SECONDS],AX		; Save
	MOV	AL,[BX+BCD_MIN]			; Get BCD minutes
	CALL	BCD2BIN				; Convert
	MOV	CX,60				; Seconds in a minute
	MUL	CX				; Times seconds per min
	ADD	[BX+TOT_SECONDS],AX		; Accumulate
	MOV	AL,[BX+BCD_HRS]			; Get BCD hours
	CALL	BCD2BIN				; Convert
	MOV	CX,3600				; Seconds in an hour
	MUL	CX				; Times seconds per min
	ADD	AX,[BX+TOT_SECONDS]		; Accumulate
	ADC	DX,0				; Any overflow
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Timer ticks occur 18.206 times per second. I use the 32-bit
;	instructions to do ticks = seconds * 18206 / 1000 to get
;	the correct value.
;

	SHL	EDX,10H				; Put into high
	AND	EAX,0FFFFH			; Clear high word
	OR	EAX,EDX				; Set high word
	MOV	ECX,18206			; 18,206 per second
	MUL	ECX				; Multiply by 18206
	XOR	EDX,EDX				; Ready for divide
	MOV	ECX,1000			; Divide by 1,000
	DIV	ECX				; EAX = 18.206 * seconds
	MOV	EDX,EAX				; Copy to EDX
	SHR	EDX,10H				; Move into DX
	PUSH	BX				; Save index
	PUSH	DS				; Save segment
	MOV	BX,BIOS_SEG			; Get BIOS segment
	MOV	DS,BX				; DS = BIOS
	XOR	BX,BX				; Index zero
	CLI					; Don't allow timer update
	MOV	[BX+TIMER_LOW],AX		; Set low word of time
	MOV	[BX+TIMER_HIG],DX		; Set high word of time
	STI					; Okay to interrupt
	POP	DS				; Restore segment
	POP	BX				; Restore index
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The following is probably ignored. The newer BIOS just read
;	stuff from the CMOS clock without ever touching the value
;	at 40:CE. However, here goes.
;
	MOV	AL,[BX+BCD_CENT]		; Get century
	CALL	BCD2BIN				; Convert to binary
	MOV	CX,100				; Convert century
	MUL	CX				; AX = years due to century
	MOV	CX,AX				; Save
	MOV	AL,[BX+BCD_YRS]			; Get current year
	CALL	BCD2BIN				; Convert to binary
	ADD	AX,CX				; AX = total years
	MOV	BYTE PTR [BX + LEAP_YEAR],0	; Assume no leap year
	TEST	AL,00000011B			; Check for a leap year
	JNZ	SHORT NOLEAP			; Not a leap year
	INC	BYTE PTR [BX + LEAP_YEAR]	; It is a leap year
NOLEAP:	SUB	AX,1980				; IBM Birthdate
	MOV	CX,AX				; Use as a count
	XOR	AX,AX				; Zero accumulator
CVTDY:	MOV	DX,365				; Years, not a leap year
	TEST	CL,00000011B			; See if divisible by 4
	JNZ	SHORT NOLY			; Nope
	INC	DX				; Yes
NOLY:	ADD	AX,DX				; Accumulate
	LOOP	CVTDY
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Ignoring the bit about century-leap-years, this should
;	have gotten us to the beginning of this year.
;	Now do the months up to, but not including this month.
;
	MOV	DX,AX				; Save days so far
	MOV	AL,[BX+BCD_MON]			; Get month
	CALL	BCD2BIN				; Convert
	MOV	CX,AX				; Month count
	SUB	CX,1				; Ignore this month, zero-base
	JBE	SHORT NOMONS			; No months so far
	PUSH	BX				; Save index
	PUSH	DS				; Save segment
	MOV	BX,CS				; Get code segment
	MOV	DS,BX				; DS = CS
	MOV	BX,OFFSET DAYS			; Point to table
DOMON:	ADD	DL,[BX]				; Accumulate days
	ADC	DH,0				; Take care of CY
	INC	BX				; Ready next
	LOOP	DOMON
	POP	DS				; Restore segment
	POP	BX				; Restore index
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Now take care of the current day-of-the-month. Note that
;	today has not completed yet, so don't include it.
;
NOMONS:	ADD	DL,[BX+ LEAP_YEAR]		; Adjust for leap-year
	ADC	DH,0				; Take care of CY
	MOV	AL,[BX+BCD_DATE]		; Pointer to date
	CALL	BCD2BIN				; Convery to binary
	DEC	AX				; Make zero based
	ADD	AX,DX				; Accumulate
	MOV	BX,BIOS_SEG			; Point to BIOS segment
	MOV	DS,BX				; DS = BIOS
	MOV	BX,DAYS_SINCE			; Where to put the result
	MOV	[BX],AX				; Set days since 1/1/80
	POP	ES				; Restore segments
	POP	DS
	MOV	SP,BP				; Collapse local storage
	POP	BP				; Restore frame-pointer
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the default Screen control vector, used if the
;	screen BIOS is not found.
ALIGN	4
INT_10H:
	CMP	AX,1A00H			; Read configuration?
	JZ	SHORT DS_CF			; Yes
	CMP	AH,DUMB_TERM			; Dumb screen mode?
	JNZ	NO_SCR				; Nope
;
;	Okay, here we are going to try to send data out RS-232C.
;
	PUSH	DX				; Save
	XOR	DX,DX				; First port
	MOV	AH,1				; Function code 1
	INT	SERIAL				; Call RS-232 functions
	POP	DX				; Restore
;
NO_SCR:	XOR	AX,AX				; Show no other functions
	JMP	SHORT SC_EX			; All done
DS_CF:	XOR	BX,BX				; Show no screen
SC_EX:	IRET					; No flags returned
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the interrupt service routine for the CMOS timer.
;
ALIGN	4
INT_IRQ8:
	PUSH	AX
	PUSH	BX
	PUSH	DS
	MOV	AL,0BH				; CMOS register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Read CMOS register 'B'
	SLOW_IO					; Slow down I/O
	MOV	AL,0CH				; CMOS register 'C'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Read CMOS register 'C'
	SLOW_IO					; Slow down I/O
	XOR	AL,AL				; Zero
	OUT	CMOS_INDEX,AL			; Leave at zero
	SLOW_IO					; Slow down I/O
	MOV	AL,NOS_EOI			; Non-specific end-of-inter
	OUT	ACK_CTL0,AL			; ACK first controller
	OUT	ACK_CTL1,AL			; ACK second controller
	MOV	AX,BIOS_SEG			; Get BIOS data segment
	MOV	DS,AX				; DS = BIOS
	MOV	BX,MFG_TEST_PORT		; Where to save data
	INC	BYTE PTR [BX]			; Bump data in area
	POP	DS
	POP	BX
	POP	AX
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This checks the main timer-channel-zero with the 12 millisecond
;	floppy-disk wait code.
;
ALIGN	4
CHK_TIMER0:
	MOV	SI,OFFSET CHKTIM		; Point to prompt
	CALL	PRINT				; Write to screen
	SET_TIMER 01H				; 1 timer tick
WTNEW:	WAIT_TIMER WTNEW			; Wait for new tick
	XOR	CX,CX				; Start count at zero
	SET_TIMER 01H				; Start new
WTCHG:	CALL	WAITUS				; Call wait routine
	INC	CX				; Bump counter
	WAIT_TIMER WTCHG			; Wait for a time-out
	PUSH	CX				; Save count
	MOV	SI,OFFSET CHKTOK		; Point to prompt
	CALL	PRINT				; Write to output
	POP	AX				; Restore count
	CALL	BIN2ASC				; Convert
	MOV	SI,OFFSET PCRLF			; End of line
	JMP	PRINT				; To screen, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This checks both the CMOS timer and the second-level
;	interrupt controller. It does this by enabling the CMOS
;	timer and watching it interrupt.
;
ALIGN	4
CHK_CMOS:
	MOV	SI,OFFSET CHKCMOS		; Point to prompt
	CALL	PRINT				; Write message
	PUSH	DS				; Save segments
	PUSH	ES
	XOR	AX,AX				; Get a zero
	MOV	ES,AX				; ES = 0
	MOV	DI,(IRQ8 SHL 2)			; Vector to patch
	CLI					; No interrupts
	MOV	AX,OFFSET INT_IRQ8		; Procedure
	STOSW					; Set offset
	MOV	AX,CS				; Current code segment
	STOSW					; Patch segment
	STI					; Allow interrupts
	IN	AL,MSK_CTL1			; Get current mask
	SLOW_IO					; Slow down I/O
	AND	AL,11111110B			; Turn ON first bit
;                  ||||||||____ IRQ8
;                  |||||||_____ IRQ9
;                  ||||||______ IRQ10
;                  |||||_______ IRQ11
;                  ||||________ IRQ12
;                  |||_________ IRQ13
;                  ||__________ IRQ14
;                  |___________ IRQ15
;
	OUT	MSK_CTL1,AL			; Set controller
	SLOW_IO					; Slow down I/O
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Program the CMOS realtime clock to generate a periodic interrupt
;	at 488.281 microseconds. The CMOS clock exists at ports CMOS_INDEX
;	and CMOS_DATA.
;
	MOV	AL,0AH				; Init/status register 'A'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	MOV	AL, 00100101B			; Initialize status register
;		    ||||||||___ RS<0:3>    0101 = 488.281 us
;		    ||||_______ DV<4:6>    010  = 32.768 kHz
;		    |__________ UPI<7>     Update in progress
;
	OUT	CMOS_DATA, AL			; Init
	SLOW_IO					; Slow down I/O
	MOV	AL,0BH				; Control register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL, CMOS_DATA			; Get current
	SLOW_IO					; Slow down I/O
	AND	AL, 00000011B			; Save DST/24hr mode
	OR	AL, 01000010B			; Turn on clock, 24 hr mode
;		    ||||||||___ DSE<0>     Daylight savings time = 1
;		    |||||||____ 12/24<1>   24-hour mode = 1
;		    ||||||_____ ZR<2:3>    Always zero
;		    ||||_______ UIE<4>     Update ended interrupt enable
;		    |||________ AIE<5>     Alarm interrupt enable
;		    ||_________ PIE<6>     Periodic interrupt enable
;		    |__________ SET<7>     Turn on updates
;
	OUT	CMOS_DATA,AL			; Set timer ON
	SLOW_IO					; Slow down I/O
	XOR	AL,AL				; Get a zero
	OUT	CMOS_INDEX,AL			; Leave index at 0
	SLOW_IO					; Slow down I/O
	INT	IRQ8				; Execute the interrupt
	MOV	AX,BIOS_SEG			; Get BIOS data segment
	MOV	DS,AX				; DS = BIOS
	MOV	BX,MFG_TEST_PORT		; Where to save data
	MOV	BYTE PTR [BX],0			; Clear data area
	CALL	WAITUS				; Wait just a bit
	MOV	CX,0FFFFH			; Loop count
CHKINT:	CMP	BYTE PTR [BX],0			; See if it changed
	LOOPZ	CHKINT
	MOV	AL,0BH				; CMOS register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL, CMOS_DATA			; Get current
	AND	AL, 00000011B			; Save DST/24hr mode
;		    ||||||||___ DSE<0>     Daylight savings time = 1
;		    |||||||____ 12/24<1>   24-hour mode = 1
;		    ||||||_____ ZR<2:3>    Always zero
;		    ||||_______ UIE<4>     Update ended interrupt enable
;		    |||________ AIE<5>     Alarm interrupt enable
;		    ||_________ PIE<6>     Periodic interrupt enable
;		    |__________ SET<7>     Turn on updates
;
	OUT	CMOS_DATA,AL
	SLOW_IO					; Slow down I/O
	MOV	AL,0BH				; CMOS register 'B'
	OUT	CMOS_INDEX,AL			; Set the index
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA 			; Read CMOS register 'B'
	SLOW_IO					; Slow down I/O
	MOV	AL,0CH				; CMOS register 'C'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA 			; Read CMOS register 'C'
	SLOW_IO					; Slow down I/O
	XOR	AL,AL				; For index of zero
	OUT	CMOS_INDEX,AL			; Leave at offset zero
	SLOW_IO					; Slow down I/O
	IN	AL,MSK_CTL1			; Get current mask
	SLOW_IO					; Slow down I/O
	OR	AL,00000001B			; Turn OFF first bit
;                  ||||||||____ IRQ8
;                  |||||||_____ IRQ9
;                  ||||||______ IRQ10
;                  |||||_______ IRQ11
;                  ||||________ IRQ12
;                  |||_________ IRQ13
;                  ||__________ IRQ14
;                  |___________ IRQ15
;
	OUT	MSK_CTL1,AL			; Set controller
	SLOW_IO					; Slow down I/O
	POP	ES				; Restore segments
	POP	DS
	MOV	AL,[BX]				; Get result
	PUSH	AX				; Save for later
	MOV	SI,OFFSET CHKTOK		; Assume it's okay
	OR	CX,CX				; Loop-count can't be zero!
	JNZ	SHORT CTLOK			; Controller was okay
	MOV	SI,OFFSET CTLFAL		; Point to message
CTLOK:	CALL	PRINT				; Print the message
	POP	AX				; Restore result
	CALL	HEXB				; Write to screen
	MOV	SI,OFFSET PCRLF			; End of line
	JMP	PRINT				; To screen, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This does a BIOS module scan and installs any modules found.
;
ALIGN	4
SET_MODULES:
	MOV	AL,CMOS_BOOTF		; Boot flag
	CALL	GET_CMOS		; Get flag
	CMP	AL,'N'			; Is it NVRAM?
	JZ	SHORT NVRAM		; Yes
	RET				; No, don't install it
NVRAM:	MOV	SI,OFFSET ROMMOD	; Point to prompt
	CALL	PRINT			; Print it
	PUSH	DS			; Save segment registers
	PUSH	ES
	MOV	SI,BIOS_SCAN_START	; Possible BIOS module location
SCAN:	PUSHF				; May screw up flags (STI/CLD)
	PUSH	SI			; Save scan location
	CALL	CHKSUM_MOD		; Checksum
	OR	AX,AX			; See if it's there
	JNZ	SHORT NEXTM		; No, next module
	MOV	AX,BIOS_SEG		; Get BIOS segment
	MOV	DS,AX			; Into data segment
	MOV	ES,AX			; Into extra segment
	MOV	AX,SI			; Segment of module
	CALL	HEXW			; Write to screen
	MOV	AX,0E20H		; Dumb terminal mode
	INT	SCREEN			; Write to console
	PUSH	CS			; Current segment
	PUSH	OFFSET NEXTM		; Where to go later
	PUSH	SI			; Segment of feature card
	PUSH	3			; Offset to 'call'
	RETF				; Far 'return' to BIOS module
;
NEXTM:	POP	SI			; Restore scan location
	POPF				; Restore flags
	ADD	SI,BIOS_SCAN_INCR	; Ready next location
	CMP	SI,BIOS_SCAN_END	; Are we done?
	JC	SHORT SCAN		; No.
	POP	ES			; Restore segment registers
	POP	DS
	MOV	SI,OFFSET CHKTOK	; Show it's okay
	CALL	PRINT			; Write to screen
	MOV	SI,OFFSET PCRLF		; Point to prompt
	JMP	PRINT			; Print, implied return
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is hardware interrupt 0, the divide by zero interrupt.
;	This is NotGood(tm). Therefore we write a message and wait
;	forever.
;
ALIGN	4
INT_00:
	MOV	SI,OFFSET DIVZERO		; Point to the prompt
	CALL	PRINT				; Print to screen
	JMP	SHORT RETADR			; Show return address
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the single-step interrupt (and others). It just returns.
;
ALIGN	4
INT_IGN:
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the mon-maskable interrupt. This is NotGood(tm). It
;	generally means that there was a memory error. It prints the
;	approximate address of the failing location and waits forever.
;
;	On some mother-boards, the BIOS prints:
;		PEARTY ERROR BUT SEGMENT NOT FIND (sic)
;
ALIGN	4
INT_02:
	MOV	SI,OFFSET PARITY		; Point to message
	CALL	PRINT				; Print message
RETADR:	POP	BX				; Offset address
	POP	AX				; Segment address
	PUSH	BX				; Save offset address
	CALL	HEXW				; Print the segment
	MOV	AL,':'				; Delimiter
	MOV	AH,DUMB_TERM			; Dumb terminal mode
	XOR	BX,BX				; Page zero
	INT	SCREEN				; Screen BIOS
	POP	AX				; Offset
	CALL	HEXW				; Print offset
	MOV	SI,OFFSET PCRLF			; End of line
	CALL	PRINT				; Print it
	STI					; Interrupts on
	JMP	$				; Wait
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	IRQ2 is cascaded. This interrupt is not supposed to happen.
;	But, hardware being what it is today, it just might. We need
;	to handle it or the cascade might not work.
;
ALIGN	4
IRQ_02:
	PUSH	AX				; Save register
	PUSH	BX				; Save index
	PUSH	DS				; Save segment
	MOV	BX,BIOS_SEG			; Get BIOS segment
	MOV	DS,BX				; DS = BIOS data
	MOV	AL,NOS_EOI			; Non-specific EOI
	OUT	ACK_CTL0,AL			; ACK the controller
	MOV	BX,LAST_INTER			; Last unexpected interrupt
	INC	BYTE PTR [BX]			; Bump flag
	POP	DS				; Restore segment
	POP	BX				; Restore index
	POP	AX				; Restore register
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the debugging interrupt. Unfortunately it has two
;	forms, both the long form and the short form. This makes the
;	code very complicated ;^)
;
ALIGN	4
INT_03:	IRET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	IRQ9 is cascaded. This interrupt is not supposed to happen.
;	But, hardware being what it is today, it just might. We need
;	to handle it or the cascade might not work.
;
ALIGN	4
IRQ_09:
	PUSH	AX				; Save register
	MOV	AL,NOS_EOI			; Non-specific EOI
	OUT	ACK_CTL1,AL			; ACK the controller
	POP	AX				; Restore register
	INT	IRQ2				; Execute its parent
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This handles the coprocessor interrupt if it barfs during
;	boot. This is generally handled by a runtime library once
;	the system is up.
;
ALIGN	4
IRQ_13:
	PUSH	AX				; Save register
	MOV	AL,NOS_EOI			; Non-specific EOI
	OUT	ACK_CTL1,AL			; ACK second controller
	OUT	ACK_CTL0,AL			; ACK first controller
	XOR	AL,AL				; Zero
	OUT	COPROCESSOR,AL			; Quiet the device
	POP	AX				; Restore register
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the first 7 initial interrupts in the interrupt vector
;	table.
;
;	It also sets the unexpected interrupt handlers for the cascaded
;	interrupts, IRQ2 and IRQ9 and the coprocessor at IRQ13.
;
ALIGN	4
SET_INIT_INTERS:
	PUSHF
	CLI					; No interrupts
	PUSH	ES				; Save segment
	XOR	AX,AX				; Get a zero
	MOV	ES,AX				; ES = 0
	MOV	DI,AX				; ES:DI = 0
	MOV	AX,OFFSET INT_00		; Divide / 0 interrupt
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_IGN		; Single-step
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_02		; Mon-maskable
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_03		; Debug (INT) interrupt
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_IGN		; INTO (overflow) interrupt
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_IGN		; Print-screen interrupt
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_06		; Reserved interrupt 6
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	AX, OFFSET INT_IGN		; Reserved interrupt 7
	STOSW					; Set offset
	MOV	AX,CS				; Current segment
	STOSW					; Set segment
	MOV	DI, (IRQ2 SHL 2)		; Unexpected interrupt
	MOV	AX,OFFSET IRQ_02		; Unexpected interrupt offset
	STOSW					; Set offset
	MOV	AX,CS				; Get current segment
	STOSW					; Set segment
	MOV	DI, (IRQ9 SHL 2)		; Unexpected interrupt
	MOV	AX,OFFSET IRQ_09		; Unexpected interrupt offset
	STOSW					; Set offset
	MOV	AX,CS				; Get current segment
	STOSW					; Set segment
	MOV	DI, (IRQ13 SHL 2)		; Coprocessor interrupt
	MOV	AX,OFFSET IRQ_13		; Coprocessor handler offset
	STOSW					; Set offset
	MOV	AX,CS				; Get current segment
	STOSW					; Set segment
	MOV	DI, (IRQ7 SHL 2)		; Line printer
	MOV	AX,OFFSET DUMMY			; Temporary vector
	STOSW					; Set offset
	MOV	AX,CS				; Get current segment
	STOSW					; Set segment
	MOV	DI, (IRQ4 SHL 2)		; RS-232C
	MOV	AX,OFFSET RS232_ISR		; RS-232C vector
	STOSW					; Set offset
	MOV	AX,CS				; Get current segment
	STOSW					; Set segment
	POP	ES				; Restore segment
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is a simple timer-channel 0 interrupt. It's been programmed
;	to interrupt at 18.206 times per second.
;
ALIGN	4
TIMER0_ISR:
	PUSH	AX
	PUSH	BX
	PUSH	DS
	MOV	AL,NOS_EOI			; Non-Specific EIO
	OUT	ACK_CTL0,AL			; Out base controller
	MOV	AX,BIOS_SEG			; Get BIOS data
	MOV	DS,AX				; DS = BIOS data
	XOR	BX,BX				; For NASM compatibility
	SUB	WORD PTR [BX+INTV_TIM],1	; Bump private timer
	ADC	WORD PTR [BX+INTV_TIM],0	; But not below zero
	ADD	WORD PTR [BX+TIMER_LOW],1	; INC dosen't set carry
	ADC	WORD PTR [BX+TIMER_HIG],0	; Overflow into high word
	CMP	WORD PTR [BX+TIMER_HIG],TIMER_MAX	; Ready to wrap?
	JC	SHORT NODAY				; No
	CMP	WORD PTR [BX+TIMER_LOW],TIMER_DAT	; New day yet?
	JNZ	SHORT NODAY			; Not a new day yet
	MOV	WORD PTR [BX+TIMER_LOW],0	; Yes, zero low word
	MOV	WORD PTR [BX+TIMER_HIG],0	; Zero high word
	MOV	WORD PTR [BX+TIMER_DAY],1	; Show a new day
NODAY:	CMP	BYTE PTR [BX+DISK_MOT_TIM],0	; Diskette motor?
	JZ	SHORT NO_MOT			; No, It's off
	DEC	BYTE PTR [BX+DISK_MOT_TIM]	; Bump the count
	JNZ	SHORT NO_MOT			; Not yet
	CALL	MOTOR_OFF			; Turn off diskette motor
NO_MOT:	INT	USER_TICK			; Execute user code.
	POP	DS
	POP	BX
	POP	AX
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This puts a key value in AX into the keyboard buffer.
;	If the buffer is full, we throw away the key (no bell).
;	All registers except AX are saved.
;
;	The method used in the IBM reference manual sucks. I need
;	to have some sort of compatibility with the head/tail pointer
;	values so I have some extra instructions. However, this
;	is quite a bit better. Only one jump, and the code is much
;	shorter.
;
ALIGN	4
PUT_KEY:
	PUSHF				; Save flags
	PUSH	BX			; Save registers
	PUSH	DX
	CLI				; Interrupts OFF
	MOV	BX,KBD_TAIL		; Get tail pointer offset
	MOV	DX,[BX]			; Get tail pointer
	SUB	DX,(KBD_BUF + 2)	; Now -2 to 31
	AND	DX,1FH			; Now 0 to 31
	MOV	BX,KBD_HEAD		; Get head pointer offset
	MOV	BX,[BX]			; Get pointer
	SUB	BX,KBD_BUF		; Now 0 to 31
	CMP	DX,BX			; Check for end
	JZ	SHORT FULL		; No room at the inn
	MOV	DX,BX			; Save pointer
	ADD	BX,KBD_BUF		; Adjust offset
	MOV	[BX],AX			; Put word into buffer
	ADD	DX,2			; Ready for next WORD
	AND	DX,1FH			; Wrap to buffer beginning
	ADD	DX,KBD_BUF		; Restore offset
	MOV	BX,KBD_HEAD		; Keyboard head offset
	MOV	[BX],DX			; Update head pointer
FULL:	POP	DX			; Restore registers
	POP	BX
	POPF				; Restore flags
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This gets a value from the keyboard buffer. If there is
;	nothing in the buffer, AX = 0 upon return.
;
;	The method used in the IBM reference manual sucks. I need
;	to have some sort of compatibility with the head/tail pointer
;	values so I have some extra instructions. However, this
;	is quite a bit better. Only one jump, and the code is much
;	shorter.
;
ALIGN	4
GET_KEY:
	PUSHF				; Save flags
	PUSH	BX			; Save registers
	PUSH	DX
	CLI				; Interrupts OFF
	MOV	BX,KBD_TAIL		; Get tail pointer offset
	MOV	DX,[BX]			; Get tail pointer
	MOV	BX,KBD_HEAD		; Get head pointer offset
	MOV	AX,[BX]			; Get head pointer
	SUB	AX,DX			; See if the same
	JZ	SHORT KEXIT		; Yes, no data available
	MOV	BX,DX			; Tail pointer
	MOV	AX,[BX]			; Get data
	SUB	BX,(KBD_BUF - 2)	; Index 2 to 33
	AND	BX,1FH			; Again 0 to 31
	ADD	BX,KBD_BUF		; Reset offset
	MOV	DX,BX			; Save new tail pointer
	MOV	BX,KBD_TAIL		; Get tail pointer offset
	MOV	[BX],DX			; Set new tail pointer
KEXIT:	POP	DX			; Restore registers
	POP	BX
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This gets the keyboard status. If there is nothing in the buffer,
;	it returns 0. If there is something in the buffer, it returns
;	the next key without removing it from the buffer.
;
ALIGN	4
KEY_STATUS:
	CLI				; No interrupts
	PUSH	BX			; Save registers
	PUSH	DX
	MOV	BX,KBD_HEAD		; Get head pointer offset
	MOV	DX,[BX]			; Get head pointer
	MOV	BX,KBD_TAIL		; Get tail pointer offset
	MOV	AX,[BX]			; Get tail pointer
	MOV	BX,AX			; Save tail pointer
	SUB	AX,DX			; See if the same
	JZ	SHORT NOKEY		; Yes, no data available
	MOV	AX,[BX]			; Get data available
NOKEY:	POP	DX			; Restore registers
	POP	BX
	STI				; Allow interrupts
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is the keyboard scan-code translation table if no special
;	keys are active, i.e., lower case, no Alt.
;
;	If 80H is the translated result, the scan-code will not be
;	put into the output buffer (Caps Lock, etc).
;
ALIGN	4
XTAB_NOR:
	DB	000H	; Scancode 00
	DB	01BH	; Scancode 01
	DB	031H	; Scancode 02
	DB	032H	; Scancode 03
	DB	033H	; Scancode 04
	DB	034H	; Scancode 05
	DB	035H	; Scancode 06
	DB	036H	; Scancode 07
	DB	037H	; Scancode 08
	DB	038H	; Scancode 09
	DB	039H	; Scancode 0A
	DB	030H	; Scancode 0B
	DB	02DH	; Scancode 0C
	DB	03DH	; Scancode 0D
	DB	008H	; Scancode 0E
	DB	009H	; Scancode 0F
	DB	071H	; Scancode 10
	DB	077H	; Scancode 11
	DB	065H	; Scancode 12
	DB	072H	; Scancode 13
	DB	074H	; Scancode 14
	DB	079H	; Scancode 15
	DB	075H	; Scancode 16
	DB	069H	; Scancode 17
	DB	06FH	; Scancode 18
	DB	070H	; Scancode 19
	DB	05BH	; Scancode 1A
	DB	05DH	; Scancode 1B
	DB	00DH	; Scancode 1C
	DB	080H	; Scancode 1D
	DB	061H	; Scancode 1E
	DB	073H	; Scancode 1F
	DB	064H	; Scancode 20
	DB	066H	; Scancode 21
	DB	067H	; Scancode 22
	DB	068H	; Scancode 23
	DB	06AH	; Scancode 24
	DB	06BH	; Scancode 25
	DB	06CH	; Scancode 26
	DB	03BH	; Scancode 27
	DB	027H	; Scancode 28
	DB	060H	; Scancode 29
	DB	000H	; Scancode 2A
	DB	05CH	; Scancode 2B
	DB	07AH	; Scancode 2C
	DB	078H	; Scancode 2D
	DB	063H	; Scancode 2E
	DB	076H	; Scancode 2F
	DB	062H	; Scancode 30
	DB	06EH	; Scancode 31
	DB	06DH	; Scancode 32
	DB	02CH	; Scancode 33
	DB	02EH	; Scancode 34
	DB	02FH	; Scancode 35
	DB	080H	; Scancode 36
	DB	02AH	; Scancode 37
	DB	080H	; Scancode 38
	DB	020H	; Scancode 39
	DB	080H	; Scancode 3A
	DB	000H	; Scancode 3B
	DB	000H	; Scancode 3C
	DB	000H	; Scancode 3D
	DB	000H	; Scancode 3E
	DB	000H	; Scancode 3F
	DB	000H	; Scancode 40
	DB	000H	; Scancode 41
	DB	000H	; Scancode 42
	DB	000H	; Scancode 43
	DB	000H	; Scancode 44
	DB	080H	; Scancode 45
	DB	080H	; Scancode 46
	DB	000H	; Scancode 47
	DB	000H	; Scancode 48
	DB	000H	; Scancode 49
	DB	02DH	; Scancode 4A
	DB	000H	; Scancode 4B
	DB	000H	; Scancode 4C
	DB	000H	; Scancode 4D
	DB	02BH	; Scancode 4E
	DB	000H	; Scancode 4F
	DB	000H	; Scancode 50
	DB	000H	; Scancode 51
	DB	000H	; Scancode 52
	DB	000H	; Scancode 53
	DB	000H	; Scancode 54
	DB	000H	; Scancode 55
	DB	000H	; Scancode 56
	DB	080H	; Scancode 57
	DB	080H	; Scancode 58
	DB	000H	; Scancode 59
	DB	000H	; Scancode 5A
	DB	000H	; Scancode 5B
	DB	000H	; Scancode 5C
	DB	000H	; Scancode 5D
	DB	000H	; Scancode 5E
	DB	000H	; Scancode 5F
	DB	000H	; Scancode 60
	DB	000H	; Scancode 61
	DB	000H	; Scancode 62
	DB	000H	; Scancode 63
	DB	000H	; Scancode 64
	DB	000H	; Scancode 65
	DB	000H	; Scancode 66
	DB	000H	; Scancode 67
	DB	000H	; Scancode 68
	DB	000H	; Scancode 69
	DB	000H	; Scancode 6A
	DB	000H	; Scancode 6B
	DB	000H	; Scancode 6C
	DB	000H	; Scancode 6D
	DB	000H	; Scancode 6E
	DB	000H	; Scancode 6F
	DB	000H	; Scancode 70
	DB	000H	; Scancode 71
	DB	000H	; Scancode 72
	DB	000H	; Scancode 73
	DB	000H	; Scancode 74
	DB	000H	; Scancode 75
	DB	000H	; Scancode 76
	DB	000H	; Scancode 77
	DB	000H	; Scancode 78
	DB	000H	; Scancode 79
	DB	000H	; Scancode 7A
	DB	000H	; Scancode 7B
	DB	000H	; Scancode 7C
	DB	000H	; Scancode 7D
	DB	000H	; Scancode 7E
	DB	000H	; Scancode 7F
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the scan-code translation if the characters are "shifted".
;	Shifted means that the Cap Lock key is active or the shift key
;	is active, but not both.
;
ALIGN	4
XTAB_SHF:
	DB	000H	; Scancode 00
	DB	01BH	; Scancode 01
	DB	021H	; Scancode 02
	DB	040H	; Scancode 03
	DB	023H	; Scancode 04
	DB	024H	; Scancode 05
	DB	025H	; Scancode 06
	DB	05EH	; Scancode 07
	DB	026H	; Scancode 08
	DB	02AH	; Scancode 09
	DB	028H	; Scancode 0A
	DB	029H	; Scancode 0B
	DB	05FH	; Scancode 0C
	DB	02BH	; Scancode 0D
	DB	008H	; Scancode 0E
	DB	009H	; Scancode 0F
	DB	051H	; Scancode 10
	DB	057H	; Scancode 11
	DB	045H	; Scancode 12
	DB	052H	; Scancode 13
	DB	054H	; Scancode 14
	DB	059H	; Scancode 15
	DB	055H	; Scancode 16
	DB	049H	; Scancode 17
	DB	04FH	; Scancode 18
	DB	050H	; Scancode 19
	DB	07BH	; Scancode 1A
	DB	07DH	; Scancode 1B
	DB	00DH	; Scancode 1C
	DB	080H	; Scancode 1D
	DB	041H	; Scancode 1E
	DB	053H	; Scancode 1F
	DB	044H	; Scancode 20
	DB	046H	; Scancode 21
	DB	047H	; Scancode 22
	DB	048H	; Scancode 23
	DB	04AH	; Scancode 24
	DB	04BH	; Scancode 25
	DB	04CH	; Scancode 26
	DB	03AH	; Scancode 27
	DB	022H	; Scancode 28
	DB	07EH	; Scancode 29
	DB	080H	; Scancode 2A
	DB	07CH	; Scancode 2B
	DB	05AH	; Scancode 2C
	DB	058H	; Scancode 2D
	DB	043H	; Scancode 2E
	DB	056H	; Scancode 2F
	DB	042H	; Scancode 30
	DB	04EH	; Scancode 31
	DB	04DH	; Scancode 32
	DB	03CH	; Scancode 33
	DB	03EH	; Scancode 34
	DB	03FH	; Scancode 35
	DB	080H	; Scancode 36
	DB	02AH	; Scancode 37
	DB	080H	; Scancode 38
	DB	020H	; Scancode 39
	DB	080H	; Scancode 3A
	DB	080H	; Scancode 3B
	DB	080H	; Scancode 3C
	DB	080H	; Scancode 3D
	DB	080H	; Scancode 3E
	DB	080H	; Scancode 3F
	DB	080H	; Scancode 40
	DB	080H	; Scancode 41
	DB	080H	; Scancode 42
	DB	080H	; Scancode 43
	DB	080H	; Scancode 44
	DB	080H	; Scancode 45
	DB	080H	; Scancode 46
	DB	000H	; Scancode 47
	DB	000H	; Scancode 48
	DB	000H	; Scancode 49
	DB	02DH	; Scancode 4A
	DB	034H	; Scancode 4B
	DB	035H	; Scancode 4C
	DB	036H	; Scancode 4D
	DB	02BH	; Scancode 4E
	DB	000H	; Scancode 4F
	DB	000H	; Scancode 50
	DB	000H	; Scancode 51
	DB	030H	; Scancode 52
	DB	000H	; Scancode 53
	DB	000H	; Scancode 54
	DB	000H	; Scancode 55
	DB	000H	; Scancode 56
	DB	080H	; Scancode 57
	DB	080H	; Scancode 58
	DB	000H	; Scancode 59
	DB	000H	; Scancode 5A
	DB	000H	; Scancode 5B
	DB	000H	; Scancode 5C
	DB	000H	; Scancode 5D
	DB	000H	; Scancode 5E
	DB	000H	; Scancode 5F
	DB	000H	; Scancode 60
	DB	000H	; Scancode 61
	DB	000H	; Scancode 62
	DB	000H	; Scancode 63
	DB	000H	; Scancode 64
	DB	000H	; Scancode 65
	DB	000H	; Scancode 66
	DB	000H	; Scancode 67
	DB	000H	; Scancode 68
	DB	000H	; Scancode 69
	DB	000H	; Scancode 6A
	DB	000H	; Scancode 6B
	DB	000H	; Scancode 6C
	DB	000H	; Scancode 6D
	DB	000H	; Scancode 6E
	DB	000H	; Scancode 6F
	DB	000H	; Scancode 70
	DB	000H	; Scancode 71
	DB	000H	; Scancode 72
	DB	000H	; Scancode 73
	DB	000H	; Scancode 74
	DB	000H	; Scancode 75
	DB	000H	; Scancode 76
	DB	000H	; Scancode 77
	DB	000H	; Scancode 78
	DB	000H	; Scancode 79
	DB	000H	; Scancode 7A
	DB	000H	; Scancode 7B
	DB	000H	; Scancode 7C
	DB	000H	; Scancode 7D
	DB	000H	; Scancode 7E
	DB	000H	; Scancode 7F
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the scan-code translation if the control key is depressed.
;
ALIGN	4
XTAB_CTL:
	DB	000H	; Scancode 00
	DB	080H	; Scancode 01
	DB	080H	; Scancode 02
	DB	003H	; Scancode 03
	DB	080H	; Scancode 04
	DB	080H	; Scancode 05
	DB	080H	; Scancode 06
	DB	01EH	; Scancode 07
	DB	080H	; Scancode 08
	DB	080H	; Scancode 09
	DB	080H	; Scancode 0A
	DB	080H	; Scancode 0B
	DB	01FH	; Scancode 0C
	DB	080H	; Scancode 0D
	DB	07FH	; Scancode 0E
	DB	080H	; Scancode 0F
	DB	011H	; Scancode 10
	DB	017H	; Scancode 11
	DB	005H	; Scancode 12
	DB	012H	; Scancode 13
	DB	014H	; Scancode 14
	DB	019H	; Scancode 15
	DB	015H	; Scancode 16
	DB	009H	; Scancode 17
	DB	00FH	; Scancode 18
	DB	010H	; Scancode 19
	DB	01BH	; Scancode 1A
	DB	01DH	; Scancode 1B
	DB	00AH	; Scancode 1C
	DB	080H	; Scancode 1D
	DB	001H	; Scancode 1E
	DB	013H	; Scancode 1F
	DB	004H	; Scancode 20
	DB	006H	; Scancode 21
	DB	007H	; Scancode 22
	DB	008H	; Scancode 23
	DB	00AH	; Scancode 24
	DB	00BH	; Scancode 25
	DB	00CH	; Scancode 26
	DB	080H	; Scancode 27
	DB	080H	; Scancode 28
	DB	080H	; Scancode 29
	DB	080H	; Scancode 2A
	DB	080H	; Scancode 2B
	DB	01AH	; Scancode 2C
	DB	018H	; Scancode 2D
	DB	003H	; Scancode 2E
	DB	016H	; Scancode 2F
	DB	002H	; Scancode 30
	DB	00EH	; Scancode 31
	DB	00DH	; Scancode 32
	DB	080H	; Scancode 33
	DB	080H	; Scancode 34
	DB	080H	; Scancode 35
	DB	080H	; Scancode 36
	DB	080H	; Scancode 37
	DB	080H	; Scancode 38
	DB	020H	; Scancode 39
	DB	080H	; Scancode 3A
	DB	080H	; Scancode 3B
	DB	080H	; Scancode 3C
	DB	080H	; Scancode 3D
	DB	080H	; Scancode 3E
	DB	080H	; Scancode 3F
	DB	080H	; Scancode 40
	DB	080H	; Scancode 41
	DB	080H	; Scancode 42
	DB	080H	; Scancode 43
	DB	080H	; Scancode 44
	DB	080H	; Scancode 45
	DB	080H	; Scancode 46
	DB	000H	; Scancode 47
	DB	080H	; Scancode 48
	DB	080H	; Scancode 49
	DB	080H	; Scancode 4A
	DB	000H	; Scancode 4B
	DB	080H	; Scancode 4C
	DB	080H	; Scancode 4D
	DB	080H	; Scancode 4E
	DB	000H	; Scancode 4F
	DB	000H	; Scancode 50
	DB	000H	; Scancode 51
	DB	030H	; Scancode 52
	DB	000H	; Scancode 53
	DB	000H	; Scancode 54
	DB	000H	; Scancode 55
	DB	000H	; Scancode 56
	DB	080H	; Scancode 57
	DB	080H	; Scancode 58
	DB	000H	; Scancode 59
	DB	000H	; Scancode 5A
	DB	000H	; Scancode 5B
	DB	000H	; Scancode 5C
	DB	000H	; Scancode 5D
	DB	000H	; Scancode 5E
	DB	000H	; Scancode 5F
	DB	000H	; Scancode 60
	DB	000H	; Scancode 61
	DB	000H	; Scancode 62
	DB	000H	; Scancode 63
	DB	000H	; Scancode 64
	DB	000H	; Scancode 65
	DB	000H	; Scancode 66
	DB	000H	; Scancode 67
	DB	000H	; Scancode 68
	DB	000H	; Scancode 69
	DB	000H	; Scancode 6A
	DB	000H	; Scancode 6B
	DB	000H	; Scancode 6C
	DB	000H	; Scancode 6D
	DB	000H	; Scancode 6E
	DB	000H	; Scancode 6F
	DB	000H	; Scancode 70
	DB	000H	; Scancode 71
	DB	000H	; Scancode 72
	DB	000H	; Scancode 73
	DB	000H	; Scancode 74
	DB	000H	; Scancode 75
	DB	000H	; Scancode 76
	DB	000H	; Scancode 77
	DB	000H	; Scancode 78
	DB	000H	; Scancode 79
	DB	000H	; Scancode 7A
	DB	000H	; Scancode 7B
	DB	000H	; Scancode 7C
	DB	000H	; Scancode 7D
	DB	000H	; Scancode 7E
	DB	000H	; Scancode 7F
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets the translate table, based upon the status of the
;	various keyboard shift/alt/lock keys. It returns the address
;	of the table in BX. All other registers are saved.
;
ALIGN	4
GET_XLATE_TABLE:
	PUSH	AX				; Save
	MOV	BX,KEY_SHIFT			; Point to shift status
	MOV	BX,[BX]				; Get shift and xshift
	MOV	AX,OFFSET XTAB_NOR		; AX points to default table
	TEST	BL,ANY_CTL			; Check for control-key
	JZ	SHORT NOCTL			; Not a control-key
	MOV	AX, OFFSET XTAB_CTL		; Point to control-table
	JMP	SHORT FOUND			; Wa are done
NOCTL:	TEST	BL,(RIGHT_SHIFT OR LEFT_SHIFT)	; Check shift-key
	JZ	SHORT NOSHFT			; Not shifted
	TEST	BL,COPY_CAP			; Shifted, check Caps Lock
	JNZ	SHORT FOUND			; Was Caps, make no caps
	JMP	SHORT ISCAP			; Leave it at Caps
NOSHFT:	TEST	BL,COPY_CAP			; Not shifted, check Caps
	JZ	SHORT FOUND			; Not Caps
ISCAP:	MOV	AX,OFFSET XTAB_SHF		; It is caps
FOUND:	MOV	BX,AX				; Put selection into BX
	POP	AX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets a BCD value in register AL, and returns with its
;	binary value in AL.
;
ALIGN	4
BCD2BIN:
	MOV	AH,AL				; Copy
	SHR	AH,4				; Move off the low nibble
	AND	AL,00001111B			; Mask off the high nibble
	AAD					; Make binary
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets a binary value in register AL, and returns with its
;	BCD value in AL.
;
ALIGN	4
BIN2BCD:
	AAM					; Make BCD (sort of)
	SHL	AH,4				; Move to high nibble
	OR	AL,AH				; Combine
	XOR	AH,AH				; Clear this
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes a hex DWORD to the screen. This is used for testing.
ALIGN	4
HEXD:	PUSH	EAX			; Save
	SHR	EAX,10H			; High word into low
	CALL	HEXW			; Write high word
	POP	EAX			; Restore, fall through
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes a hex WORD to the screen. This is used for testing.
;
HEXW:	PUSH	AX			; Save
	MOV	AL,AH			; Get high byte
	CALL	HEXB			; Write the high byte
	POP	AX			; Restore low byte
					; Fall through
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes a hex BYTE to the screen. This is used for testing.
;
HEXB:	MOV	AH,AL			; Save byte copy
	SHR	AL,4			; Isolate high nibble
	CALL	HEXN			; Write high nibble
	MOV	AL,AH			; Restore saved byte
	AND	AL,00001111B		; Isolate low nibble, fall through
HEXN:					; Write a hex nibble to screen
	ADD	AL,90H			; Add bias
	DAA				; Old Intel trick
	ADC	AL,40H			; Pick up CY into high nibble
	DAA				; Readjust
	PUSH	AX
	MOV	AH,DUMB_TERM		; Dumb terminal
	XOR	BX,BX			; Page zero
	INT	SCREEN			; Write via screen BIOS
	POP	AX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Don't blame me. I didn't invent this. The IBM/PC keyboard
;	is "strange" to say the least. For instance, the "Caps Lock"
;	is a toggle. However, the "Shift" key reverses the state of
;	the "Caps Lock".
;
;	Upon entry, we have the raw scan-code in AX. This value must be
;	saved.
;
;	We check for various control keys and handle them. We don't
;	complain if we don't know what to do with them.
;
ALIGN	4
DO_KEYS:
	PUSH	AX
	XOR	BX,BX
	CMP	AL,SC_DEL		; Delete key?
	JNZ	SHORT NO_DEL		; Nope
	TEST	BYTE PTR [BX+KEY_SHIFT],ANY_CTL
	JZ	SHORT NO_DEL		; Nope
	TEST	BYTE PTR [BX+KEY_SHIFT],ANY_ALT
	JZ	SHORT NO_DEL		; Nope
	JMP	CTL_ALT_DEL		; Kill the system
NO_DEL:	CMP	AL,SC_LF_SHF		; Left shift?
	JNZ	SHORT X00		; No
	JMP	SET_LEFT_SHF		; Yes
X00:	CMP	AL,(SC_LF_SHF OR 80H)	; Release left shift?
	JNZ	SHORT X01		; No
	JMP	SHORT RELEASE_SHF	; Yes
X01:	CMP	AL,SC_RT_SHF		; Right shift?
	JNZ	SHORT X02		; No
	JMP	SET_RIGHT_SHF		; Yes
X02:	CMP	AL,(SC_RT_SHF OR 80H)	; Release right shift?
	JNZ	SHORT X03		; No
	JMP	SHORT RELEASE_SHF	; Yes
X03:	CMP	AL,SC_LF_CTL		; Left control?
	JNZ	SHORT X04		; No
	JMP	SET_LEFT_CTL		; Yes
X04:	CMP	AL,(SC_LF_CTL OR 80H)	; Release left control?
	JNZ	SHORT X05		; No
	JMP	SHORT RELEASE_CTL	; Yes
X05:	CMP	AL,SC_LF_ALT		; Left Alt?
	JNZ	SHORT X06		; No
	JMP	SET_LEFT_ALT		; Yes
X06:	CMP	AL,(SC_LF_ALT OR 80H)	; Release left alt?
	JNZ	SHORT X07		; No
	JMP	SHORT RELEASE_ALT	; Yes
X07:	CMP	AL,SC_RT_ALT		; Right alt?
	JNZ	SHORT X08		; No
	JMP	SET_RIGHT_ALT		; Yes
X08:	CMP	AL,(SC_RT_ALT OR 80H)	; Release right alt?
	JNZ	SHORT X09		; No
	JMP	SHORT RELEASE_ALT	; Yes
X09:	CMP	AL,SC_INS		; Insert?
	JNZ	SHORT X0A		; No
	JMP	SET_INS			; Yes
X0A:	CMP	AL,(SC_INS OR 80H)	; Release insert?
	JNZ	SHORT X0B		; No
	JMP	RELEASE_INS		; Yes
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The following are toggles. Ignore anything with the high-bit
;	set.
;
X0B:	TEST	AL,80H			; See if high-bit
	JZ	SHORT X0C		; Not set
	JMP	RDONE			; Set, we are done
X0C:	CMP	AL, SC_SCR_LCK		; Scroll-lock?
	JNZ	SHORT X0E		; No
	TEST	BYTE PTR [BX+KEY_SHIFT], COPY_SCRL
	JNZ	SHORT X0D		; No
	JMP	SET_SCR			; Set scroll-lock
X0D:	JMP	SHORT RELEASE_SCR	; Release scroll-lock
X0E:	CMP	AL,SC_NUM_LCK		; Toggle num-lock?
	JNZ	SHORT X10		; No
	TEST	BYTE PTR [BX+KEY_SHIFT], COPY_NUM
	JNZ	SHORT X0F		; No
	JMP	SET_NUM			; Set num-lock
X0F:	JMP	SHORT RELEASE_NUM	; Release num-lock
X10:	CMP	AL,SC_CAP_LCK		; Cap's lock?
	JNZ	SHORT X12		; No
	TEST	BYTE PTR [BX+KEY_SHIFT], COPY_CAP
	JNZ	SHORT X11		; No
	JMP	SET_CAP			; Set caps-lock
X11:	JMP	SHORT RELEASE_CAP	; Release caps-lock
X12:	JMP	RDONE
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

RELEASE_SHF:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (RIGHT_SHIFT OR LEFT_SHIFT)
	JMP	RDONE
RELEASE_ALT:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (ANY_ALT)
	AND	BYTE PTR [BX+KEY_XSHIFT], NOT (LEFT_ALT OR RIGHT_ALT)
	JMP	RDONE
RELEASE_CTL:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (ANY_CTL)
	AND	BYTE PTR [BX+KEY_XSHIFT], NOT (LEFT_CTL OR RIGHT_CTL)
	JMP	RDONE
RELEASE_SCR:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (COPY_SCRL)
	AND	BYTE PTR [BX+KEY_XSHIFT], NOT (SCRL_LOCK)
					; Turn off Scroll Lock LED
	AND	BYTE PTR [BX+KBD_LED], (NOT KBD_SCRL_LED)
	MOV	AL,[BX+KBD_LED]
	CALL	DO_LEDS
	JMP	RDONE
RELEASE_NUM:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (COPY_NUM)
	AND	BYTE PTR [BX+KEY_XSHIFT], NOT (NUM_LOCK)
					; Turn off num-lock LED
	AND	BYTE PTR [BX+KBD_LED], (NOT KBD_NUML_LED)
	MOV	AL,[BX+KBD_LED]
	CALL	DO_LEDS
	JMP	RDONE
RELEASE_CAP:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (COPY_CAP)
	AND	BYTE PTR [BX+KEY_XSHIFT], NOT (CAP_LOCK)
					; Turn off Caps-lock LED
	AND	BYTE PTR [BX+KBD_LED], (NOT KBD_CAPS_LED)
	MOV	AL,[BX+KBD_LED]
	CALL	DO_LEDS
	JMP	RDONE
RELEASE_INS:
	AND	BYTE PTR [BX+KEY_SHIFT], NOT (INS_MODE)
	JMP	RDONE
RELEASE_SYS:
	AND	BYTE PTR [BX+KEY_XSHIFT], NOT (SYS_REQ)
	JMP	RDONE
SET_RIGHT_SHF:
	OR	BYTE PTR [BX+KEY_SHIFT], RIGHT_SHIFT
	JMP	SHORT RDONE
SET_LEFT_SHF:
	OR	BYTE PTR [BX+KEY_SHIFT], LEFT_SHIFT
	JMP	SHORT RDONE
SET_RIGHT_ALT:
	OR	BYTE PTR [BX+KEY_XSHIFT], RIGHT_ALT
	OR	BYTE PTR [BX+KEY_SHIFT], ANY_ALT
	JMP	SHORT RDONE
SET_LEFT_ALT:
	OR	BYTE PTR [BX+KEY_XSHIFT], LEFT_ALT
	OR	BYTE PTR [BX+KEY_SHIFT], ANY_ALT
	JMP	SHORT RDONE
SET_RIGHT_CTL:
	OR	BYTE PTR [BX+KEY_SHIFT], ANY_CTL
	OR	BYTE PTR [BX+KEY_XSHIFT], RIGHT_CTL
	JMP	SHORT RDONE
SET_LEFT_CTL:
	OR	BYTE PTR [BX+KEY_SHIFT], ANY_CTL
	OR	BYTE PTR [BX+KEY_XSHIFT], LEFT_CTL
	JMP	SHORT RDONE
SET_SCR:
	OR	BYTE PTR [BX+KEY_SHIFT], COPY_SCRL
	OR	BYTE PTR [BX+KEY_XSHIFT], SCRL_LOCK
					; Turn on Scroll Lock LED
	OR	BYTE PTR [BX+KBD_LED], KBD_SCRL_LED
	MOV	AL,[BX+KBD_LED]
	CALL	DO_LEDS
	JMP	SHORT RDONE
SET_NUM:
	OR	BYTE PTR [BX+KEY_SHIFT], COPY_NUM
	OR	BYTE PTR [BX+KEY_XSHIFT], NUM_LOCK
					; Turn on num-lock LED
	OR	BYTE PTR [BX+KBD_LED], KBD_NUML_LED
	MOV	AL,[BX+KBD_LED]
	CALL	DO_LEDS
	JMP	SHORT RDONE
SET_CAP:
	OR	BYTE PTR [BX+KEY_SHIFT], COPY_CAP
	OR	BYTE PTR [BX+KEY_XSHIFT], CAP_LOCK
					; Turn on Caps-lock LED
	OR	BYTE PTR [BX+KBD_LED], KBD_CAPS_LED
	MOV	AL,[BX+KBD_LED]
	CALL	DO_LEDS
	JMP	SHORT RDONE
SET_INS:
	OR	BYTE PTR [BX+KEY_SHIFT], INS_MODE
	JMP	SHORT RDONE
SET_SYS:
	OR	BYTE PTR [BX+KEY_XSHIFT], SYS_REQ
RDONE:	POP	AX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sends bits to the keyboard controller to operate the LEDS.
;	The LED bits are in AL upon entry.
;
ALIGN	4
DO_LEDS:
	PUSH	AX			; Save LED bits
	MOV	AL,LED_CMD		; Get LED command
	CALL	SEND_8042		; Send to controller
	POP	AX			; Restore LED bits
	CALL	SEND_8042		; Send LED bits to controller
	MOV	AL, KBD_ENABLE		; Get keyboard enable
	JMP	SHORT SEND_8042		; Send to controller (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the alternate keyboard interrupt. We read the base port
;	and the status port. We don't actually care what the status is
;	because if the keyboard is disconnected we want to continue anyway.
;
ALIGN	4
WAIT_STA:
	PUSH	AX
	PUSH	BX
	PUSH	DS
	XOR	BX,BX
	MOV	AX,BIOS_SEG			; Get BIOS segment
	MOV	DS,AX				; DS = BIOS
	IN	AL,PORT_A			; Get controller response
	MOV	[BX+MFG_TEST_PORT],AL		; Save response
	IN	AL,STATUS_PORT			; Get controller status
	MOV	[BX+KBD_STAT],AL		; Save status
	MOV	AL, NOS_EOI			; Non-specific end-of-inter
	OUT	ACK_CTL0,AL			; ACK first controller
	POP	DS
	POP	BX
	POP	AX
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is somewhat evil. I need to enable the interrupts to
;	read the keyboard status. I really don't care what it is,
;	but I need to get the status and any received data via an
;	interrupt or else I will screw up the interrupt sequencing
;	like the IBM/AT BIOS does.
;
;	So, I patch in a different ISR so the current one isn't
;	reentered. Then I enable interrupts.
;
ALIGN	4
SEND_8042:
	PUSHF
	CLI				; Make sure they are OFF
	PUSH	BX
	PUSH	CX
	PUSH	DI
	PUSH	ES			; Save segment
	MOV	CX,AX			; Save byte to transmit
	XOR	BX,BX			; Get a zero
	MOV	ES,BX			; Set segment
	MOV	DI,(IRQ1 SHL 2)		; Vector to patch
	PUSH	WORD PTR ES:[DI]	; Save Offset
	MOV	AX,OFFSET WAIT_STA	; Status interrupt
	STOSW				; Set offset
	PUSH	WORD PTR ES:[DI]	; Save segment
	MOV	AX,CS			; Get segment
	STOSW				; Set segment
	STI				; Allow interrupts
	SET_TIMER 02H			; Set timeout
WAIT_8042:
	IN	AL,STATUS_PORT		; Get status
	TEST	AL, INP_FULL		; See if it's full
	JZ	SHORT OKAY_8042		; Not full
	WAIT_TIMER WAIT_8042		; Wait until it changes
OKAY_8042:
	MOV	BYTE PTR [BX+MFG_TEST_PORT],0AAH
	MOV	BYTE PTR [BX+KBD_STAT],055H
	MOV	AL,CL			; Get saved byte
	OUT	PORT_A,AL		; Send to controller
	SET_TIMER 02H			; Set up timer
RECV_8042:
	CMP	BYTE PTR [BX+MFG_TEST_PORT],0AAH
	JNZ	SHORT RCV_DONE		; Interrupt occurred
	CMP	BYTE PTR [BX+KBD_STAT],055H
	JNZ	SHORT RCV_DONE		; Interrupt occurred
	WAIT_TIMER RECV_8042		; Wait for timeout or interrupt
RCV_DONE:
	MOV	DI,(IRQ1 SHL 2)		; Vector to patch
	POP	CX			; Restore saved segment
	POP	AX			; Restore saved offset
	CLI				; No interrupts
	STOSW				; Patch previous offset
	MOV	AX,CX			; Get previous segment
	STOSW				; Patch previous segment
	POP	ES
	POP	DI
	POP	CX
	POP	BX
	POPF
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the main keyboard Interrupt service routine.
;
ALIGN	4
KBD_ISR:
	PUSH	AX				; Save what we distroy
	PUSH	BX
	PUSH	DS
	MOV	BX,BIOS_SEG			; Where our data exists
	MOV	DS,BX				; Set segment
	MOV	AL,NOS_EOI			; Non-specific end-of-inter
	OUT	ACK_CTL0,AL			; Ack it now, right away.
	IN	AL,STATUS_PORT			; Read status and throw away
	IN	AL,PORT_A			; Get data from controller
	MOV	AH,AL				; Scan-code into AH also
	CALL	DO_KEYS				; Check upon special keys
	TEST	AL,80H				; See if release-code
	JNZ	SHORT PASS			; Yes, no translation
	CALL	GET_XLATE_TABLE			; Get correct conversion
	PUSH	CS				; Get current code segment
	POP	DS				; Into data segment
	XLAT					; Translate scan-codes
	TEST	AL,80H				; See if we put into buffer
	JNZ	SHORT PASS			; Nope
	MOV	BX,BIOS_SEG			; Get BIOS segment again
	MOV	DS,BX				; DS = BIOS memory
	CALL	PUT_KEY				; Put code into buffer
PASS:	POP	DS
	POP	BX
	POP	AX
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Dummy user tick vector. Everybody needs some place that they can
;	call home.
;
ALIGN	4
INT_1CH:
	PUSH	AX
	PUSH	DX
	PUSH	DS
	MOV	AX,CS			; Get current segment
	MOV	DS,AX			; DS = CS
	MOV	DX,[COM1]		; Get COM1 port value
	ADD	DX,IER			; Interrupt enable
	IN	AL,DX			; Get interrupt enables
	OR	AL,AL			; See if okay
	JNZ	INTOK			; Still enabled
;
	PUSH	BX			; Save the other registers
	PUSH	CX
	PUSH	SI
	PUSH	DI
	PUSH	ES
	CALL	SET_RS232C_INT		; Restore the interrupt
	POP	ES
	POP	DI
	POP	SI
	POP	CX
	POP	BX
;
INTOK:	POP	DS
	POP	DX
	POP	AX
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the Equipment list vector. It returns the contents of
;	40:10  in AX.
ALIGN	4
INT_11H:
	STI					; User function, enable
	PUSH	BX				; Save registers
	PUSH	DS				; Save segment
	MOV	AX,BIOS_SEG			; Get BIOS segment
	MOV	DS,AX				; DS = BIOS data
	MOV	BX,DEVICE_LIST			; Equipment list
	MOV	AX,[BX]				; Get data
	POP	DS				; Restore segment
	POP	BX				; Restore register
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the memory size vector. It returns the contents of
;	40:13 in AX.
ALIGN	4
INT_12H:
	STI					; User function, enable
	PUSH	BX				; Save registers
	PUSH	DS				; Save segment
	MOV	AX,BIOS_SEG			; Get BIOS segment
	MOV	DS,AX				; DS = BIOS data
	MOV	BX,MEM_KBYTES			; Memory kilobytes
	MOV	AX,[BX]				; Get data
	POP	DS				; Restore segment
	POP	BX				; Restore register
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the keyboard input vector. Is not very complete.
;	I map the 'extended' key code input to the regular input
;	by ANDing off the '1' bit.
;
;	This is a user function. The function code is contained
;	in register AH.
;
;	Function code 0x00   (or 0x10)
;	Read a character. If none are ready, wait forever for one.
;
;	Function code 0x01   (or 0x11)
;	Read a character. Return immediately with ZF=1 if none exists.
;
;	Runction code 0x02   (or 0x12)
;	Read keyboard status.
;
ALIGN	4
INT_16H:
	STI				; User function, enable
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	SI
	PUSH	DI
	PUSH	DS
	MOV	DX,BIOS_SEG		; Get BIOS data segment
	MOV	DS,DX			; Into DS
	AND	AH,00000011B		; Combine 'extended' codes
	OR	AH,AH			; Function 0?
	JNZ	SHORT NF0		; No
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function 0
;	This is supposed to call the 'multitasking hook' while
;	busy-waiting. I may ignore it. This BIOS is for booting.
;
KEYW:	CALL	GET_KEY			; Get character from keyboard
	OR	AX,AX			; See if one exists
	JNZ	SHORT KBDEXT		; Yes
	JMP	SHORT KEYW		; Wait for key
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
NF0:	CMP	AH,1			; Function 1?
	JNZ	SHORT NF1		; No
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function 1
;
	CALL	KEY_STATUS		; Get keyboard buffer status
	JMP	SHORT KBDEXT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
NF1:	CMP	AH,2			; Function 2?
	JNZ	SHORT NF2		; No
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function 2
;
	MOV	BX,KEY_SHIFT		; Shift status and extended
	MOV	AX,[BX]			; Return both status values
	JMP	SHORT KBDEXT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function unknown.
;
NF2:	STC				; Show we can't do it.

KBDEXT:	POP	DS
	POP	DI
	POP	SI
	POP	DX
	POP	CX
	POP	BX
	RETF 2

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the boot vector. It should not return, but maybe it
;	has problems and must, Therefore we save segment registers
;	just in case. Also, the floppy code always retries reads so
;	we don't bother to retry upon an error.
;
ALIGN	4
INT_19H:
	STI
	PUSH	DS
	PUSH	ES
	XOR	DX,DX			; Get a zero
	MOV	DS,DX			; Set segments
	MOV	ES,DX			; DS = ES = 0
	MOV	CX,256			; The IBM/AT bios clears 256 WORDS
	MOV	DI,7C00H		; Boot location
	XOR	AX,AX			; Get a zero
	REP	STOSW			; Clear that area.
	XOR	DX,DX			; Reset any floppy disk
	XOR	AX,AX			; Reset Disk subsystem
	INT	DISKS			; Ignore any error
	MOV	AX,0201H		; Read one sector
	MOV	BX,7C00H		; DS:BX points to buffer
	MOV	CX,1			; First sector
	XOR	DX,DX			; First floppy disk
	INT	DISKS			; Disk vector
	JC	SHORT FAIL		; Can't read it
IPL:	CMP	WORD PTR [BX+1FEH],0AA55H
	JNZ	SHORT FAIL		; IPL bad
	DB	0EAH			; Jmp FAR
	DW	7C00H			; Offset
	DW	0			; Segment
FAIL:	POP	ES			; Restore segments
	POP	DS
	IRET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Read System timer counter (Int 1AH BIOS function 0)
;	Upon exit, the following registers are set:
;	AX = Day overflow flag, which is reset.
;	CX = High word of tick count.
;	DX = Low word of tick count.
;
ALIGN	4
RED_SYSTIME:
	CLI				; No interrupts
	MOV	BX,TIMER_LOW		; Point to timer tick, low count
	MOV	DX,[BX]			; Get count
	MOV	BX,TIMER_HIG		; Point to timer tick, high count
	MOV	CX,[BX]			; Get count
	MOV	BX,TIMER_DAY		; Point to overflow flag
	MOV	AX,[BX]			; Get timer overflow
	MOV	WORD PTR [BX],0		; Zero that flag
	STI				; Enable interrupts
	OR	AX,AX			; Clear carry
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set System timer counter (Int 1AH BIOS function 1)
;	The system clock is set as:
;	Day overflow flag is reset.
;	CX = High word of tick count.
;	DX = Low word of tick count.
;
ALIGN	4
SET_SYSTIME:
	CLI				; No interrupts
	MOV	BX,TIMER_LOW		; Point to timer tick, low count
	MOV	[BX],DX			; Set count
	MOV	BX,TIMER_HIG		; Point to timer tick, high count
	MOV	[BX],CX			; Set count
	MOV	BX,TIMER_DAY		; Point to overflow flag
	MOV	WORD PTR [BX],0		; Zero that flag
	STI				; Enable interrupts
	XOR	AX,AX			; Clear carry
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Read realtime clock time. (Int 1AH BIOS function 2). I do not
;	turn off interrupts nor turn off the clock. I just read it until
;	I get two successive readings in which the seconds value has not
;	changed.
;
;	Return values are:
;	AH = 0
;	AL = Hours in BCD
;	CL = Minutes in BCD
;	CH = Hours in BCD
;	DH = Seconds in BCD
;	DL = 1 if DST, 0 if LST.
;
ALIGN	4
RED_RTCT:
	XOR	AL,AL			; Offset zero
	OUT	CMOS_INDEX,AL		; Set offset 0
	SLOW_IO				; Slow down I/O
	IN	AL,CMOS_DATA		; Get seconds
RTCT0:	SLOW_IO				; Slow down I/O
	MOV	DH,AL			; Save seconds
	MOV	AL,2			; Offset for minutes
	OUT	CMOS_INDEX,AL		; Set offset 2
	SLOW_IO				; Slow down I/O
	IN	AL,CMOS_DATA		; Get minutes
	SLOW_IO				; Slow down I/O
	MOV	CL,AL			; Save minutes
	MOV	AL,4			; Offset for hours
	OUT	CMOS_INDEX,AL		; Set offset 4
	SLOW_IO				; Slow down I/O
	IN	AL,CMOS_DATA		; Get hours
	SLOW_IO				; Slow down I/O
	MOV	CH,AL			; Save hours
	MOV	AL,0BH			; Control register 'B'
	OUT	CMOS_INDEX,AL		; Set offset
	SLOW_IO				; Slow down I/O
	IN	AL,CMOS_DATA		; Get control register
	SLOW_IO				; Slow down I/O
	AND	AL,00000001B		; Last bit is DST option
	MOV	DL,AL			; Save DST bit
	XOR	AL,AL			; Set offset zero
	OUT	CMOS_INDEX,AL		; Set offset 0
	SLOW_IO				; Slow down I/O
	IN	AL,CMOS_DATA		; Get seconds
	SLOW_IO				; Slow down I/O
	CMP	DH,AL			; Check for a change
	JNZ	SHORT RTCT0		; Changed, try again
	MOV	AL,CH			; Must return hours?? Wierd.
	XOR	AH,AH			; Clear
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set realtime clock time. (Int 1AH BIOS function 3)
;
;	Input values are:
;	CL = Minutes in BCD
;	CH = Hours in BCD
;	DH = Seconds in BCD
;	DL = 1 if DST, 0 if LST.
;
;	Output values are:
;	AH = 0
;	AL = Value written to CMOS 0BH register
ALIGN	4
SET_RTCT:
	MOV	AL,0BH				; Control register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL, CMOS_DATA			; Get current
	SLOW_IO					; Slow down I/O
	AND	AL, 00000011B			; Save DST/24hr mode
	OR	AL, 10000000B			; Turn OFF clock
;		    ||||||||___ DSE<0>     Daylight savings time = 1
;		    |||||||____ 12/24<1>   24-hour mode = 1
;		    ||||||_____ ZR<2:3>    Always zero
;		    ||||_______ UIE<4>     Update ended interrupt enable
;		    |||________ AIE<5>     Alarm interrupt enable
;		    ||_________ PIE<6>     Periodic interrupt enable
;		    |__________ SET<7>     Turn on updates
;
	OUT	CMOS_DATA,AL			; Set timer OFF
	SLOW_IO					; Slow down I/O
	XOR	AL,AL				; Offset zero
	OUT	CMOS_INDEX,AL			; Set seconds offset
	SLOW_IO					; Slow down I/O
	MOV	AL,DH				; Get seconds
	OUT	CMOS_DATA,AL			; Set seconds
	SLOW_IO					; Slow down I/O
	MOV	AL,2				; Minutes offset
	OUT	CMOS_INDEX,AL			; Set minutes offset
	SLOW_IO					; Slow down I/O
	MOV	AL,CL				; Get minutes
	OUT	CMOS_DATA,AL			; Set minutes
	SLOW_IO					; Slow down I/O
	MOV	AL,4				; Offset to hours
	OUT	CMOS_INDEX,AL			; Set hours offset
	SLOW_IO					; Slow down I/O
	MOV	AL,CH				; Get hours
	OUT	CMOS_DATA,AL			; Set hours
	SLOW_IO					; Slow down I/O
	MOV	AL,0BH				; Control register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL, CMOS_DATA			; Get current
	SLOW_IO					; Slow down I/O
	AND	AL, 00000010B			; Save 24hr mode
	OR	AL,DL				; DST bit
	OUT	CMOS_DATA,AL			; Set timer ON
	SLOW_IO					; Slow down I/O
	MOV	AH,AL				; Save this value
	XOR	AL,AL				; Offset zero
	OUT	CMOS_INDEX,AL			; Leave index at zero
	SLOW_IO
	XCHG	AH,AL				; AL = value in register 'B'
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Read realtime clock date. (Int 1AH BIOS function 4)
;
;	Upon exit, the following are set:
;	AH = 0;
;	CH = Century in BCD (19 or 20)
;	CL = Year in BCD
;	DH = Month in BCD
;	DL = Day in BCD
;
ALIGN	4
RED_RTCD:
	XOR	AL,AL				; Offset zero
	OUT	CMOS_INDEX,AL			; Set to seconds
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Get seconds
	SLOW_IO					; Slow down I/O
RTCD0:	MOV	AH,AL				; Save current seconds
	MOV	AL,07H				; Offset of day of month
	OUT	CMOS_INDEX,AL			; Set day offset
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Get day
	SLOW_IO					; Slow down I/O
	MOV	DL,AL				; Save day
	MOV	AL,8H				; Get month offset
	OUT	CMOS_INDEX,AL			; Set month offset
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Get month
	SLOW_IO					; Slow down I/O
	MOV	DH,AL				; Save month
	MOV	AL,9H				; Get year offset
	OUT	CMOS_INDEX,AL			; Set year offset
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Get year
	SLOW_IO					; Slow down I/O
	MOV	CL,AL				; Save year
	MOV	AL,32H				; Century offset
	OUT	CMOS_INDEX,AL			; Set century offset
	SLOW_IO					; Slow down I/O
	IN	AL,CMOS_DATA			; Get century
	SLOW_IO					; Slow down I/O
	MOV	CH,AL				; Save century
	XOR	AL,AL				; Offset zero
	OUT	CMOS_INDEX,AL			; Leave at offset zero
	SLOW_IO
	IN	AL,CMOS_DATA			; Get century
	SLOW_IO					; Slow down I/O
	CMP	AH,AL				; See if seconds changed
	JNZ	SHORT RTCD0			; Yes, try again
	XOR	AX,AX				; No, clear
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set realtime clock date. (Int 1AH BIOS function 5)
;	Upon entry, the following parameters are passed:
;	CH = Century in BCD
;	CL = Year in BCD
;	DH = Month in BCD
;	DL = Day in BCD
;
;	Upon exit:
;	AH = 0
;	AL = value written to CMOS register 'B'
;
ALIGN	4
SET_RTCD:
	MOV	AL,0BH				; Control register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL, CMOS_DATA			; Get current
	SLOW_IO					; Slow down I/O
	AND	AL, 00000011B			; Save DST/24hr mode
	OR	AL, 10000000B			; Turn OFF clock
;		    ||||||||___ DSE<0>     Daylight savings time = 1
;		    |||||||____ 12/24<1>   24-hour mode = 1
;		    ||||||_____ ZR<2:3>    Always zero
;		    ||||_______ UIE<4>     Update ended interrupt enable
;		    |||________ AIE<5>     Alarm interrupt enable
;		    ||_________ PIE<6>     Periodic interrupt enable
;		    |__________ SET<7>     Turn on updates
;
	OUT	CMOS_DATA,AL			; Set timer OFF
	SLOW_IO					; Slow down I/O
	MOV	AL,07H				; Offset of day of month
	OUT	CMOS_INDEX,AL			; Set day offset
	SLOW_IO					; Slow down I/O
	MOV	AL,DL				; Get day
	OUT	CMOS_DATA,AL			; Set day
	SLOW_IO					; Slow down I/O
	MOV	AL,08H				; Get month offset
	OUT	CMOS_INDEX,AL			; Set month offset
	SLOW_IO					; Slow down I/O
	MOV	AL,DH				; Get month
	OUT	CMOS_DATA,AL			; Set month
	SLOW_IO					; Slow down I/O
	MOV	AL,09H				; Get year offset
	OUT	CMOS_INDEX,AL			; Set year offset
	SLOW_IO					; Slow down I/O
	MOV	AL,CL				; Get year
	OUT	CMOS_DATA,AL			; Set year
	SLOW_IO					; Slow down I/O
	MOV	AL,32H				; Get sentury offset
	OUT	CMOS_INDEX,AL			; Set century offset
	SLOW_IO					; Slow down I/O
	MOV	AL,CH				; Get century byte
	OUT	CMOS_DATA,AL			; Set century
	SLOW_IO					; Slow down I/O
	MOV	AL,0BH				; Control register 'B'
	OUT	CMOS_INDEX,AL			; Set index
	SLOW_IO					; Slow down I/O
	IN	AL, CMOS_DATA			; Get current
	SLOW_IO					; Slow down I/O
	AND	AL, 00000011B			; Save DST/24hr mode
	OUT	CMOS_DATA,AL			; Set timer ON
	SLOW_IO					; Slow down I/O
	MOV	AH,AL				; Copy CMOS register 'B'
	XOR	AL,AL				; Zero
	OUT	CMOS_INDEX,AL			; Leave at index 0
	SLOW_IO					; Slow down I/O
	XCHG	AL,AH				; AL = 'B' contents
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Set realtime clock alarm. (Int 1AH BIOS function 6)
;
ALIGN	4
SET_RTCA:
	XOR	AX,AX				; Clear return value
	STC					; Pretend its busy.
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Reset realtime clock alarm. (Int 1AH BIOS function 7)
;
ALIGN	4
RSET_RTCA:
	XOR	AX,AX				; Clear return value
	RET					; Pretend we did it.
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Catch-all for bad function codes.
;
ALIGN	4
BAD_FUNC:
	STC
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Function code for clock service command-branch table.
;
ALIGN	4
INT_1AFTAB:
	DW	RED_SYSTIME		; Read system time (0)
	DW	SET_SYSTIME		; Set system time (1)
	DW	RED_RTCT		; Read realtime clock time (2)
	DW	SET_RTCT		; Set realtime clock time (3)
	DW	RED_RTCD		; Read realtime clock date (4)
	DW	SET_RTCD		; Set realtime clock date (5)
	DW	SET_RTCA		; Set realtime clock alarm (6)
	DW	RSET_RTCA		; Reset realtime clock alarm (7)
INVAL:	DW	BAD_FUNC		; Bad function code
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This provides the BIOS time-of-day services. Note. I have discovered
;	that M$ spends all of its spare time accessing this routine with
;	function code 0 (Get BIOS timer-tick). So, we want to keep this
;	short and sweet.
;
ALIGN	4
INT_1AH:
	CMP	AH,0B1H			; PCI BIOS function call?
	JNZ	SHORT NOPCI		; No.
	CALL	PCI_ENTRY		; Yes, call the function
	RETF	2			; Flags on the stack
;
NOPCI:	STI				; User function, enable
	PUSH	BX			; Save work registers
	PUSH	DS
	MOV	BX,BIOS_SEG		; BIOS data segment
	MOV	DS,BX			; DS = BIOS data
	MOV	BL,AH			; Function into low
	XOR	BH,BH			; Clear high
	SHL	BX,1			; Table has words
	ADD	BX,OFFSET INT_1AFTAB	; Function table offset
	CMP	BX,OFFSET INVAL		; Check limits
	JC	SHORT FUNOK		; Valid function code
	MOV	BX,OFFSET INVAL		; Substitute invalid code
FUNOK:	CALL	CS:[BX]			; Go there
	POP	DS			; Restore work registers
	POP	BX
	RETF	2
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the floppy-disk interrupt. It is very simple because
;	we do not muck with the controller within the ISR. Instead
;	we just record the fact that an interrupt happened.
;
ALIGN	4
FDC_INTER:
	PUSH	AX			; Save registers we use
	PUSH	BX
	PUSH	DS
	MOV	AX,BIOS_SEG		; Get BIOS data segment
	MOV	DS,AX			; DS = BIOS
	MOV	BX,DISK_CAL_STA		; Where to write the byte
	OR	BYTE PTR [BX],80H 	; Set interrupt flag
	MOV	AL,NOS_EOI		; Non-specific end-of-interrupt
	OUT	ACK_CTL0,AL		; Tell first controller
	POP	DS
	POP	BX
	POP	AX			; Restore registers
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the disk parameter table for a 1.44 Mb diskette.
;
ALIGN	4
DISK_PARTAB:
SPC0	DB	0DFH			; First byte of  'specify'
SPC1	DB	002H			; Second byte of 'specify'
MOTD	DB	025H			; Motor turn-off delay
BYPS	DB	002H			; 512 bytes/sector
SPTK	DB	012H			; 18 sectors/track
GAP0	DB	01BH			; Gap length between sectors
DATL	DB	0FFH			; Data length
GAP1	DB	054H			; Gap length for format
FILB	DB	0A5H			; Fill byte
HSTL	DB	00FH			; Head settle
MOTS	DB	008H			; Motor start time
	DB	11 DUP(0)		; Next drive?
SEC_LOC	EQU	(OFFSET SPTK - OFFSET DISK_PARTAB)
MOT_LOC	EQU	(OFFSET MOTD - OFFSET DISK_PARTAB)
STR_LOC EQU	(OFFSET MOTS - OFFSET DISK_PARTAB)
BPS_LOC	EQU	(OFFSET BYPS - OFFSET DISK_PARTAB)
GAP_LOC	EQU	(OFFSET GAP0 - OFFSET DISK_PARTAB)
DAT_LOC	EQU	(OFFSET DATL - OFFSET DISK_PARTAB)
GFT_LOC	EQU	(OFFSET GAP1 - OFFSET DISK_PARTAB)
FIL_LOC	EQU	(OFFSET FILB - OFFSET DISK_PARTAB)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This gets parameters, the offset of which is in AL, and returns
;	the parameter in AL. All other registers are preserved.
;
ALIGN	4
GET_PAR:
	PUSH	BX
	PUSH	DS				; Save segment
	XOR	BX,BX				; Zero
	MOV	DS,BX				; DS = BX
	MOV	BX,(FLOPPY_TAB SHL 2)		; DS:BX points to vector
	LDS	BX,[BX]				; DS:BX = parameter table
	XOR	AH,AH				; Clear high byte
	ADD	BX,AX				; Adjust offset
	MOV	AL,[BX]				; Get parameter
	POP	DS
	POP	BX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This waits about 12 microseconds. No registers are destroyed.
;	This can be tested by calling it 1000 * 1000 times. It should
;	take 12 seconds. Adjust the loop count until it is reasonable.
;
ALIGN	4
WAITUS:	PUSHF				; Don't mess with flags
	PUSH	AX			; Save register used
	PUSH	CX			; Save count
	MOV	CX,12			; Loop count
USWT0:	SLOW_IO				; Start with a clean I/O cycle
	IN	AL,HW_TIMER		; Get hardware timer count
	MOV	AH,AL			; Save count
USWT1:	IN	AL,HW_TIMER		; Get timer count again
	CMP	AL,AH			; See if it changed
	JZ	SHORT USWT1		; Not changed yet
	DEC	CX			; Faster than 'loop'
	JNZ	SHORT USWT0		; Continue
	POP	CX			; Restore count
	POP	AX			; Restore register used
	POPF				; Restore flags
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This waits enough time for the printer to operate. All registers
;	are saved.
;
ALIGN	4
WAITPRN:
	PUSH	CX
	MOV	CX,100			; Need 1200 microseconds
WPRN0:	CALL	WAITUS			; Wait 12 microseconds
	LOOP	WPRN0			; Continue
	POP	CX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
SUBTTL	Floppy Disk controller operation
PAGE	,132
;
;	This reads the status from the NEC chip. The NEC chip is slow
;	and requires at least 12 microseconds to respond! Of course
;	floppies are slow anyway so it's hidden in the noise. However,
;	we must wait for various status and data responses or it will
;	hang the chip.
;
ALIGN	4
GET_NEC_STATUS:
	MOV	DI,NEC_STATUS		; Buffer to clear
	MOV	CX,7			; Bytes to clear
	XOR	AL,AL			; Get a zero
	REP	STOSB			; Clear the buffer
	MOV	DX,FDC_STATUS		; NEC Status
	SET_TIMER FDC_TIMEOUT		; Long timeout
RDNEC0:	CALL	WAITUS			; Wait 12 microseconds
	IN	AL,DX			; Get master status
	AND	AL,11010000B		; Request for master and direction
	CMP	AL,11010000B		; Both set?
	JZ	SHORT RDNEC1		; Yes, it's ready
	CMP	AL,10000000B		; Check for no result phase
	JZ	SHORT RDNEC3		; No result phase
	WAIT_TIMER RDNEC0		; Continue
	STC				; Show an error occurred
	RET
;
RDNEC1:	MOV	DI,NEC_STATUS		; Where to put status
	MOV	CX,7			; Maximum status bytes
RDNEC2:	CALL	WAITUS			; Wait 12 microseconds
	INC	DX			; Point to data port
	IN	AL,DX			; INSB doesn't work, data corrupt.
	STOSB				; Put byte in buffer
	DEC	DX			; Back to status port
	CALL	WAITUS			; Wait 12 microseconds
	IN	AL,DX			; Master port
	TEST	AL,01010000B		; See if it's busy
	JZ	SHORT RDNEC3		; Should both go away
	LOOP	RDNEC2			; Try more
RDNEC3:	CLC				; Show okay
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This converts the error status into something the upper levels
;	will understand. If an error occurs, it will be put into register
;	AH and the carry-flag will be set.
;
ALIGN	4
CVT_STATUS:
	JNC	SHORT CHKERR		; There may be an real error
	MOV	DX,FDC_TIMEO		; Timeout error sets CY
	JMP	SHORT STATFL		; Timed out.
CHKERR:	XOR	DX,DX			; No error
	MOV	SI,NEC_STATUS		; Where we put the data
	LODSB				; Get first status byte
	AND	AL,0C0H			; Check normal status
	JZ	SHORT STATOK		; It's okay
	MOV	DX,FDC_CONTF		; Assume controller failed
	CMP	AL,40H			; See if abnormal termination
	JNZ	SHORT STATFL		; Nope, controller failed
	LODSB				; Get next status byte
	MOV	DX,FDC_SECNF		; Assume record not found
	TEST	AL,FDC_STA_RNF		; Check for record not found
	JNZ	SHORT STATFL		; Correct
	MOV	DX,FDC_NOCRC		; Assume CRC error
	TEST	AL,FDC_STA_CRC		; Check for CRC error
	JNZ	SHORT STATFL		; It was a CRC error
	MOV	DX,FDC_DMAOV		; Assume bad DMA
	TEST	AL,FDC_STA_ORE		; Check data overrun
	JNZ	SHORT STATFL		; It was bad DMA
	MOV	DX,FDC_SECNF		; Sector not found
	TEST	AL,FDC_STA_EOC		; Check it
	JNZ	SHORT STATFL		; Correct
	MOV	DX,FDC_WPROT		; Assume write protect
	TEST	AL,FDC_STA_WPT		; Check write-protect
	JNZ	SHORT STATFL		; Correct
	MOV	DX,FDC_ADDRM		; Assume address mark not found
	TEST	AL,FDC_STA_ADR		; Check it
	JNZ	SHORT STATFL		; Correct
	MOV	DX,FDC_CONTF		; Controller must have failed
STATFL:	MOV	RETURN_CODE,DX		; Set error code, zero sectors
	MOV	DS:[DISK_RET_STA],DX	; Save the result
	STC				; Set carry
	RET
STATOK:	AND	RETURN_CODE,00FFH	; Clear error code
	MOV	DS:[DISK_RET_STA],DX	; Save the result
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This waits for a seek operation or a recalibrate to complete.
;	If the NEC chip has data available, it is copied to 'NEC_STATUS'.
;
ALIGN	4
WAIT_SEEK:
	CALL	WAIT_NEC_INTER		; Wait for an interrupt
	JC	SHORT WAITS1		; Did not interrupt
	MOV	AL,FDC_CMD_SENSE	; Sense interrupt status
	CALL	SEND_NEC		; Send to NEC
	JC	SHORT WAITS1		; Timed out
	CALL	GET_NEC_STATUS		; Get status from controller
WAITS1:	JMP	CVT_STATUS		; Convert it (implied return)
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This waits for an interrupt from the NEC controller. When
;	a command is sent to the NEC, the byte at 'DISK_CAL_STA' is
;	set to zero. The ISR is very simple and just sets the high
;	bit of the existing byte.
;
ALIGN	4
WAIT_NEC_INTER:
	SET_TIMER FDC_TIMEOUT		; Timeout
WNECI0:	CALL	WAITUS			; Wait 12 microseconds
	MOV	AL,DS:[DISK_CAL_STA]	; Byte will be written in interrupt
	TEST	AL,10000000B		; Check for status
	JNZ	SHORT WNECI1		; Interrupt complete
	WAIT_TIMER WNECI0		; Wait until done
	STC				; Show it timed out
WNECI1:	RET
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Wait for the NEC to complete. If the NEC chip has data available,
;	it is copied to "NEC_STATUS".
;
ALIGN	4
WAIT_NEC:
	CALL	WAIT_NEC_INTER		; Wait for an interrupt
	JC	SHORT WNEC		; It timed out
	CALL	GET_NEC_STATUS		; Read status from NEC
WNEC:	JMP	CVT_STATUS		; Convert status (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sends a CONFIGURE command to the FDC. It is possible
;	for some clones to not understand this command. Therefore,
;	we don't bother to check any status.
CFG_FDC:
	MOV	AL,FDC_CMD_CONFIG		; Configure command
	CALL	SEND_NEC			; Send to controller
	XOR	AL,AL				; Dummy
	CALL	SEND_NEC			; Send to controller
	MOV	AL,00001111B			; Enable FIFO and threshold
	CALL	SEND_NEC			; Send to controller
	XOR	AL,AL				; Precompensation track
	JMP	SEND_NEC			; Send to NEC, implied return
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This converts the data transfer area in ES:BX to a DMA page
;	number and an offset into that page. It also checks the other
;	input parameters and adjusts them if possible to be valid for
;	the pending read or write operation.
;
ALIGN	4
CHK_PARAMS:
	MOV	AL,SEC_LOC			; Where to get data
	CALL	GET_PAR				; Sectors in AL
	MOV	SECTORS_TRACK,AL		; Save in temporary storage
	CMP	DRIVE_NUMBER,2			; Check for valid drive
	JC	SHORT DRVOK			; It's 0 -> 1
	MOV	RETURN_CODE,FDC_TIMEO		; Timeout error
	POP	BX				; Last return address
	STC					; Set error flag
	RET
;
DRVOK:	MOV	AL,SECTORS_TO_TRANSFER		; Get sectors to read/write
	OR	AL,AL				; Zero sectors?
	JNZ	SHORT SECTOK			; No, fine
	POP	BX				; Last return address
	RET					; Return to previous.
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay number of sectors was greater than zero. Now convert
;	the data-transfer area to a page register number and offset
;	within that page.
;
;	The PC/AT whatever, has DMA page registers that live on 64k
;	boundaries, i.e., segment addresses of 0000H, 1000H, 2000H,
;	etc.
;
;	So, given an input value of 1234:5678 (hex), we do the
;	following:
;	(1)	Shift 1234 right 12 bits = 1 page number (first guess).
;	(2)	Shift 1234 left 4 bits (* 16) = 2340
;	(3)	Add offset to result above 5678 + 1234 = 68AC
;		This is the offset into the page.
;	(4)	If the 'add' above caused a carry, we increment the
;		page number.
;
SECTOK:	MOV	AX,DTA_SEGMENT			; DTA segment
	MOV	DX,AX				; Save
	SHR	AX,0CH				; Isolate page number
	SHL	DX,4				; Times 16
	ADD	DX,DTA_OFFSET			; Sum DTA offset
	ADC	AX,0				; Carry to next page
	MOV	DMA_OFFSET,DX			; Set offset
	MOV	DMA_PAGE,AX			; Set page
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	See if the number of sectors requested will still fit on a track.
;	If they can't, just adjust the number of sectors requested. We
;	could cheat and use the hard-coded parameter table but this
;	would defeat the purpose of having it available by vector.
;
	MOV	AL,SECTORS_TRACK		; Get sectors per track
	SUB	AL,STARTING_SECTOR		; AL = sectors available - 1
	JNC	SHORT STRTOK			; Was okay
BADVAL:	MOV	RETURN_CODE,FDC_INVAL		; Invalid request
	POP	AX				; Bump return address
	STC					; Show error
	RET					; Initial caller
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Okay, we have something that's valid. See if we can fix it up
;	to work.
;
STRTOK:	INC	AL				; AL = sectors available
	CMP	AL,SECTORS_TO_TRANSFER		; See if they fit
	JNC	SHORT SECOK0			; They will fit
	MOV	SECTORS_TO_TRANSFER,AL		; Substitute max available
SECOK0:	MOV	AL,BPS_LOC			; Bytes per sector
	CALL	GET_PAR				; Bytes per sector in AL
	MOV	CL,AL				; Save (shift value)
	XOR	AL,AL				; Clear low byte
	MOV	AH,SECTORS_TO_TRANSFER		; Sectors * 256
	SHR	AX,1				; Sectors * 128
	SHL	AX,CL				; Shift value from table
	DEC	AX				; DTA was zero-based
	MOV	BYTES_TO_TRANSFER,AX		; Save bytes to transfer -1
	ADD	AX,DX				; See if it fits
	JNBE	SHORT SECOK1			; Yes
	MOV	RETURN_CODE,FDC_DMABY		; DMA boundary error
	POP	BX				; Last return address
SECOK1:	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This resets the NEC chip to a condition where it can take
;	commands.
;
ALIGN	4
RESET_NEC:
	MOV	DX,FDC_STATUS			; Status port
	MOV	CX,0FFFFH			; Max spin-wait
NECR0:	CALL	WAITUS				; Wait for 12 microseconds
	IN	AL,DX				; Get status
	TEST	AL,10000000B			; Is it ready?
	LOOPZ	NECR0				; Wait until it's ready
	MOV	CX,0FFFFH			; Max spin-wait
NECR1:	CALL	WAITUS				; Wait for 12 microseconds
	IN	AL,DX				; Get status
	AND	AL,11000000B			; Keep direction and status
	CMP	AL,10000000B			; Status okay, direction 0?
	JZ	SHORT NECR3			; Yes
	INC	DX				; Data port
	IN	AL,DX				; Get data (throw away)
	DEC	DX				; Back to status port
	LOOP	NECR1				; Try again
NECR3:	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sends a command to the NEC controller. The command is
;	in register AL. We ignore any errors here. If the command
;	is not accepted, the errors will be found out later.
;
ALIGN	4
SEND_NEC:
	AND	BYTE PTR DS:[DISK_CAL_STA],7FH	; Clear high bite
	SET_TIMER NEC_TIMEOUT			; Short time-out
	MOV	DX,FDC_STATUS			; Status port
	MOV	AH,AL				; Copy the command
NEC0:	CALL	WAITUS				; Wait for 12 microseconds
	IN	AL,DX				; Get status
	TEST	AL,10000000B			; Is it ready?
	JNZ	SHORT NEC1			; Yes
	WAIT_TIMER NEC0				; Continue to wait
	STC					; Failed
	RET					; Quit
;
NEC1:	SET_TIMER NEC_TIMEOUT			; Short time-out
NEC2:	CALL	WAITUS				; Wait for 12 microseconds
	IN	AL,DX				; Get status
	AND	AL,11000000B			; Keep direction and status
	CMP	AL,10000000B			; Status okay, direction 0?
	JZ	SHORT NEC3			; Yes
	WAIT_TIMER NEC2				; Wait some more
	STC					; Failed
	RET					; Quit
;
NEC3:	INC	DX				; FCD data port
	MOV	AL,AH				; Get byte to output
	OUT	DX,AL				; Send to NEC
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This resets the FDC just like the reset switch. Note: A discovered
;	problem was that a pending DMA operation could corrupt memory,
;	causing a processor reset. This was fixed by aborting any pending
;	DMA operation before a NEC reset!
;
FDC_HARD_RESET:
	XOR	AL,AL				; Zero
	OUT	DMA2_CLEAR,AL			; Abort any pending DMA
	MOV	DX,FDC_MOTOR			; Digital output register
	OUT	DX,AL				; Resets the controller
	CALL	WAITUS				; Wait just a bit
	MOV	AL,(FDC_ENAB OR FDC_MOTON)	; Default FDC enable bits
	OUT	DX,AL				; Restore previous
	CALL	WAITUS				; Wait just a bit
	MOV	WORD PTR DS:[DISK_CAL_STA],0	; Clear status area
	JMP	CFG_FDC				; Configure, implied return
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	Recalibrate the floppy disk drive. Presently we have only one
;	disk type specified, but use the parameter table vector so
;	we can add other types in the future.
;
ALIGN	4
RECALIB:
	CALL	FDC_HARD_RESET			; Reset everything
	CALL	MOTOR_ON			; Turn on the FDC motor
	CALL	RESET_NEC			; Make it ready for commands
	MOV	AL,FDC_CMD_SENSE		; Sense interrupt status
	CALL	SEND_NEC			; Send to NEC
	CALL	GET_NEC_STATUS			; Get status from controller
	MOV	AL,FDC_RATE_PAR			; Get data rate for floppy
	MOV	DX,FDC_RATE			; PLL divisor
	OUT	DX,AL				; Output (write-only)
	SLOW_IO					; Let it take effect
	PUSH	DS				; Save segment
	XOR	BX,BX				; Get a zero
	MOV	DS,BX				; DS = 0
	MOV	BX,(FLOPPY_TAB SHL 2)		; DS:BX points to vector
	LDS	BX,[BX]				; DS:BX = parameter table
	MOV	SI,[BX]				; First and second specify
	POP	DS				; Restore segment
	MOV	AL,FDC_CMD_SPECIFY		; FCD Specify byte
	CALL	SEND_NEC			; Send to the controller
	MOV	AX,SI				; Get Specify word
	CALL	SEND_NEC			; First Specify byte
	MOV	AX,SI				; Get specify word again
	MOV	AL,AH				; Second specify byte
	CALL	SEND_NEC			; No status, no interrupt
	MOV	AL,FDC_CMD_RECAL		; Recalibrate (no status)
	CALL	SEND_NEC			; Send to FCD
	MOV	AL,DRIVE_NUMBER			; Current drive
	CALL	SEND_NEC			; Send drive number to FCD
	JMP	WAIT_SEEK			; Wait for it (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This turns on the floppy drive motor if it is not already on.
;	If it is not on, wait for motor startup time. In every case,
;	update the motor shutdown time.
;
ALIGN	4
MOTOR_ON:
	PUSHF					; Save interrupt status
	CLI					; Don't let anything change
	MOV	AL,MOT_LOC			; Location in parameters
	CALL	GET_PAR				; Get motor time-out
	MOV	CH,AL				; Save motor time-out
	MOV	AL,STR_LOC			; Motor start time
	CALL	GET_PAR				; Get parameter
	MOV	CL,AL				; Save motor start time
	MOV	BX,DISK_MOT_TIM			; Motor timer tick
	MOV	AL,BYTE PTR [BX]		; Get timer value
	MOV	BYTE PTR [BX],CH		; Reset the time
	OR	AL,AL				; See if it timed out
	JNZ	SHORT ISON			; It's already on
	MOV	DX,FDC_MOTOR			; Floppy disk motor port
	IN	AL,DX				; Get current
	SLOW_IO					; Slow down I/O
	OR	AL,(FDC_MOTON OR FDC_ENAB)	; Set motor on bit and enable
	OUT	DX,AL				; Turn ON motor
	MOV	DX,CX				; Save
	XOR	CH,CH				; Clear high byte
	STI					; Allow interrupts
	SET_TIMER CX				; Set motor start timer
WMOT:	WAIT_TIMER WMOT				; Wait for motor start
	MOV	BX,DISK_MOT_TIM			; Motor timer tick
	MOV	BYTE PTR [BX],DH		; Reset the time
ISON:	POPF					; Restore status
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This turns the floppy drive motor off. This is called from
;	an interrupt but AX has already been saved.
;
ALIGN	4
MOTOR_OFF:
	PUSH	DX				; AX was saved in interrupt
	MOV	DX,FDC_MOTOR			; Floppy disk motor port
	IN	AL,DX				; Get current
	SLOW_IO					; Slow down I/O
	AND	AL,(NOT FDC_MOTON)		; Turn motor on bit off
	OUT	DX,AL				; Turn OFF motor
	POP	DX				; Restore register
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the DMA controller for the correct read or write
;	parameters. Upon entry, NEXT_COMMAND contains the read or
;	write mode byte.
;
ALIGN	4
SET_DMA:
	CLI					; No interrupts
	XOR	AL,AL				; Get a zero
	OUT	DMA2_CLEAR,AL			; Clear byte pointer
	SLOW_IO					; Slow down I/O
	CALL	WAITUS				; Wait to set up
	MOV	AL,NEXT_COMMAND			; Mode byte
	OUT	DMA2_MODE,AL			; Output the mode byte
	SLOW_IO					; Slow down I/O
	MOV	AX,DMA_PAGE			; Get transfer page
	OUT	DMA2_PAGE,AL			; Output to page register
	SLOW_IO					; Slow down I/O
	MOV	AX,DMA_OFFSET			; Get offset
	OUT	DMA2_ADDR,AL			; Output low address
	SLOW_IO					; Slow down I/O
	MOV	AL,AH				; Get high address
	OUT	DMA2_ADDR,AL			; Output high address
	SLOW_IO					; Slow down I/O
	MOV	AX,BYTES_TO_TRANSFER		; Get byte count - 1
	OUT	DMA2_COUNT,AL			; Low byte of count
	SLOW_IO					; Slow down I/O
	MOV	AL,AH				; High byte of count
	OUT	DMA2_COUNT,AL			; Set high count byte
	SLOW_IO					; Slow down I/O
	MOV	AL,2				; Select channel 2
	OUT	DMA2_MASK,AL			; Init FCD channel
	SLOW_IO					; Slow down I/O
	STI					; Now allow interrupts
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the combined read and write code for the NEC controller.
;	Upon entry, the following conditions have already been accomplished:
;
;	(1)	Motor is on.
;	(2)	Correct track has been sought.
;	(3)	DMA controller has been programmed and primed.
;	(4)	The NEC command byte is in NEXT_COMMAND.
;
ALIGN	4
READ_WRITE:
	MOV	AL,NEXT_COMMAND			; Command byte
	CALL	SEND_NEC			; Send COMMAND to NEC
	JNC	SHORT RW0			; Sent to NEC okay
	JMP	SHORT RWERR
RW0:	MOV	AX,HEAD_DRIVE			; AH = head, AL = drive
	SHL	AH,2				; Move to bit 2
	AND	AH,00000100B			; Isolate head bit
	OR	AL,AH				; Or in the head
	CALL	SEND_NEC			; Send DRIVE to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,TRACK_NUMBER			; Get cylinder number
	CALL	SEND_NEC			; Send CYLINDER to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,HEAD_NUMBER			; Get head number
	CALL	SEND_NEC			; Send HEAD to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,SECTOR_NUMBER		; Get sector number
	CALL	SEND_NEC			; Send SECTOR to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,BPS_LOC			; Bytes per sector
	CALL	GET_PAR				; Get parameter from table
	CALL	SEND_NEC			; Send BPS code to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,SEC_LOC			; Sectors per track
	CALL	GET_PAR				; Get parameter from table
	CALL	SEND_NEC			; Send SPT to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,GAP_LOC			; Sector gap
	CALL	GET_PAR				; Get parameter from table
	CALL	SEND_NEC			; Send gap to NEC
	JC	SHORT RWERR			; Timed out on send to NEC
;
	MOV	AL,DAT_LOC			; Data length
	CALL	GET_PAR				; Get parameter from table
	CALL	SEND_NEC			; Send length to controller
	JC	SHORT RWERR			; Timed out on send to NEC
;
	CALL	WAIT_NEC			; Wait for results
	JC	SHORT RWERR			; Timed out
	CALL	CVT_STATUS			; Convert result.
	RET
;
RWERR:	XOR	AL,AL				; Mode for 8237
	OUT	DMA2_CLEAR,AL			; Shut down FCD channel
	OR	RETURN_CODE,FDC_TIMEO		; Set timeout error
	STC					; Reset error
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This formats a track. Upon entry, the disk heads have already
;	been set to the specified track.
;
ALIGN	4
FORMAT:	MOV	AL,FDC_CMD_FORMAT
	CALL	SEND_NEC			; Send to controller
	MOV	AX,HEAD_DRIVE			; Get head/drive number
	SHL	AH,2				; Put into proper place
	OR	AL,AH				; Drive/head
	CALL	SEND_NEC			; Send to controller
	MOV	AL,BPS_LOC			; Bytes per sector
	CALL	GET_PAR				; Returns in AL
	CALL	SEND_NEC			; Send to controller
	MOV	AL,SEC_LOC			; Sectors per track
	CALL	GET_PAR				; Get parameter from table
	CALL	SEND_NEC			; Send to controller
	MOV	AL,GFT_LOC			; Gap length for format
	CALL	GET_PAR				; Get parameter
	CALL	SEND_NEC			; Send to controller
	MOV	AL,FIL_LOC			; Filler byte
	CALL	GET_PAR				; Get parameter
	CALL	SEND_NEC			; Send to controller
	CALL	WAIT_NEC			; Wait for results
	CALL	CVT_STATUS			; Convert result.
	XOR	AL,AL				; Mode for 8237
	OUT	DMA2_CLEAR,AL			; Shut down FCD channel
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This seeks to the correct track even if the track has already
;	been selected. Unlike the IBM/PC BIOS I do not presume that
;	it saves any time by remembering if it's already been selected.
;	The seek-time to the same track is overwhelmed by the time
;	necessary to find the next sector if one sector I/O is being
;	done. If not, it's likely that the next read is on a different
;	track altogether.
;
ALIGN	4
SEEK:	CALL	MOTOR_ON			; Extend motor timeout
	CALL	RESET_NEC			; Make it ready for commands
	MOV	AL,FDC_CMD_SEEK			; Seek command to NEC
	CALL	SEND_NEC			; Send to controller
	JC	SHORT SEEK0			; Can't talk to controller
	MOV	AL,DRIVE_NUMBER			; Drive number
	CALL	SEND_NEC			; Send to controller
	JC	SHORT SEEK0			; Can't talk to controller
	MOV	AL,TRACK_NUMBER			; Track number
	CALL	SEND_NEC			; Send to the controller
	JC	SHORT SEEK0			; Can't talk to controller
	CALL	WAIT_SEEK			; Wait for completion
	JNC	SHORT SEEK1			; Seek seemed okay
;
SEEK0:	CALL	RESET_NEC			; Make it ready for commands
	CALL	RECALIB				; Recalibrate the drive
	MOV	AL,FDC_CMD_SEEK			; Seek command to NEC
	CALL	SEND_NEC			; Send to controller
	JC	SHORT SEEK3			; It's broken, give up
	MOV	AL,DRIVE_NUMBER			; Drive number
	CALL	SEND_NEC			; Send to controller
	JC	SHORT SEEK3			; It's broken, give up
	XOR	AL,AL				; Track number 0
	CALL	SEND_NEC			; Send to the controller
	JC	SHORT SEEK3			; It's broken, give up
	CALL	WAIT_SEEK			; Wait for completion
	JC	SHORT SEEK3			; It's broken, give up
	MOV	AL,FDC_CMD_SEEK			; Seek command to NEC
	CALL	SEND_NEC			; Send to controller
	JC	SHORT SEEK3			; It's broken, give up
	MOV	AL,DRIVE_NUMBER			; Drive number
	CALL	SEND_NEC			; Send to controller
	JC	SHORT SEEK3			; It's broken, give up
	MOV	AL,TRACK_NUMBER			; Track number
	CALL	SEND_NEC			; Send to the controller
	JC	SHORT SEEK3			; It's broken, give up
	CALL	WAIT_SEEK			; Wait for completion
	JC	SHORT SEEK3			; It failed
SEEK1:	MOV	BX,TRACK_DRV1			; Assume drive 1
	CMP	DRIVE_NUMBER,1			; See if correct
	SBB	BX,0				; Of course not, it's a trick
	MOV	AL,TRACK_NUMBER			; Track in AL
	MOV	[BX],AL				; Set current track
SEEK2:	RET
SEEK3:	OR	RETURN_CODE,FDC_SEEKF		; Set error code
	STC					; Set error
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is for an invalid function code. Return an error code
;	with the carry flag set.
;
ALIGN	4
DSK_NOTHING:
	MOV	RETURN_CODE,FDC_INVAL		; Invalid function
	STC					; Set CY
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the media type for format. We ignore the input parameter
;	because we support only one diskette at the present.
;
;	This returns a pointer to the parameter table in "ES_SAV:DI_SAV".
;
ALIGN	4
DSK_SETMEDIA:
	CLI
	XOR	AX,AX				; Zero
	PUSH	ES				; Save segment
	MOV	ES,AX				; ES = 0
	MOV	DI,(FLOPPY_TAB SHL 2)		; Parameter table vector
	MOV	AX,OFFSET DISK_PARTAB		; Local procedure
	STOSW					; Patch offset
	MOV	AX,CS				; Current code segment
	STOSW					; Patch segment
	POP	ES				; Restore segment
	STI
	MOV	RET_SEGMENT,CS			; Table segment
	MOV	RET_OFFSET,OFFSET DISK_PARTAB	; Table offset
	MOV	RETURN_CODE,0			; Return status
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This sets the transfer rate for the specified disk.
;
ALIGN	4
DSK_SETDISK:
	MOV	AL,FDC_RATE_PAR			; Get data rate for floppy
	MOV	DX,FDC_RATE			; PLL divisor
	OUT	DX,AL				; Output (write-only)
	XOR	AX,AX				; Clear carry
	MOV	RETURN_CODE,AX			; Return status
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This returns the disk-change status in register AH. Currently
;	it pretends that nothing has changed.
;
ALIGN	4
DSK_CHANGE:
	XOR	AX,AX				; Clear carry
	MOV	RETURN_CODE,AX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This returns the drive type. It should return AH = 2, a 80 track
;	diskette that supports media change.
;
ALIGN	4
DSK_TYPE:
	MOV	RETURN_CODE,0200H
	OR	AX,AX				; Clear carry
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This returns a pointer to a disk parameter table. It is also
;	supposed to return a lot of other stuff which makes parameter
;	passing difficult. This is why we saved/restored registers
;	above the stack in local storage.
;
ALIGN	4
DSK_PARAMS:
	PUSH	DS				; Save segment
	XOR	AX,AX				; Get a zero
	MOV	DS,AX				; Set segment
	MOV	BX,(FLOPPY_TAB SHL 2)		; DS:BX = vector.
	LDS	BX,[BX]				; Get table in DS:BX
	MOV	RET_SEGMENT,DS			; Current segment
	MOV	RET_OFFSET,BX			; Offset of parameter table
	MOV	AL,10H				; CMOS offset
	CALL	GET_CMOS			; Get saved CMOS data
	SHR	AL,4				; Adjust
	MOV	BH,79				; Assume 80 tracks
	MOV	BL,18				; Assume 18 sectors per track
	CMP	AL,4				; See if 1.44 Mb drive
	JZ	SHORT TRKOK			; Yes
	MOV	BL,9				; Assume 9 sectors
	CMP	AL,3				; See if 720 kb drive?
	JZ	SHORT TRKOK			; It's okay
	MOV	BH,39				; Now use 40 tracks
	MOV	BL,15				; Assume 15 sectors
	CMP	AL,2				; 1.2 Mb drive?
	JZ	SHORT TRKOK			; It's okay
	MOV	BL,8				; Assume 8 sectors
	JNZ	SHORT TRKOK			; Yes
	XOR	BX,BX				; No, no such drive
TRKOK:	MOV	[BP - CX_SAV],BX		; Save result
	MOV	BX,0101H			; 1 diskette, max head = 1
	MOV	[BP - DX_SAV],BX		; Save result
	XOR	AH,AH				; Clear high byte
	MOV	[BP - BX_SAV],AX		; Save return value
	XOR	AX,AX				; Zero, clear carry
	MOV	RETURN_CODE,AX			; Show good status
	POP	DS				; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This formats a diskette track.
;	Warning, CHK_PARAMS must be the FIRST called procedure!
;
ALIGN	4
DSK_FORMAT:
	MOV	SECTOR_NUMBER,1			; Start at sector 1
	CALL	CHK_PARAMS			; Check input parameters
	CALL	MOTOR_ON			; Turn on motor
	CALL	SEEK				; Seek to correct track
	JNC	SHORT FORMA			; Seek was okay
	MOV	RETURN_CODE,FDC_SEEKF		; Seek failed
	RET
FORMA:	PUSH	TRACK_SECTOR			; Save cylinder/sector
	MOV	SECTOR_NUMBER,1			; Set sector 1
	MOV	NEXT_COMMAND,FDC_DMA_WRITE	; Write command for DMA
	CALL	SET_DMA				; Set DMA controller
	CALL	FORMAT				; Format the track
	POP	TRACK_SECTOR			; Restore cylinder/sector
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This verifies a Diskette track.
;	Warning, CHK_PARAMS must be the FIRST called procedure!
;
ALIGN	4
DSK_VERIFY:
	CALL	CHK_PARAMS			; Check input parameters
	CALL	MOTOR_ON			; Turn on motor
	CALL	SEEK				; Seek to correct track
	JNC	SHORT VERIY0			; So far, so good
	MOV	RETURN_CODE,FDC_SEEKF		; Seek failed
	RET
;
VERIY0:	MOV	NEXT_COMMAND,FDC_DMA_VERIFY	; Verify command for DMA
	CALL	SET_DMA				; Set DMA controller
	MOV	NEXT_COMMAND,FDC_CMD_READ	; Read command for NEC
	JMP	READ_WRITE			; Go do it (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This writes to a diskette track.
;	Warning, CHK_PARAMS must be the FIRST called procedure!
;
ALIGN	4
DSK_WRITE:
	CALL	CHK_PARAMS			; Check input parameters
	CALL	MOTOR_ON			; Turn on motor
	CALL	SEEK				; Seek to correct track
	JNC	SHORT WRITE0			; Seek was okay
	MOV	RETURN_CODE,FDC_SEEKF		; Seek failed
	RET
;
WRITE0:	MOV	NEXT_COMMAND,FDC_DMA_WRITE	; Write command for DMA
	CALL	SET_DMA				; Set DMA controller
	MOV	NEXT_COMMAND,FDC_CMD_WRITE	; Write command for NEC
	JMP	READ_WRITE			; Go do it (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This reads from a diskette track.
;	Warning, CHK_PARAMS must be the FIRST called procedure!
;
ALIGN	4
DSK_READ:
	CALL	CHK_PARAMS			; Check input parameters
	CALL	MOTOR_ON			; Turn on motor
	CALL	SEEK				; Seek to correct track
	JNC	SHORT READ0			; So far, so good
	MOV	RETURN_CODE,FDC_SEEKF		; Seek failed
	RET
;
READ0:	MOV	NEXT_COMMAND,FDC_DMA_READ	; Read command for DMA
	CALL	SET_DMA				; Set DMA controller
	MOV	NEXT_COMMAND,FDC_CMD_READ	; Read command for NEC
	JMP	READ_WRITE			; Go do it (implied return)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This returns the status of the selected disk. If the disk is
;	not drive 0, return a timeout error. Currently, the status
;	returned has little to do with reality. I don't know why.
;
ALIGN	4
DSK_STATUS:
	CMP	DRIVE_NUMBER,2			; Check 0 -> 1
	JC	SHORT DSKOK0			; Correct drive
	MOV	RETURN_CODE,FDC_TIMEO		; Timeout error
	STC					; Show an error
	RET
;
DSKOK0:	MOV	AX,DS:[DISK_RET_STA]		; Get the last disk status
	OR	RETURN_CODE,AX
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This resets the selected disk. If the disk is not drive 0,
;	return a timeout error. Currently, this function does not
;	fail otherwise, even if there is no diskette in the drive.
;
ALIGN	4
DSK_RESET:
	CMP	DRIVE_NUMBER,2			; Check 0 -> 1
	JC	SHORT DSKOK1			; Correct drive
	MOV	RETURN_CODE,FDC_TIMEO		; Timeout error
	STC					; Show an error
	RET
;
DSKOK1:	CALL	FDC_HARD_RESET			; Reset the controller
	CALL	MOTOR_ON			; Turn on motor
	CALL	RESET_NEC			; Reset controller
	MOV	AL,FDC_RATE_PAR			; Get data rate for floppy
	MOV	DX,FDC_RATE			; PLL divisor
	OUT	DX,AL				; Output (write-only)
	CALL	MOTOR_ON			; Extend motor timeout
	CALL	RECALIB				; Recalibrate the drive
	PUSH	TRACK_SECTOR			; Save current
	MOV	TRACK_SECTOR,4F01H		; Set cylinder to max
	CALL	SEEK				; Seek to track 0
	MOV	TRACK_SECTOR,0001H		; Set cylinder to 0
	CALL	SEEK				; Seek to track 0
	POP	TRACK_SECTOR			; Restore previous
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is a command-branch table for all of the floppy disk functions
;	supported.
;
ALIGN	4
CMDTAB	DW	OFFSET DSK_RESET	; Function code 0
	DW	OFFSET DSK_STATUS	; Function code 1
	DW	OFFSET DSK_READ		; Function code 2
	DW	OFFSET DSK_WRITE	; Function code 3
	DW	OFFSET DSK_VERIFY	; Function code 4
	DW	OFFSET DSK_FORMAT	; Function code 5
	DW	OFFSET DSK_NOTHING	; Function code 6
	DW	OFFSET DSK_NOTHING	; Function code 7
	DW	OFFSET DSK_PARAMS	; Function code 8
	DW	OFFSET DSK_NOTHING	; Function code 9
	DW	OFFSET DSK_NOTHING	; Function code 0A
	DW	OFFSET DSK_NOTHING	; Function code 0B
	DW	OFFSET DSK_NOTHING	; Function code 0C
	DW	OFFSET DSK_NOTHING	; Function code 0D
	DW	OFFSET DSK_NOTHING	; Function code 0E
	DW	OFFSET DSK_NOTHING	; Function code 0F
	DW	OFFSET DSK_NOTHING	; Function code 10
	DW	OFFSET DSK_NOTHING	; Function code 11
	DW	OFFSET DSK_NOTHING	; Function code 12
	DW	OFFSET DSK_NOTHING	; Function code 13
	DW	OFFSET DSK_NOTHING	; Function code 14
	DW	OFFSET DSK_TYPE		; Function code 15
	DW	OFFSET DSK_CHANGE	; Function code 16
	DW	OFFSET DSK_SETDISK	; Function code 17
	DW	OFFSET DSK_SETMEDIA	; Function code 18
DSK_LIMIT	LABEL	WORD
BAD_CMD	EQU	OFFSET CMDTAB + (15 SHL 1)	; Bad command code
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This is the disk I/O vector. It is presently used for a floppy
;	only, but can be readily modified to use a hard disk.
;
ALIGN	4
INT_DISK:
	STI				; User entry, turn ON interrupts
	PUSH	BP			; Save frame-pointer
	MOV	BP,SP			; Adjust frame-pointer
	SUB	SP,D_ALLOC		; Allocate local storage
	MOV	[BP - AX_SAV],AX	; Save AX
	MOV	[BP - BX_SAV],BX	; Save BX
	MOV	[BP - CX_SAV],CX	; Save CX
	MOV	[BP - DX_SAV],DX	; Save DX
	MOV	[BP - SI_SAV],SI	; Save SI
	MOV	[BP - DI_SAV],DI	; Save DI
	MOV	WORD PTR [BP-ES_SAV],ES	; Save ES
	MOV	WORD PTR [BP-DS_SAV],DS	; Save DS
IFDEF	TEST_FLOPPY
	PUSH	AX			; Save function code
	XOR	AX,AX			; Clear flag
	CALL	SHOW			; Show entry parameters
	POP	AX			; Restore function code
ENDIF
	MOV	BX,BIOS_SEG		; Get BIOS segment
	MOV	DS,BX			; DS = BIOS
	MOV	ES,BX			; ES = BIOS
	MOV	BL,AH			; Function code in AH
	XOR	BH,BH			; Clear high byte
	SHL	BX,1			; Table has words
	ADD	BX,OFFSET CMDTAB	; Offset of the table
	CMP	BX,OFFSET DSK_LIMIT	; Should always set CY
	JC	SHORT CMDOK		; Command within limits
	MOV	BX,BAD_CMD		; Substitute bad command
CMDOK:	CALL	CS:[BX]			; Execute the command
IFDEF	TEST_FLOPPY
	PUSHF				; Save possible CY flag
	MOV	AX,0FFFFH		; Flag to show status
	CALL	SHOW			; Show return status
	POPF				; Restore CY flag
ENDIF
	MOV	DS,WORD PTR [BP-DS_SAV]	; Restore DS
	MOV	ES,WORD PTR [BP-ES_SAV]	; Restore ES
	MOV	DI,[BP - DI_SAV]	; Restore DI
	MOV	SI,[BP - SI_SAV]	; Restore SI
	MOV	DX,[BP - DX_SAV]	; Restore DX
	MOV	CX,[BP - CX_SAV]	; Restore CX
	MOV	BX,[BP - BX_SAV]	; Restore BX
	MOV	AX,[BP - AX_SAV]	; Restore AX
	MOV	SP,BP			; Collapse local variables
	POP	BP			; Restore frame-pointer
	RETF	2
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
SUBTTL	Support code for compatible operation
PAGE	,132
;
;	This is a dummy ISR to handle interrupts during startup.
;
ALIGN	4
DUMMY:	PUSH	AX
	PUSH	BX
	PUSH	DS
	MOV	AX,BIOS_SEG			; Get BIOS segment
	MOV	DS,AX				; DS = BIOS segment
	MOV	AL,NOS_EOI			; Non-specific end-of-inter
	OUT	ACK_CTL0,AL			; ACK first controller
	SLOW_IO					; Slow down I/O
	OUT	ACK_CTL1,AL			; ACK second controller
	SLOW_IO					; Slow down I/O
	MOV	BX,LAST_INTER			; Get pointer to junk
	INC	BYTE PTR [BX]			; Bump the byte there
	POP	DS
	POP	BX
	POP	AX
	IRET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	The three-finger salute has been entered. Crash the system.
;
ALIGN	4
CTL_ALT_DEL:
	IN	AL,MSK_CTL0			; Interrupt control mask
	OR	AL,00000010B			; Mask the keyboard
;                  ||||||||____ IRQ0
;                  |||||||_____ IRQ1
;                  ||||||______ IRQ2
;                  |||||_______ IRQ3
;                  ||||________ IRQ4
;                  |||_________ IRQ5
;                  ||__________ IRQ6
;                  |___________ IRQ7
;
	OUT	MSK_CTL0,AL			; Disable keyboard
	STI					; Allow interrupts
	MOV	SI,OFFSET CTLDEL		; Point to message
	CALL	PRINT				; Write to the screen
	SET_TIMER 10H				; Set wait interval
CTLW:	WAIT_TIMER CTLW				; Wait
	JMP	TRAP				; Hit the reset button.
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;
;	This patches our private vector for 32-bit operation.
;
SET_PRIVATE:
	PUSH	ES			; Save segment
	XOR	AX,AX			; Get a zero
	MOV	ES,AX			; Set segment to 0
	MOV	AX,OFFSET EXT_PRIVATE	; Where we put it in PSEG
	SHR	AX,4			; Should be a paragraph
	MOV	BX,CS			; Current code segment
	ADD	BX,AX			; This should be the segment
	MOV	DI,(PRIVATE SHL 2)	; Private vector
	XOR	AX,AX			; Offset 0
	STOSW				; Patch offset
	MOV	AX,BX			; Get segment
	STOSW				; Patch segment
	POP	ES			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
CODE	EQU	8			; Default code segment
DATA	EQU	16			; Default data segment
APPLC	EQU	24			; Application code
APPLD	EQU	32			; Application data
APPLS	EQU	40			; Application stack
NONCS	EQU	48			; 16-bit nonconforming code
BACK	EQU	56			; How to get back
DOS	EQU	64			; DOS segment
SCR	EQU	72			; Screen segment
MBYTE	EQU	80			; One megabyte base
REL_EXE	EQU	5000H			; Segment for real mode
PRO_EXE	EQU	5100H			; Segment for protected
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
SUBTTL	Startup and rundown code for 32-bit Protected Mode
PAGE	,132
;
;	This is the startup and rundown code for 32-bit operation.
;	We need to save and restore the interrupt control masks
;	so that interrupts that may happen while we are in protected
;	mode will be latched, and recognized, as soon as we return.
;
ALIGN	10H
EXT_PRIVATE	LABEL	BYTE
INT_PRIVATE	SEGMENT	USE16 PARA PUBLIC 'ThirtyTwoBitSetup'
	ASSUME	CS:INT_PRIVATE, DS:INT_PRIVATE, ES:INT_PRIVATE, SS:INT_PRIVATE
	ORG	0
DOSTART:
	STI				; User interrupt, allow interrupts
	PUSHA				; Save GP registers
	PUSH	ES			; Save extra segment
	PUSH	DS			; Save data segment
	MOV	BP,AX			; Save function code
	IN	AL,MSK_CTL0		; Get current interrupt mask
	MOV	AH,AL			; Save in AH
	IN	AL,MSK_CTL1		; Get current interrupt mask
	PUSH	AX			; Dummy
	MOV	BX,BIOS_SEG		; BX = BIOS data
	MOV	DS,BX			; Into data segment
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	In all cases, turn the disk drive motor OFF even if it is
;	off. This is to stop it while the interrupts are disabled in
;	32-bit protected mode.
;
	MOV	DX,FDC_MOTOR		; Floppy disk motor port
	IN	AL,DX			; Get current
	SLOW_IO				; Slow down I/O
	AND	AL,(NOT FDC_MOTON)	; Turn motor on bit off
	OUT	DX,AL			; Turn OFF motor
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	MOV	WORD PTR DS:[POST_RES_HIGH],SS	; Save stack segment
	MOV	WORD PTR DS:[POST_RES_LOW],SP	; Save stack pointer
	CALL	START32			; Start 32-bit procedure
	CLI				; Don't want races here
	MOV	BX,BIOS_SEG		; Get BIOS data
	MOV	DS,BX			; DS = BIOS data segment
	CMP	BYTE PTR DS:[DISK_MOT_TIM],0	; Was motor turned OFF?
	JZ	SHORT WASOFF		; Yes
	MOV	DX,FDC_MOTOR		; Floppy disk motor port
	IN	AL,DX			; Get current
	SLOW_IO				; Slow down I/O
	OR	AL,FDC_MOTON		; Turn motor back on
	OUT	DX,AL			; Turn ON motor
	SLOW_IO				; Slow down I/O
WASOFF:	PUSH	BP			; Save return value on old stack
	MOV	SS,WORD PTR DS:[POST_RES_HIGH]	; Get stack segment
	MOV	SP,WORD PTR DS:[POST_RES_LOW]	; Get stack pointer
	POP	AX			; Interrupt control mask
	OUT	MSK_CTL1,AL		; Restore second controller mask
	MOV	AL,AH			; Recover for first controller
	OUT	MSK_CTL0,AL		; Restore first controller mask
	POP	DS			; Restore user's data segment
	POP	ES			; Restore user's extra segment
	POPA				; Restore GP registers
	STI
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Get the results in the DWORD at BPTR.
;
	PUSH	BX			; Save register
	PUSH	DS			; Save segment
	MOV	AX,REL_EXE		; Set segment
	MOV	DS,AX			; Segment of old stack
	MOV	BX,OFFSET LCL_STK	; Where we saved return value
	MOV	AX,[BX-2]		; Get return value
	CMP	AX,0FFFFH		; Did it fail?
	STC				; Assume it did
	JZ	SHORT WASBAD		; Yes
	MOV	AX,PRO_EXE		; Set segment
	MOV	DS,AX			; 32-bit segment
	MOV	BX,(OFFSET SEG32:BPTR - OFFSET SEG32:0)
	MOV	AX,[BX]			; Get low word of number
	MOV	DX,[BX+2]		; Get high word of number
	OR	AX,AX			; Clear carry
WASBAD:	POP	DS			; Restore segment
	POP	BX			; Restore register
	RETF	2			; DWORD in AX:DX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is how the main GDT entries are organized
;
;	DW	1001001000000000B
;               ||||||||||||||||____ More base address 23:16
;               ||||||||____________ type (data R/W)
;               ||||________________ System segment
;               |||_________________ Priv level (0)
;               |___________________ Segment present
;
;
;	DW	0000000011111111B
;               ||||||||||||||||____ More limit 19:16
;               ||||||||||||________ Available for use
;               |||||||||||_________ Segment size (32-Bit)
;               ||||||||||__________ Code size size 0 = 16, 1 = 32-bit
;               |||||||||___________ Granularity (4096)
;               ||||||||____________ More base 24:31
;

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is the real GDT. It is NOT indexed by 8 when a segment
;	(selector) is loaded as stated in the defective documentation.
;	Therefore selectors are numbered 0, 8, 16, etc.
;
ALIGN	08H
GDTPRV	DW	0		; Dummy  Segment = 0;
	DW	0
	DW	0
	DW	0
				; Segment = 8
	DW	0FFFFH		; Limit = 4 Gb
	DW	0000H		; Base address = 0
	DW	9A00H		; Code read/execute
	DW	00FFH		; Granularity = 4096, 386
				; Segment = 16
	DW	0FFFFH		; Limit = 4 Gb
	DW	0000H		; Base address = 15:0
	DW	9200H		; Data read/write
	DW	00FFH		; Granularity=4096
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	The following is supposed to define the code, data and stack
;	segments to be used for the procedure being called. They are
;	32-bit addressing with 32-bit code.
;
				; Segment 24
	DW	0FFFFH		; Gigantic limit
B_ADDC	DW	0000H		; Base address
B_ADHC	DW	9A00H		; Code read/execute (and more base address)
	DW	00FFH		; Granularity=4096, 386
				; Segment = 32
	DW	0FFFFH		; Gigantic limit
B_ADDD	DW	0000H		; Base address
B_ADHD	DW	9200H		; Data read/write (and more base address)
	DW	00FFH		; Granularity=4096, 386
				; Segment = 40
	DW	0FFFFH		; Gigantic limit
B_ADDS	DW	0000H		; Base address
B_ADHS	DW	9200H		; Data read/write (and more base address)
	DW	00FFH		; Granularity=4096, 386
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This selector is used to get back to 16-bit code with 16-bit
;	offsets and 16-bit segments.
;
				; Segment = 48
	DW	0FFFFH		; limit = 64k
R_BASL	DW	0000H		; Base address = Relocated segment
R_BASH	DW	9A00H		; Code read/execute (and more base address)
	DW	0000000000010000B
;               ||||||||||||||||____ More limit 19:16
;               ||||||||||||________ Available for use
;               |||||||||||_________ Segment size 0 = 16-bit
;               ||||||||||__________ Code size 0 = 16-bit
;               |||||||||___________ Granularity Byte granularity
;               ||||||||____________ More base 24:31
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is a 'CALL' Gate. It is used to get back to 16-bit
;	segments so we can return to real mode.
;
				; Segment 56
R_ADDL	DW	0000H		; Offset in segment LOW word
	DW	NONCS		; Segment selector (Non conforming)
	DB	00H		; DWORD count (no parameters passed)
	DB	10001100B	; Show it's a CALL gate, segment present
R_ADDH	DW	0000H		; More offset HIGH word
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This defines the data and stack segments used in 16-bit real mode
;	so we have valid descriptors when returning to real mode.
;
				; Segment = 64 (DOS segment)
	DW	0FFFFH		; limit = 64k
	DW	0000H		; Base address = 0
	DW	9200H		; Readable / writable
	DW	0000000000010000B
;               ||||||||||||||||____ More limit 19:16
;               ||||||||||||________ Available for use
;               |||||||||||_________ Segment size (32-Bit)
;               ||||||||||__________ Code size 0 = 16, 1 = 32-bit
;               |||||||||___________ Granularity bytes
;               ||||||||____________ More base 24:31
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This lets us write to the screen board at 0xB8000 absolute.
;
				; Segment 72
	DW	0004H		; 4 page limit
	DW	8000H		; Base address
	DW	920BH		; Data read/write (and more base address)
	DW	00FFH		; Granularity=4096, 386
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This lets us read/write RAM starting at 1 megabyte.
;
				; Segment 80
	DW	0FFFFH		; Large limit
	DW	0000H		; Base address
	DW	9210H		; Data read/write (and high base address)
	DW	00FFH		; Granularity=4096, 386
LIMPRV	EQU	($ - GDTPRV) - 1
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is supposed to be the GDT that is loaded with the LDGT
;	instruction. The first WORD element is the number of real GDT
;	entries - 1. The second DWORD element is the linear address
;	of the real GDT.
;
ALIGN	08H
GDT_486		DW	LIMPRV		; (GDT entries * 8) -1
GDT_LOW		DW	?		; Low word of GDT
GDT_HIG		DW	?		; High DWORD of GDT
		DD	0
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Here we save the old interrupt descriptor table to be restored
;	when we return to 16-bit real mode. It is not used in 16-bit
;	mode, but it should be reloaded to clear the 32-bit one.
;
ALIGN	08H
OLD_IDT		DW	3FFH		; Old interrupt descriptor limit
		DD	0		; Base
		DD	0		; Dummy
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This interrupt descriptor table is used during 32-bit mode
;	to trap even software interrupts. All interrupts and
;	exceptions trap to this code. The startup code will examine
;	the CMOS shutdown byte and return to the caller with an error
;	if such an exception occurred but could not be handled.
;
NEW_IDT		DW	100H		; New interrupt descriptor limit
NEW_IDT_LOW	DW	0		; New interrupt descriptor base
NEW_IDT_HIG	DW	0
		DD	0		; Dummy
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	First I will relocate the 32-bit code to real memory, aligned
;	on a page boundary. Presently, we were called on the user's
;	stack.
;
START32:
	CLI				; No interrupts from now on.
	CLD				; Forwards
	MOV	AX,CS			; Get where we are
	MOV	DS,AX			; Set data segment
	MOV	AX,PRO_EXE		; Segment we will copy to
	MOV	ES,AX			; Set the segment
	MOV	SI,OFFSET HOME		; DS:SI where we get the code
	XOR	DI,DI			; ES:DI where we put the code
	MOV	CX,(OFFSET SEG32:MSTACK - OFFSET SEG32:0)
	INC	CX			; Need to make even
	SHR	CX,1			; Make words
	REP	MOVSW			; Copy the data
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now I will relocate this code to a new writable segment.
;
	MOV	AX,REL_EXE		; Segment we will copy to
	MOV	ES,AX			; Set the segment
	MOV	SI,OFFSET DOSTART	; DS:SI where we get the code
	MOV	DI,SI			; ES:DI where we put the code
	MOV	CX,(OFFSET SAVE_STUFF - OFFSET DOSTART)
	INC	CX			; Need to make even
	SHR	CX,1			; Make words
	REP	MOVSW			; Copy the data
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now get to the code that was just relocated. We do this by doing
;	a 'FAR RETURN' to that code after preparing the stack.
;
	MOV	AX,REL_EXE		; Relocated segment
	PUSH	AX			; Put that on the stack
	MOV	AX,OFFSET GOTO		; Offset of the relocated code
	PUSH	AX			; Put that on the stack
	RETF				; FAR 'return' to the relocated code
	HLT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This code is now executing out of the relocated location.
;
GOTO:	POP	BX			; Previous near return (offset)
	MOV	AX,CS			; This is where we are
	MOV	DS,AX			; Make sure segments are set.
	MOV	ES,AX
	MOV	SS,AX			; The stack-pointer is still level.
	MOV	SP,OFFSET LCL_STK	; New writable stack
	PUSH	BX			; Return (offset) on the new stack
;
	MOV	AL,SHUTDOWN 		; Shutdown location
	OUT	CMOS_INDEX,AL		; Set index
	SLOW_IO
	MOV	AL,PROT_RET		; Return from protected
	OUT	CMOS_DATA,AL		; Set shutdown reason
	SLOW_IO
	MOV	AL,NMI_OFF		; Set high bit of index.
	OUT	CMOS_INDEX,AL		; Disable NMI
	SLOW_IO
	MOV	AL,0FFH			; Mask off all interrupts
	OUT	MSK_CTL1,AL		; Second controller
	SLOW_IO
	OUT	MSK_CTL0,AL		; First controller
	SLOW_IO
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Get the linear address of the relocated code and fix the
;	addresses in the discriptors
;
	MAKE_REAL PRO_EXE
	MOV	[B_ADDC],AX		; Base address for data
	MOV	[B_ADDD],AX		; Base address for code
	MOV	[B_ADDS],AX		; Base address for stack
	OR	[B_ADHC],DX		; Fix up high word
	OR	[B_ADHD],DX		; Fix up high word
	OR	[B_ADHS],DX		; Fix up high word
;
	MAKE_REAL CS
	MOV	[R_BASL],AX		; Set low base address
	OR	[R_BASH],DX		; Set high base address
	MOV	[R_ADDL],OFFSET DROP	; Set address, low word
	MOV	WORD PTR R_ADDH,0	; Set address, high word
	MOV	[DROPCS],CS		; Set up jump segment
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Get the linear address of the GDT base and fix the address
;	in the discriptor.
;
	MAKE_REAL CS, <OFFSET GDTPRV>
	MOV	[GDT_LOW],AX
	MOV	[GDT_HIG],DX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Get the linear address of the IDT base and fix the address
;	in the discriptor.
;
	MAKE_REAL PRO_EXE, <(OFFSET SEG32:NEW_IDT_DESC - OFFSET SEG32:0)>
	MOV	[NEW_IDT_LOW],AX
	MOV	[NEW_IDT_HIG],DX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now load the Global discriptor table. We will also save the
;	current state of the IDT so we can restore it when we return
;	to 16-bit real mode.
;
	SIDT	FWORD PTR [OLD_IDT]	; Save old interrupt descriptor
	LIDT	FWORD PTR [NEW_IDT]	; Load new interrupt descriptor
	LGDT	FWORD PTR [GDT_486]	; Load the GDT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now  enable A20. The keyboard chip is actually a microprocessor
;	that is sometimes 'sticky' so we have to make sure the bit does
;	get connected.
;
	SET_A20_ON			; Turn ON A<20> enable
	XOR	CX,CX
	LOOP	$			; Make sure it is on
	MOV	[SP_SAV],SP		; Save current stack pointer
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now go into protected mode. Intel says that 'LMSW' is obsolete so
;	we use the longword move instruction.
;
	MOV	EAX,CR0			; Get machine status word
	OR	EAX,PROTECTED		; Set PE bit
	MOV	CR0,EAX			; This puts us into protected mode
;
	MOV	AX,APPLD		; Application data descriptor
	MOV	DS,AX			; Set segments before transfer
	MOV	ES,AX
	MOV	FS,AX
	MOV	GS,AX
	MOV	AX,APPLS		; Application stack descriptor
	MOV	SS,AX
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This will make a FAR jump through the descriptor to the application
;	code. Since we don't use a CALL gate, the offset is ignored.
;	Execution will begin at the first instruction in the relocated
;	32-bit segment.
;
	DB	66H			; Size override prefix
	DB	0EAH			; Jump immediate
	DD	0			; Offset (ignored anyway)
	DW	APPLC			; Application code segment
	HLT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	These are the first instructions when we are called from the
;	application code. The 'call' must be to a 64k segment, located
;	in the correct place, with the correct attributes, shown in
;	p.p. 22-4 for this to work.
;
DROP:	MOV	AX,DOS			; Segment discriptor defined pp 22-4
	MOV	DS,AX			; Set all segment registers.
	MOV	ES,AX
	MOV	FS,AX
	MOV	GS,AX
	MOV	SS,AX
;
	MOV	EAX,CR0			; Get machine status word
	AND	AX,(NOT PROTECTED)	; Reset PE bit
	MOV	CR0,EAX			; This should return us to real mode
	DB	0EAH			; Jump immediate
	DW	OFFSET WHERE		; Offset
DROPCS	DW	0			; Segment
	HLT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	These are the first instructions once (if) we get back to real
;	mode.
;
WHERE:	MOV	AX,CS			; Our 16-bit data segment
	MOV	DS,AX			; Set real-mode segments
	MOV	ES,AX			; Reload ALL of them!
	MOV	SS,AX
	MOV	FS,AX
	MOV	GS,AX
	MOV	SP,[SP_SAV]		; Restore local stack
	LIDT	FWORD PTR [OLD_IDT]	; Load old interrupt descriptor
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Now  disable A20. We could leave it on with no problem, but
;	we want to return the machine to the state at which we started.
;
	SET_A20_OFF			; Turn it OFF
	XOR	CX,CX
	LOOP	$			; Make sure it is off
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Clear the shutdown byte in CMOS.
;
	MOV	AL,SHUTDOWN		; CMOS shutdown location
	OUT	CMOS_INDEX,AL		; Point to location
	SLOW_IO
	XOR	AL,AL			; Get a zero
	OUT	CMOS_DATA,AL		; Clear shutdown byte
	SLOW_IO
	OUT	CMOS_INDEX,AL		; Enable NMI
	SLOW_IO
	STI				; Allow interrupts
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
SP_SAV		DW	0	; Save stack-pointer
		DW	100H DUP(?)
LCL_STK	LABEL	WORD
SAVE_STUFF	LABEL	BYTE
SUBTTL	32-bit Protected mode code
PAGE	,132
;
;	This is a 32-bit segment using 32-bit code and data. Note the
;	'USE32' which tells the assembler to use the 32-bit form.
;
;	The BASE of this procedure and its stack is at 'PRO_EXE',
;	currently absolute address 0x51000.
;
;	However, segment  'MBYTE' starts at the 1 megabyte boundary.
;	The total RAM reported will be the RAM above that boundary.
;
;	Note that there is a MASM bug work-around shown in the code
;	several places as "MASM *BUG* (really MOV DS,AX)". The operand
;	size override, 66H, should never, ever, ever, be used when
;	referencing segment registers.
;
ALIGN	10H
HOME	LABEL	BYTE
SEG32	SEGMENT USE32 PARA PUBLIC 'ThirtyTwoBitLinear'
	ASSUME	DS:SEG32, ES:SEG32, SS:SEG32, CS:SEG32
	ORG	0
	MOV	ESP,(OFFSET MSTACK) + STACK32_LEN
	CALL	CHKCMD			; Check function code
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This gets us back to real mode, actually it starts us on
;	the long trip back.
;
	DB	0EAH			; Jump immediate
	DD	0			; Offset (ignored anyway)
	DW	BACK			; Startup segment
	HLT
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Command-branch table.
;
CBT	DD	RAMSIZ			; Get extended RAM size
	DD	GET_MMCR		; Get a copy of the MMCR
	DD	COPY_BIOS		; Copy a ROM BIOS
	DD	CHEK_BIOS		; Check for a ROM BIOS
INV	LABEL	DWORD
;
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This checks for the 8-bit command code, passed in EBP. This
;	register was used because it's not touched in the 16/32-bit
;	transition code.
;
CHKCMD:	MOV	EBX,EBP			; Function code
	SHR	EBX,8			; Was in AH in 16-bit land
	AND	EBX,0FFH		; Keep 8 bits only
	SHL	EBX,2			; 4 Bytes per DWORD entry
	ADD	EBX,OFFSET CBT		; Command-branch table
	CMP	EBX,OFFSET INV		; Within limits?
	JC	SHORT LMOK		; Limit is okay
	OR	EBP,0FFFFFFFFH		; Error flag
	RET				; Return to caller
LMOK:	JMP	[EBX]			; Implied return from function
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This copies the contents from a ROM BIOS to a memory location
;	visible in real mode.
;
COPY_BIOS:
	PUSH	DS
	PUSH	ES
	MOV	EAX,DATA		; Base starts at zero
	MOV	ES,EAX			; Set segments
	MOV	DS,EAX			; ES = DS = (0 -> 0xffffffff)
	MOV	EBX,(CPY_ROM_SEG SHL 4)
	MOV	ECX, DWORD PTR [EBX+CPY_ROM_LEN]
	MOV	ESI, DWORD PTR [EBX+CPY_ROM_SRC]
	MOV	EDI, DWORD PTR [EBX+CPY_ROM_DST]
	CLD				; Forwards
	MOV	EAX,ECX			; Get copy
	AND	EAX,00000003H		; Keep low two bits
	SHR	ECX,2			; Make DWORDS
	REP	MOVSD			; Copy DWORDS
	MOV	ECX,EAX			; Get remaining bytes
	SHR	ECX,1			; Convert to words
	REP	MOVSW			; Copy any odd WORD
	ADC	ECX,0			; Pick up any odd byte count
	REP	MOVSB			; Copy the odd byte
	POP	ES
	POP	DS
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This checks for the presence of a ROM BIOS at the location,
;	CPY_ROM_DST. If it is not found, CPY_ROM_DST is cleared to
;	zero.
;
CHEK_BIOS:
	PUSH	DS
	MOV	EAX,DATA			; Base starts at zero
	MOV	ES,EAX				; Set segment
	MOV	EBX,(CPY_ROM_SEG SHL 4)		; Where we put the table
	MOV	ESI,DWORD PTR [EBX+CPY_ROM_SRC]	; Get absolute address
	CMP	WORD PTR [ESI], 0AA55H		; Check for signature
	JZ	SHORT SIGOK			; Signature was found
	MOV	DWORD PTR [EBX+CPY_ROM_SRC],0	; Clear that address
SIGOK:	POP	DS
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This gets the size of RAM above 1 megabytes.
;
RAMSIZ:	MOV	EAX,CR0			; Get control register
	OR	EAX,(CR0_CD OR CR0_NW)	; Disable caching
	MOV	CR0,EAX			; Set control register
	WBINVD				; Flush the cache
	MOV	DWORD PTR [BPTR],0	; Clear memory found
	CALL	SON32			; Write signon message
	MOV	EAX,MBYTE		; Segment starts at 1Mb
	MOV	DS,EAX			; MASM *BUG* (really MOV DS,AX)
	MOV	ES,EAX			; MASM *BUG* (really MOV ES,AX)
;
	MOV	EDI,-10000H		; Where to start
	MOV	ECX,055AA55AAH		; First pattern
	MOV	EAX,0AA55AA55H		; Second pattern
	JMP	SHORT PATERN		; Make fast as possible
ALIGN	8
PATERN:	LEA	EDI,[EDI+10000H]	; Ready next location
	MOV	[EDI],ECX		; Put in somthing first
	MOV	[EDI],EAX		; Put in pattern
	XOR	[EDI],EAX		; See if it went, zero RAM
	JZ	SHORT PATERN		; Yes, continue
	OR	EDI,EDI			; Last good DWORD
	JZ	SHORT DONE32		; There is no RAM here
;
;	Save the results. Also save 64k chunks in MEM32_LOC.
;
	MOV	EAX,APPLD		; Data segment
	MOV	DS,EAX			; MASM *BUG* (really MOV DS,AX)
	MOV	[BPTR],EDI		; Save the results so far
	MOV	EAX,DATA		; The whole machine
	MOV	DS,EAX			; MASM *BUG* (really MOV DS,AX)
	MOV	EAX,EDI			; Get results again
	SHR	EAX,10H			; Put high into low (64k blocks)
	MOV	EBX,MEM32_LOC		; One of the printer-port locations
	MOV	WORD PTR [EBX],AX	; Save in low data
;
;	Now zero all of RAM.
;
	XOR	EAX,EAX			; Zero
	MOV	ECX,EDI			; Byte count
	SHR	ECX,2			; DWORD count
	XOR	EDI,EDI			; Zero
	REP	STOSD			; Clear
DONE32:	MOV	EAX,CR0			; Get control register
	AND	EAX,(NOT (CR0_CD OR CR0_NW))	; Enable caching
	MOV	CR0,EAX			; Set control register
	WBINVD				; Flush the cache
	CALL	SOF32			; Clear signon message
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This just writes a message from protected mode. This is just
;	to show we know how to do it. If we run out of PROM space
;	this goes.
;
	MSG	DB	' Testing high memory '
MSGLEN	EQU	$ - MSG
SON32:	PUSH	DS			; Save these segments
	PUSH	ES
	MOV	EAX,APPLD		; This is where our text is
	MOV	DS,EAX			; MASM *BUG* (really MOV DS,AX)
	MOV	EAX,SCR			; This is the screen segment
	MOV	ES,EAX			; MASM *BUG* (really MOV ES,AX)
	MOV	ECX,MSGLEN		; Length of message
	MOV	EDI,PUT_TEXT		; ES:EDI = destination
	MOV	ESI,OFFSET MSG		; DS:ESI = source
	MOV	AH,97H			; Flashing blue attribute
SO32:	LODSB				; Get byte
	STOSW				; Store word
	LOOP	SO32			; Do whole string
	POP	ES			; Restore segments
	POP	DS
	RET
;
SOF32:	PUSH	ES			; Save segment
	MOV	EAX,SCR			; Screen segment
	MOV	ES,EAX			; MASM *BUG* (really MOV ES,AX)
	MOV	ECX,(MSGLEN + 2 SHR 1)	; Previous message length
	MOV	EDI,PUT_TEXT		; ES:DI = destination
	MOV	EAX,07200720H		; Normal attribute and blank
	REP	STOSD			; Write longwords to clear
	POP	ES			; Restore segment
	RET
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is going to copy the real MMCR register table used in the
;	SC520 from its absolute address of 0xFFFEF000. It is 4k (0x1000)
;	in length. I will copy this to the second 'page' of screen-memory
;	which is at 0x000B8800. This lets anybody read it. Since the
;	memory-selector starts at 1 megabyte (0x100000), we will grab
;	the table from offset 0xFFFEF000 - 0x100000 = 0xFFFDF000.
;
GET_MMCR:
	PUSH	DS
	PUSH	ES
	MOV	EAX,SCR			; Screen segment
	MOV	ES,AX			; Set destination segment
	MOV	EDI,1000H		; ES:DI = second page of screen
	MOV	EAX,MBYTE		; Segment starts at 1Mb
	MOV	DS,AX			; Set source segment
	MOV	ESI,0FFFDF000H		; DS:SI = MMCR
	MOV	ECX,(1000H SHR 1)	; 4 kb (in words)
	REP	MOVSW			; Copy the data
	POP	ES
	POP	DS
	RET

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
SUBTTL	32-bit Protected Mode exception handler
PAGE	,132
;
;	This code is to handle traps while in 32-bit protected mode.
;	When working on broken hardware, you get lots of these. This
;	traps exceptions and tries to return the machine to real mode
;	after printing the exception information. This works quite well
;	and saves a lot of time re-booting.
;
ALIGN	08H
TRPMSG	DB	' Exception '
TRPNO	DB	'nn occurred '
TRPBUF	DB	20H	DUP(0)		; Buffer for largest error string
EXCPT0	DB	'Divide Error',0
EXCPT1	DB	'Debug Exception',0
EXCPT2	DB	'NMI (parity error)',0
EXCPT3	DB	'Breakpoint',0
EXCPT4	DB	'Overflow',0
EXCPT5	DB	'Bounds Check',0
EXCPT6	DB	'Invalid Opcode',0
EXCPT7	DB	'Device Not Available',0
EXCPT8	DB	'Double Fault',0
EXCPT9	DB	'Reserved',0
EXCPT10	DB	'Invalid TSS',0
EXCPT11	DB	'Segment Not Present',0
EXCPT12	DB	'Stack Exception',0
EXCPT13	DB	'General Protection',0
EXCPT14	DB	'Page Fault',0
EXCPT15	DB	'Undefined',0
EXCPT16	DB	'Floating Point Exception',0
EXCPT17	DB	'Alignment Check',0
EXCP_TAB	DD	OFFSET EXCPT0
		DD	OFFSET EXCPT1
		DD	OFFSET EXCPT2
		DD	OFFSET EXCPT3
		DD	OFFSET EXCPT4
		DD	OFFSET EXCPT5
		DD	OFFSET EXCPT6
		DD	OFFSET EXCPT7
		DD	OFFSET EXCPT8
		DD	OFFSET EXCPT9
		DD	OFFSET EXCPT10
		DD	OFFSET EXCPT11
		DD	OFFSET EXCPT12
		DD	OFFSET EXCPT13
		DD	OFFSET EXCPT14
		DD	OFFSET EXCPT15
		DD	OFFSET EXCPT16
		DD	OFFSET EXCPT17
	LAST_ENTRY EQU (($ - EXCP_TAB) SHR 2)
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This is a MACRO to generate 32 trap gates, each pointing to the
;	same generic code, but with different offsets into that code.
;
MAKE_DISC	MACRO	N
	DW	(OFFSET SEG32:CRASH&N - OFFSET SEG32:0)
	DW	APPLC			; Application code segment
	DW	1000111100000000B
;		||||||||||||||||________ Reserved        <0:4>
;		|||||||||||_____________ Must be zero    <5:7>
;		||||||||________________ Trap gate       <8:12>
;		|||_____________________ Privilege       <13:14>
;		|_______________________ Segment present <15>
;
	DW	0
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	This macro loads the exception number into register BL before
;	jumping to the display and reset code.
;
MAKE_CODE	MACRO	N
CRASH&N:	MOV	BL,N
		JMP	SHORT RESET
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	These ugly MACROS actually generate the code with replacable
;	parameters in 'X'.
;
ALIGN	08H
NEW_IDT_DESC:
	X = 0
REPT	32
	MAKE_DISC %X
	X = X + 1
ENDM
ALIGN	08H
	X = 0
REPT	32
	MAKE_CODE %X
	X = X + 1
ENDM
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
RESET:	MOV	EAX,APPLD		; This is where our text is
	MOV	DS,EAX			; Set data segment
	MOV	ES,EAX			; Set extra segment
	MOV	EDI,OFFSET TRPNO	; Where to put the text
	MOV	AL,BL			; Get interrupt number
	SHR	AL,4			; Isolate high nibble
	ADD	AL,90H			; Add bias
	DAA				; Old Intel trick
	ADC	AL,40H			; Pick up CY into high nibble
	DAA				; Readjust
	STOSB				; Put into the string
	MOV	AL,BL			; Get saved byte again
	AND	AL,00001111B		; Isolate low nibble
	ADD	AL,90H			; Add bias
	DAA				; Old Intel trick
	ADC	AL,40H			; Pick up CY into high nibble
	DAA				; Readjust
	STOSB				; Put into the string
	XOR	EAX,EAX			; Clear
	OR	AL,BL			; Exception into low byte
	CMP	EAX,LAST_ENTRY		; Highest in our table?
	JC	SHORT EXCPOK		; It's okay
	MOV	EAX,15			; Undefined
EXCPOK:	SHL	EAX,2			; Table has DWORDS
	MOV	ESI,[EAX+EXCP_TAB]	; Get string at that location
	MOV	EDI,OFFSET TRPBUF	; Where to put information string
	MOV	AL,' '			; Get a space
	STOSB				; First in buffer
CPYTXT:	LODSB				; Get byte
	OR	AL,AL			; Check for end
	JZ	SHORT ENDCPY		; End
	STOSB				; Save byte
	JMP	SHORT CPYTXT		; Do the whole string
ENDCPY:	MOV	AL,' '			; Get a space
	STOSB				; Last in buffer
	MOV	ECX,EDI			; Last location
	SUB	ECX,OFFSET TRPMSG	; Starting location
	MOV	EAX,SCR			; This is the screen segment
	MOV	ES,EAX			; Set screen segment
	MOV	EDI,PUT_TEXT		; ES:EDI = destination
	MOV	ESI,OFFSET TRPMSG	; DS:ESI = source
	MOV	AH,17H			; Blue attribute
TR32:	LODSB				; Get byte
	STOSW				; Store word
	LOOP	TR32			; Do whole string
	MOV	ECX,0FFFFH		; Wait just a bit
	LOOP	$
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
;	Okay, now just get us back to real mode after printing the
;	exception number. The amount of extended RAM found will remain.
;	It is probably zero.
;
	POP	EAX			; Return address
	POP	EAX			; Flags
;
	DB	0EAH			; Jump immediate
	DD	0			; Offset (ignored anyway)
	DW	BACK			; Startup segment
;
	MOV	AL,0FEH			; Hit the reset button
	OUT	STATUS_PORT,AL		; Kill the machine
	JMP	$			; If previous didn't work
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
ALIGN	4
BPTR	DD	0			; Return result here
MSTACK	LABEL	DWORD			; SP bumped before use
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
SEG32		ENDS
SUBTTL	End of 32-bit Protected Mode code
INT_PRIVATE	ENDS
PSEG		ENDS
	END	START
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
